@@ -32,6 +32,7 @@ use jj_lib::dsl_util::Diagnostics;
 use jj_lib::fileset::FilePatternParseError;
 use jj_lib::fileset::FilesetParseError;
 use jj_lib::fileset::FilesetParseErrorKind;
+use jj_lib::fix::FixError;
 use jj_lib::gitignore::GitIgnoreError;
 use jj_lib::op_heads_store::OpHeadResolutionError;
 use jj_lib::op_heads_store::OpHeadsStoreError;
@@ -59,7 +60,6 @@ use jj_lib::workspace::WorkspaceInitError;
 use thiserror::Error;
 
 use crate::cli_util::short_operation_hash;
-use crate::config::ConfigEnvError;
 use crate::description_util::ParseBulkEditMessageError;
 use crate::description_util::TempTextEditError;
 use crate::description_util::TextEditError;
@@ -205,6 +205,13 @@ pub fn cli_error(err: impl Into<Box<dyn error::Error + Send + Sync>>) -> Command
     CommandError::new(CommandErrorKind::Cli, err)
 }
 
+pub fn cli_error_with_message(
+    message: impl Into<String>,
+    source: impl Into<Box<dyn error::Error + Send + Sync>>,
+) -> CommandError {
+    CommandError::with_message(CommandErrorKind::Cli, message, source)
+}
+
 pub fn internal_error(err: impl Into<Box<dyn error::Error + Send + Sync>>) -> CommandError {
     CommandError::new(CommandErrorKind::Internal, err)
 }
@@ -242,12 +249,6 @@ impl From<jj_lib::file_util::PathError> for CommandError {
     }
 }
 
-impl From<ConfigEnvError> for CommandError {
-    fn from(err: ConfigEnvError) -> Self {
-        config_error(err)
-    }
-}
-
 impl From<ConfigFileSaveError> for CommandError {
     fn from(err: ConfigFileSaveError) -> Self {
         user_error(err)
@@ -480,23 +481,17 @@ impl From<TempTextEditError> for CommandError {
 
 #[cfg(feature = "git")]
 mod git {
-    use jj_lib::git::GitConfigParseError;
     use jj_lib::git::GitExportError;
     use jj_lib::git::GitFetchError;
     use jj_lib::git::GitFetchPrepareError;
     use jj_lib::git::GitImportError;
     use jj_lib::git::GitPushError;
     use jj_lib::git::GitRemoteManagementError;
+    use jj_lib::git::GitResetHeadError;
     use jj_lib::git::UnexpectedGitBackendError;
 
     use super::*;
 
-    impl From<git2::Error> for CommandError {
-        fn from(err: git2::Error) -> Self {
-            user_error_with_message("Git operation failed", err)
-        }
-    }
-
     impl From<GitImportError> for CommandError {
         fn from(err: GitImportError) -> Self {
             let hint = match &err {
@@ -508,11 +503,8 @@ jj currently does not support partial clones. To use jj with this repository, tr
                      the full repository contents."
                         .to_string(),
                 ),
-                GitImportError::RemoteReservedForLocalGitRepo => {
-                    Some("Run `jj git remote rename` to give different name.".to_string())
-                }
-                GitImportError::InternalBackend(_) => None,
-                GitImportError::InternalGitError(_) => None,
+                GitImportError::Backend(_) => None,
+                GitImportError::Git(_) => None,
                 GitImportError::UnexpectedBackend(_) => None,
             };
             let mut cmd_err =
@@ -524,7 +516,7 @@ jj currently does not support partial clones. To use jj with this repository, tr
 
     impl From<GitExportError> for CommandError {
         fn from(err: GitExportError) -> Self {
-            internal_error_with_message("Failed to export refs to underlying Git repo", err)
+            user_error_with_message("Failed to export refs to underlying Git repo", err)
         }
     }
 
@@ -540,12 +532,13 @@ jj currently does not support partial clones. To use jj with this repository, tr
             }
             match err {
                 GitFetchError::NoSuchRemote(_) => user_error(err),
-                GitFetchError::RemoteWithSlash(_) => user_error_with_hint(
+                GitFetchError::RemoteName(_) => user_error_with_hint(
                     err,
                     "Run `jj git remote rename` to give a different name.",
                 ),
                 GitFetchError::InvalidBranchPattern(_) => user_error(err),
-                GitFetchError::InternalGitError(err) => map_git2_error(err),
+                #[cfg(feature = "git2")]
+                GitFetchError::Git2(err) => map_git2_error(err),
                 GitFetchError::Subprocess(_) => user_error(err),
             }
         }
@@ -554,6 +547,7 @@ jj currently does not support partial clones. To use jj with this repository, tr
     impl From<GitFetchPrepareError> for CommandError {
         fn from(err: GitFetchPrepareError) -> Self {
             match err {
+                #[cfg(feature = "git2")]
                 GitFetchPrepareError::Git2(err) => map_git2_error(err),
                 GitFetchPrepareError::UnexpectedBackend(_) => user_error(err),
             }
@@ -564,22 +558,12 @@ jj currently does not support partial clones. To use jj with this repository, tr
         fn from(err: GitPushError) -> Self {
             match err {
                 GitPushError::NoSuchRemote(_) => user_error(err),
-                GitPushError::RemoteWithSlash(_) => user_error_with_hint(
+                GitPushError::RemoteName(_) => user_error_with_hint(
                     err,
                     "Run `jj git remote rename` to give a different name.",
                 ),
-                GitPushError::RemoteReservedForLocalGitRepo => user_error(err),
-                GitPushError::RefInUnexpectedLocation(refs) => user_error_with_hint(
-                    format!(
-                        "Refusing to push a bookmark that unexpectedly moved on the remote. \
-                         Affected refs: {}",
-                        refs.join(", ")
-                    ),
-                    "Try fetching from the remote, then make the bookmark point to where you want \
-                     it to be, and push again.",
-                ),
-                GitPushError::RefUpdateRejected(_) => user_error(err),
-                GitPushError::InternalGitError(err) => map_git2_error(err),
+                #[cfg(feature = "git2")]
+                GitPushError::Git2(err) => map_git2_error(err),
                 GitPushError::Subprocess(_) => user_error(err),
                 GitPushError::UnexpectedBackend(_) => user_error(err),
             }
@@ -592,9 +576,9 @@ jj currently does not support partial clones. To use jj with this repository, tr
         }
     }
 
-    impl From<GitConfigParseError> for CommandError {
-        fn from(err: GitConfigParseError) -> Self {
-            internal_error_with_message("Failed to parse Git config", err)
+    impl From<GitResetHeadError> for CommandError {
+        fn from(err: GitResetHeadError) -> Self {
+            user_error_with_message("Failed to reset Git HEAD state", err)
         }
     }
 
@@ -604,6 +588,7 @@ jj currently does not support partial clones. To use jj with this repository, tr
         }
     }
 
+    #[cfg(feature = "git2")]
     fn map_git2_error(err: git2::Error) -> CommandError {
         if err.class() == git2::ErrorClass::Ssh {
             let hint = if err.code() == git2::ErrorCode::Certificate
@@ -732,6 +717,20 @@ impl From<AbsorbError> for CommandError {
     }
 }
 
+impl From<FixError> for CommandError {
+    fn from(err: FixError) -> Self {
+        match err {
+            FixError::Backend(err) => err.into(),
+            FixError::RevsetEvaluation(err) => err.into(),
+            FixError::IO(err) => err.into(),
+            FixError::FixContent(err) => internal_error_with_message(
+                "An error occurred while attempting to fix file content",
+                err,
+            ),
+        }
+    }
+}
+
 fn find_source_parse_error_hint(err: &dyn error::Error) -> Option<String> {
     let source = err.source()?;
     if let Some(source) = source.downcast_ref() {
@@ -782,7 +781,10 @@ fn config_get_error_hint(err: &ConfigGetError) -> Option<String> {
 
 fn file_pattern_parse_error_hint(err: &FilePatternParseError) -> Option<String> {
     match err {
-        FilePatternParseError::InvalidKind(_) => None,
+        FilePatternParseError::InvalidKind(_) => Some(String::from(
+            "See https://jj-vcs.github.io/jj/latest/filesets/#file-patterns or `jj help -k \
+             filesets` for valid prefixes.",
+        )),
         // Suggest root:"<path>" if input can be parsed as repo-relative path
         FilePatternParseError::UiPath(UiPathParseError::Fs(e)) => {
             RepoPathBuf::from_relative_path(&e.input).ok().map(|path| {
@@ -877,9 +879,11 @@ fn revset_resolution_error_hint(err: &RevsetResolutionError) -> Option<String> {
 
 fn string_pattern_parse_error_hint(err: &StringPatternParseError) -> Option<String> {
     match err {
-        StringPatternParseError::InvalidKind(_) => {
-            Some("Try prefixing with one of `exact:`, `glob:`, `regex:`, or `substring:`".into())
-        }
+        StringPatternParseError::InvalidKind(_) => Some(
+            "Try prefixing with one of `exact:`, `glob:`, `regex:`, `substring:`, or one of these \
+             with `-i` suffix added (e.g. `glob-i:`) for case-insensitive matching"
+                .into(),
+        ),
         StringPatternParseError::GlobPattern(_) | StringPatternParseError::Regex(_) => None,
     }
 }
@@ -960,7 +964,8 @@ fn print_error(
     Ok(())
 }
 
-fn print_error_sources(ui: &Ui, source: Option<&dyn error::Error>) -> io::Result<()> {
+/// Prints error sources one by one from the given `source` inclusive.
+pub fn print_error_sources(ui: &Ui, source: Option<&dyn error::Error>) -> io::Result<()> {
     let Some(err) = source else {
         return Ok(());
     };
