@@ -43,11 +43,8 @@ use crate::ui::Ui;
 #[command(group(clap::ArgGroup::new("source").multiple(true).required(true)))]
 pub struct BookmarkMoveArgs {
     /// Move bookmarks from the given revisions
-    // We intentionally do not support the short `-f` for `--from` since it
-    // could be confused with a shorthand for `--force`, and people might not
-    // realize they need `-B`/`--allow-backwards` instead.
     #[arg(
-        long,
+        long, short,
         group = "source",
         value_name = "REVSETS",
         add = ArgValueCandidates::new(complete::all_revisions),
@@ -56,11 +53,10 @@ pub struct BookmarkMoveArgs {
 
     // TODO(#5374): Make required in jj 0.32+
     /// Move bookmarks to this revision
-    // We intentionally do not support the short `-t` for `--to` since we don't
-    // support `-f` for `--from`. Currently this defaults to the working copy, but in the near
+    // Currently this defaults to the working copy, but in the near
     // future it will be required to explicitly specify it.
     #[arg(
-        long,
+        long, short,
         value_name = "REVSET",
         add = ArgValueCandidates::new(complete::all_revisions),
     )]
@@ -154,7 +150,10 @@ pub fn cmd_bookmark_move(
             .find(|(_, old_target)| !is_fast_forward(repo.as_ref(), old_target, target_commit.id()))
         {
             return Err(user_error_with_hint(
-                format!("Refusing to move bookmark backwards or sideways: {name}"),
+                format!(
+                    "Refusing to move bookmark backwards or sideways: {name}",
+                    name = name.as_symbol()
+                ),
                 "Use --allow-backwards to allow it.",
             ));
         }
@@ -182,7 +181,10 @@ pub fn cmd_bookmark_move(
         ui,
         format!(
             "point bookmark {names} to commit {id}",
-            names = matched_bookmarks.iter().map(|(name, _)| name).join(", "),
+            names = matched_bookmarks
+                .iter()
+                .map(|(name, _)| name.as_symbol())
+                .join(", "),
             id = target_commit.id().hex()
         ),
     )?;
