@@ -29,7 +29,7 @@ use crate::template_builder::TemplateLanguage as _;
 use crate::templater::TemplatePropertyExt as _;
 use crate::ui::Ui;
 
-/// List variables set in config file, along with their values.
+/// List variables set in config files, along with their values.
 #[derive(clap::Args, Clone, Debug)]
 #[command(mut_group("config_level", |g| g.required(false)))]
 pub struct ConfigListArgs {
@@ -44,7 +44,6 @@ pub struct ConfigListArgs {
     pub include_overridden: bool,
     #[command(flatten)]
     pub level: ConfigLevelArgs,
-    // TODO(#1047): Support --show-origin using StackedConfig.
     /// Render each variable using the given template
     ///
     /// The following keywords are available in the template expression:
@@ -52,6 +51,12 @@ pub struct ConfigListArgs {
     /// * `name: String`: Config name.
     /// * `value: ConfigValue`: Value to be formatted in TOML syntax.
     /// * `overridden: Boolean`: True if the value is shadowed by other.
+    /// * `source: String`: Source of the value.
+    /// * `path: String`: Path to the config file.
+    ///
+    /// Can be overridden by the `templates.config_list` setting. To
+    /// see a detailed config list, use the `builtin_config_list_detailed`
+    /// template.
     ///
     /// See [`jj help -k templates`] for more information.
     ///
@@ -129,6 +134,20 @@ fn config_template_language(
         let out_property = self_property.map(|annotated| annotated.value.decorated("", ""));
         Ok(L::wrap_config_value(out_property))
     });
+    language.add_keyword("source", |self_property| {
+        let out_property = self_property.map(|annotated| annotated.source.to_string());
+        Ok(L::wrap_string(out_property))
+    });
+    language.add_keyword("path", |self_property| {
+        let out_property = self_property.map(|annotated| {
+            // TODO: maybe add FilePath(PathBuf) template type?
+            annotated
+                .path
+                .as_ref()
+                .map_or_else(String::new, |path| path.to_string_lossy().into_owned())
+        });
+        Ok(L::wrap_string(out_property))
+    });
     language.add_keyword("overridden", |self_property| {
         let out_property = self_property.map(|annotated| annotated.is_overridden);
         Ok(L::wrap_boolean(out_property))
