@@ -18,7 +18,7 @@ use std::sync::Arc;
 
 use clap_complete::ArgValueCandidates;
 use indexmap::IndexMap;
-use itertools::Itertools;
+use itertools::Itertools as _;
 use jj_lib::backend::ChangeId;
 use jj_lib::backend::CommitId;
 use jj_lib::commit::Commit;
@@ -29,6 +29,7 @@ use jj_lib::matchers::EverythingMatcher;
 use jj_lib::op_store::RefTarget;
 use jj_lib::op_store::RemoteRef;
 use jj_lib::op_store::RemoteRefState;
+use jj_lib::refs::diff_named_commit_ids;
 use jj_lib::refs::diff_named_ref_targets;
 use jj_lib::refs::diff_named_remote_refs;
 use jj_lib::repo::ReadonlyRepo;
@@ -161,7 +162,7 @@ pub fn cmd_op_diff(
 /// `ReadonlyRepo`s for the operations.
 /// `current_repo` should contain a `Repo` with the indices of both repos merged
 /// into it.
-#[allow(clippy::too_many_arguments)]
+#[expect(clippy::too_many_arguments)]
 pub fn show_op_diff(
     ui: &Ui,
     formatter: &mut dyn Formatter,
@@ -277,6 +278,40 @@ pub fn show_op_diff(
         }
     }
 
+    let changed_working_copies = diff_named_commit_ids(
+        from_repo.view().wc_commit_ids(),
+        to_repo.view().wc_commit_ids(),
+    )
+    .collect_vec();
+    if !changed_working_copies.is_empty() {
+        writeln!(formatter)?;
+        for (name, (from_commit, to_commit)) in changed_working_copies {
+            with_content_format.write(formatter, |formatter| {
+                // Usually, there is at most one working copy changed per operation, so we put
+                // the working copy name in the heading.
+                write!(formatter, "Changed working copy ")?;
+                write!(formatter.labeled("working_copies"), "{}@", name.as_symbol())?;
+                writeln!(formatter, ":")?;
+                write_ref_target_summary(
+                    formatter,
+                    current_repo,
+                    commit_summary_template,
+                    &RefTarget::resolved(to_commit.cloned()),
+                    true,
+                    None,
+                )?;
+                write_ref_target_summary(
+                    formatter,
+                    current_repo,
+                    commit_summary_template,
+                    &RefTarget::resolved(from_commit.cloned()),
+                    false,
+                    None,
+                )
+            })?;
+        }
+    }
+
     let changed_local_bookmarks = diff_named_ref_targets(
         from_repo.view().local_bookmarks(),
         to_repo.view().local_bookmarks(),
@@ -289,7 +324,7 @@ pub fn show_op_diff(
         })?;
         for (name, (from_target, to_target)) in changed_local_bookmarks {
             with_content_format.write(formatter, |formatter| {
-                writeln!(formatter, "{name}:")?;
+                writeln!(formatter, "{name}:", name = name.as_symbol())?;
                 write_ref_target_summary(
                     formatter,
                     current_repo,
@@ -317,7 +352,7 @@ pub fn show_op_diff(
         with_content_format.write(formatter, |formatter| writeln!(formatter, "Changed tags:"))?;
         for (name, (from_target, to_target)) in changed_tags {
             with_content_format.write(formatter, |formatter| {
-                writeln!(formatter, "{name}:")?;
+                writeln!(formatter, "{name}:", name = name.as_symbol())?;
                 write_ref_target_summary(
                     formatter,
                     current_repo,
@@ -354,7 +389,7 @@ pub fn show_op_diff(
         })?;
         let get_remote_ref_prefix = |remote_ref: &RemoteRef| match remote_ref.state {
             RemoteRefState::New => "untracked",
-            RemoteRefState::Tracking => "tracked",
+            RemoteRefState::Tracked => "tracked",
         };
         for (symbol, (from_ref, to_ref)) in changed_remote_bookmarks {
             with_content_format.write(formatter, |formatter| {
