@@ -45,7 +45,8 @@ use jj_lib::merged_tree::MergedTree;
 use jj_lib::object_id::ObjectId as _;
 use jj_lib::op_store::RefTarget;
 use jj_lib::op_store::RemoteRef;
-use jj_lib::op_store::WorkspaceId;
+use jj_lib::ref_name::WorkspaceName;
+use jj_lib::ref_name::WorkspaceNameBuf;
 use jj_lib::repo::Repo;
 use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::repo_path::RepoPathUiConverter;
@@ -102,12 +103,12 @@ pub trait CommitTemplateLanguageExtension {
 pub struct CommitTemplateLanguage<'repo> {
     repo: &'repo dyn Repo,
     path_converter: &'repo RepoPathUiConverter,
-    workspace_id: WorkspaceId,
+    workspace_name: WorkspaceNameBuf,
     // RevsetParseContext doesn't borrow a repo, but we'll need 'repo lifetime
     // anyway to capture it to evaluate dynamically-constructed user expression
     // such as `revset("ancestors(" ++ commit_id ++ ")")`.
-    // TODO: Maybe refactor context structs? RepoPathUiConverter and WorkspaceId
-    // are contained in RevsetParseContext for example.
+    // TODO: Maybe refactor context structs? RepoPathUiConverter and
+    // WorkspaceName are contained in RevsetParseContext for example.
     revset_parse_context: RevsetParseContext<'repo>,
     id_prefix_context: &'repo IdPrefixContext,
     immutable_expression: Rc<UserRevsetExpression>,
@@ -120,11 +121,11 @@ pub struct CommitTemplateLanguage<'repo> {
 impl<'repo> CommitTemplateLanguage<'repo> {
     /// Sets up environment where commit template will be transformed to
     /// evaluation tree.
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     pub fn new(
         repo: &'repo dyn Repo,
         path_converter: &'repo RepoPathUiConverter,
-        workspace_id: &WorkspaceId,
+        workspace_name: &WorkspaceName,
         revset_parse_context: RevsetParseContext<'repo>,
         id_prefix_context: &'repo IdPrefixContext,
         immutable_expression: Rc<UserRevsetExpression>,
@@ -144,7 +145,7 @@ impl<'repo> CommitTemplateLanguage<'repo> {
         CommitTemplateLanguage {
             repo,
             path_converter,
-            workspace_id: workspace_id.clone(),
+            workspace_name: workspace_name.to_owned(),
             revset_parse_context,
             id_prefix_context,
             immutable_expression,
@@ -218,14 +219,14 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                     Self::wrap_commit_list,
                 )
             }
-            CommitTemplatePropertyKind::RefName(property) => {
-                let table = &self.build_fn_table.ref_name_methods;
+            CommitTemplatePropertyKind::CommitRef(property) => {
+                let table = &self.build_fn_table.commit_ref_methods;
                 let build = template_parser::lookup_method(type_name, table, function)?;
                 build(self, diagnostics, build_ctx, property, function)
             }
-            CommitTemplatePropertyKind::RefNameOpt(property) => {
-                let type_name = "RefName";
-                let table = &self.build_fn_table.ref_name_methods;
+            CommitTemplatePropertyKind::CommitRefOpt(property) => {
+                let type_name = "CommitRef";
+                let table = &self.build_fn_table.commit_ref_methods;
                 let build = template_parser::lookup_method(type_name, table, function)?;
                 let inner_property = property.try_unwrap(type_name);
                 build(
@@ -236,7 +237,7 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                     function,
                 )
             }
-            CommitTemplatePropertyKind::RefNameList(property) => {
+            CommitTemplatePropertyKind::CommitRefList(property) => {
                 // TODO: migrate to table?
                 template_builder::build_formattable_list_method(
                     self,
@@ -244,8 +245,8 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                     build_ctx,
                     property,
                     function,
-                    Self::wrap_ref_name,
-                    Self::wrap_ref_name_list,
+                    Self::wrap_commit_ref,
+                    Self::wrap_commit_ref_list,
                 )
             }
             CommitTemplatePropertyKind::RepoPath(property) => {
@@ -341,8 +342,8 @@ impl<'repo> CommitTemplateLanguage<'repo> {
         self.repo
     }
 
-    pub fn workspace_id(&self) -> &WorkspaceId {
-        &self.workspace_id
+    pub fn workspace_name(&self) -> &WorkspaceName {
+        &self.workspace_name
     }
 
     pub fn keyword_cache(&self) -> &CommitKeywordCache<'repo> {
@@ -371,22 +372,22 @@ impl<'repo> CommitTemplateLanguage<'repo> {
         CommitTemplatePropertyKind::CommitList(Box::new(property))
     }
 
-    pub fn wrap_ref_name(
-        property: impl TemplateProperty<Output = Rc<RefName>> + 'repo,
+    pub fn wrap_commit_ref(
+        property: impl TemplateProperty<Output = Rc<CommitRef>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
-        CommitTemplatePropertyKind::RefName(Box::new(property))
+        CommitTemplatePropertyKind::CommitRef(Box::new(property))
     }
 
-    pub fn wrap_ref_name_opt(
-        property: impl TemplateProperty<Output = Option<Rc<RefName>>> + 'repo,
+    pub fn wrap_commit_ref_opt(
+        property: impl TemplateProperty<Output = Option<Rc<CommitRef>>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
-        CommitTemplatePropertyKind::RefNameOpt(Box::new(property))
+        CommitTemplatePropertyKind::CommitRefOpt(Box::new(property))
     }
 
-    pub fn wrap_ref_name_list(
-        property: impl TemplateProperty<Output = Vec<Rc<RefName>>> + 'repo,
+    pub fn wrap_commit_ref_list(
+        property: impl TemplateProperty<Output = Vec<Rc<CommitRef>>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
-        CommitTemplatePropertyKind::RefNameList(Box::new(property))
+        CommitTemplatePropertyKind::CommitRefList(Box::new(property))
     }
 
     pub fn wrap_repo_path(
@@ -461,9 +462,9 @@ pub enum CommitTemplatePropertyKind<'repo> {
     Commit(Box<dyn TemplateProperty<Output = Commit> + 'repo>),
     CommitOpt(Box<dyn TemplateProperty<Output = Option<Commit>> + 'repo>),
     CommitList(Box<dyn TemplateProperty<Output = Vec<Commit>> + 'repo>),
-    RefName(Box<dyn TemplateProperty<Output = Rc<RefName>> + 'repo>),
-    RefNameOpt(Box<dyn TemplateProperty<Output = Option<Rc<RefName>>> + 'repo>),
-    RefNameList(Box<dyn TemplateProperty<Output = Vec<Rc<RefName>>> + 'repo>),
+    CommitRef(Box<dyn TemplateProperty<Output = Rc<CommitRef>> + 'repo>),
+    CommitRefOpt(Box<dyn TemplateProperty<Output = Option<Rc<CommitRef>>> + 'repo>),
+    CommitRefList(Box<dyn TemplateProperty<Output = Vec<Rc<CommitRef>>> + 'repo>),
     RepoPath(Box<dyn TemplateProperty<Output = RepoPathBuf> + 'repo>),
     RepoPathOpt(Box<dyn TemplateProperty<Output = Option<RepoPathBuf>> + 'repo>),
     CommitOrChangeId(Box<dyn TemplateProperty<Output = CommitOrChangeId> + 'repo>),
@@ -486,9 +487,9 @@ impl<'repo> IntoTemplateProperty<'repo> for CommitTemplatePropertyKind<'repo> {
             CommitTemplatePropertyKind::Commit(_) => "Commit",
             CommitTemplatePropertyKind::CommitOpt(_) => "Option<Commit>",
             CommitTemplatePropertyKind::CommitList(_) => "List<Commit>",
-            CommitTemplatePropertyKind::RefName(_) => "RefName",
-            CommitTemplatePropertyKind::RefNameOpt(_) => "Option<RefName>",
-            CommitTemplatePropertyKind::RefNameList(_) => "List<RefName>",
+            CommitTemplatePropertyKind::CommitRef(_) => "CommitRef",
+            CommitTemplatePropertyKind::CommitRefOpt(_) => "Option<CommitRef>",
+            CommitTemplatePropertyKind::CommitRefList(_) => "List<CommitRef>",
             CommitTemplatePropertyKind::RepoPath(_) => "RepoPath",
             CommitTemplatePropertyKind::RepoPathOpt(_) => "Option<RepoPath>",
             CommitTemplatePropertyKind::CommitOrChangeId(_) => "CommitOrChangeId",
@@ -515,11 +516,11 @@ impl<'repo> IntoTemplateProperty<'repo> for CommitTemplatePropertyKind<'repo> {
             CommitTemplatePropertyKind::CommitList(property) => {
                 Some(Box::new(property.map(|l| !l.is_empty())))
             }
-            CommitTemplatePropertyKind::RefName(_) => None,
-            CommitTemplatePropertyKind::RefNameOpt(property) => {
+            CommitTemplatePropertyKind::CommitRef(_) => None,
+            CommitTemplatePropertyKind::CommitRefOpt(property) => {
                 Some(Box::new(property.map(|opt| opt.is_some())))
             }
-            CommitTemplatePropertyKind::RefNameList(property) => {
+            CommitTemplatePropertyKind::CommitRefList(property) => {
                 Some(Box::new(property.map(|l| !l.is_empty())))
             }
             CommitTemplatePropertyKind::RepoPath(_) => None,
@@ -567,9 +568,9 @@ impl<'repo> IntoTemplateProperty<'repo> for CommitTemplatePropertyKind<'repo> {
             CommitTemplatePropertyKind::Commit(_) => None,
             CommitTemplatePropertyKind::CommitOpt(_) => None,
             CommitTemplatePropertyKind::CommitList(_) => None,
-            CommitTemplatePropertyKind::RefName(property) => Some(property.into_template()),
-            CommitTemplatePropertyKind::RefNameOpt(property) => Some(property.into_template()),
-            CommitTemplatePropertyKind::RefNameList(property) => Some(property.into_template()),
+            CommitTemplatePropertyKind::CommitRef(property) => Some(property.into_template()),
+            CommitTemplatePropertyKind::CommitRefOpt(property) => Some(property.into_template()),
+            CommitTemplatePropertyKind::CommitRefList(property) => Some(property.into_template()),
             CommitTemplatePropertyKind::RepoPath(property) => Some(property.into_template()),
             CommitTemplatePropertyKind::RepoPathOpt(property) => Some(property.into_template()),
             CommitTemplatePropertyKind::CommitOrChangeId(property) => {
@@ -597,9 +598,9 @@ impl<'repo> IntoTemplateProperty<'repo> for CommitTemplatePropertyKind<'repo> {
             (CommitTemplatePropertyKind::Commit(_), _) => None,
             (CommitTemplatePropertyKind::CommitOpt(_), _) => None,
             (CommitTemplatePropertyKind::CommitList(_), _) => None,
-            (CommitTemplatePropertyKind::RefName(_), _) => None,
-            (CommitTemplatePropertyKind::RefNameOpt(_), _) => None,
-            (CommitTemplatePropertyKind::RefNameList(_), _) => None,
+            (CommitTemplatePropertyKind::CommitRef(_), _) => None,
+            (CommitTemplatePropertyKind::CommitRefOpt(_), _) => None,
+            (CommitTemplatePropertyKind::CommitRefList(_), _) => None,
             (CommitTemplatePropertyKind::RepoPath(_), _) => None,
             (CommitTemplatePropertyKind::RepoPathOpt(_), _) => None,
             (CommitTemplatePropertyKind::CommitOrChangeId(_), _) => None,
@@ -626,9 +627,9 @@ impl<'repo> IntoTemplateProperty<'repo> for CommitTemplatePropertyKind<'repo> {
             (CommitTemplatePropertyKind::Commit(_), _) => None,
             (CommitTemplatePropertyKind::CommitOpt(_), _) => None,
             (CommitTemplatePropertyKind::CommitList(_), _) => None,
-            (CommitTemplatePropertyKind::RefName(_), _) => None,
-            (CommitTemplatePropertyKind::RefNameOpt(_), _) => None,
-            (CommitTemplatePropertyKind::RefNameList(_), _) => None,
+            (CommitTemplatePropertyKind::CommitRef(_), _) => None,
+            (CommitTemplatePropertyKind::CommitRefOpt(_), _) => None,
+            (CommitTemplatePropertyKind::CommitRefList(_), _) => None,
             (CommitTemplatePropertyKind::RepoPath(_), _) => None,
             (CommitTemplatePropertyKind::RepoPathOpt(_), _) => None,
             (CommitTemplatePropertyKind::CommitOrChangeId(_), _) => None,
@@ -652,7 +653,7 @@ pub type CommitTemplateBuildMethodFnMap<'repo, T> =
 pub struct CommitTemplateBuildFnTable<'repo> {
     pub core: CoreTemplateBuildFnTable<'repo, CommitTemplateLanguage<'repo>>,
     pub commit_methods: CommitTemplateBuildMethodFnMap<'repo, Commit>,
-    pub ref_name_methods: CommitTemplateBuildMethodFnMap<'repo, Rc<RefName>>,
+    pub commit_ref_methods: CommitTemplateBuildMethodFnMap<'repo, Rc<CommitRef>>,
     pub repo_path_methods: CommitTemplateBuildMethodFnMap<'repo, RepoPathBuf>,
     pub commit_or_change_id_methods: CommitTemplateBuildMethodFnMap<'repo, CommitOrChangeId>,
     pub shortest_id_prefix_methods: CommitTemplateBuildMethodFnMap<'repo, ShortestIdPrefix>,
@@ -671,7 +672,7 @@ impl<'repo> CommitTemplateBuildFnTable<'repo> {
         CommitTemplateBuildFnTable {
             core: CoreTemplateBuildFnTable::builtin(),
             commit_methods: builtin_commit_methods(),
-            ref_name_methods: builtin_ref_name_methods(),
+            commit_ref_methods: builtin_commit_ref_methods(),
             repo_path_methods: builtin_repo_path_methods(),
             commit_or_change_id_methods: builtin_commit_or_change_id_methods(),
             shortest_id_prefix_methods: builtin_shortest_id_prefix_methods(),
@@ -688,7 +689,7 @@ impl<'repo> CommitTemplateBuildFnTable<'repo> {
         CommitTemplateBuildFnTable {
             core: CoreTemplateBuildFnTable::empty(),
             commit_methods: HashMap::new(),
-            ref_name_methods: HashMap::new(),
+            commit_ref_methods: HashMap::new(),
             repo_path_methods: HashMap::new(),
             commit_or_change_id_methods: HashMap::new(),
             shortest_id_prefix_methods: HashMap::new(),
@@ -705,7 +706,7 @@ impl<'repo> CommitTemplateBuildFnTable<'repo> {
         let CommitTemplateBuildFnTable {
             core,
             commit_methods,
-            ref_name_methods,
+            commit_ref_methods,
             repo_path_methods,
             commit_or_change_id_methods,
             shortest_id_prefix_methods,
@@ -719,7 +720,7 @@ impl<'repo> CommitTemplateBuildFnTable<'repo> {
 
         self.core.merge(core);
         merge_fn_map(&mut self.commit_methods, commit_methods);
-        merge_fn_map(&mut self.ref_name_methods, ref_name_methods);
+        merge_fn_map(&mut self.commit_ref_methods, commit_ref_methods);
         merge_fn_map(&mut self.repo_path_methods, repo_path_methods);
         merge_fn_map(
             &mut self.commit_or_change_id_methods,
@@ -744,26 +745,26 @@ impl<'repo> CommitTemplateBuildFnTable<'repo> {
 #[derive(Default)]
 pub struct CommitKeywordCache<'repo> {
     // Build index lazily, and Rc to get away from &self lifetime.
-    bookmarks_index: OnceCell<Rc<RefNamesIndex>>,
-    tags_index: OnceCell<Rc<RefNamesIndex>>,
-    git_refs_index: OnceCell<Rc<RefNamesIndex>>,
+    bookmarks_index: OnceCell<Rc<CommitRefsIndex>>,
+    tags_index: OnceCell<Rc<CommitRefsIndex>>,
+    git_refs_index: OnceCell<Rc<CommitRefsIndex>>,
     is_immutable_fn: OnceCell<Rc<RevsetContainingFn<'repo>>>,
 }
 
 impl<'repo> CommitKeywordCache<'repo> {
-    pub fn bookmarks_index(&self, repo: &dyn Repo) -> &Rc<RefNamesIndex> {
+    pub fn bookmarks_index(&self, repo: &dyn Repo) -> &Rc<CommitRefsIndex> {
         self.bookmarks_index
             .get_or_init(|| Rc::new(build_bookmarks_index(repo)))
     }
 
-    pub fn tags_index(&self, repo: &dyn Repo) -> &Rc<RefNamesIndex> {
+    pub fn tags_index(&self, repo: &dyn Repo) -> &Rc<CommitRefsIndex> {
         self.tags_index
-            .get_or_init(|| Rc::new(build_ref_names_index(repo.view().tags())))
+            .get_or_init(|| Rc::new(build_commit_refs_index(repo.view().tags())))
     }
 
-    pub fn git_refs_index(&self, repo: &dyn Repo) -> &Rc<RefNamesIndex> {
+    pub fn git_refs_index(&self, repo: &dyn Repo) -> &Rc<CommitRefsIndex> {
         self.git_refs_index
-            .get_or_init(|| Rc::new(build_ref_names_index(repo.view().git_refs())))
+            .get_or_init(|| Rc::new(build_commit_refs_index(repo.view().git_refs())))
     }
 
     pub fn is_immutable_fn(
@@ -843,7 +844,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         "mine",
         |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
-            let user_email = language.revset_parse_context.user_email().to_owned();
+            let user_email = language.revset_parse_context.user_email.to_owned();
             let out_property = self_property.map(move |commit| commit.author().email == user_email);
             Ok(L::wrap_boolean(out_property))
         },
@@ -870,23 +871,15 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let repo = language.repo;
-            let workspace_id = language.workspace_id.clone();
-            let out_property = self_property.map(move |commit| {
-                Some(commit.id()) == repo.view().get_wc_commit_id(&workspace_id)
-            });
+            let name = language.workspace_name.clone();
+            let out_property = self_property
+                .map(move |commit| Some(commit.id()) == repo.view().get_wc_commit_id(&name));
             Ok(L::wrap_boolean(out_property))
         },
     );
     map.insert(
         "bookmarks",
-        |language, diagnostics, _build_ctx, self_property, function| {
-            if function.name != "bookmarks" {
-                // TODO: Remove in jj 0.28+
-                diagnostics.add_warning(TemplateParseError::expression(
-                    "branches() is deprecated; use bookmarks() instead",
-                    function.name_span,
-                ));
-            }
+        |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let index = language
                 .keyword_cache
@@ -896,23 +889,16 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
                 index
                     .get(commit.id())
                     .iter()
-                    .filter(|ref_name| ref_name.is_local() || !ref_name.synced)
+                    .filter(|commit_ref| commit_ref.is_local() || !commit_ref.synced)
                     .cloned()
                     .collect()
             });
-            Ok(L::wrap_ref_name_list(out_property))
+            Ok(L::wrap_commit_ref_list(out_property))
         },
     );
     map.insert(
         "local_bookmarks",
-        |language, diagnostics, _build_ctx, self_property, function| {
-            if function.name != "local_bookmarks" {
-                // TODO: Remove in jj 0.28+
-                diagnostics.add_warning(TemplateParseError::expression(
-                    "local_branches() is deprecated; use local_bookmarks() instead",
-                    function.name_span,
-                ));
-            }
+        |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let index = language
                 .keyword_cache
@@ -922,23 +908,16 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
                 index
                     .get(commit.id())
                     .iter()
-                    .filter(|ref_name| ref_name.is_local())
+                    .filter(|commit_ref| commit_ref.is_local())
                     .cloned()
                     .collect()
             });
-            Ok(L::wrap_ref_name_list(out_property))
+            Ok(L::wrap_commit_ref_list(out_property))
         },
     );
     map.insert(
         "remote_bookmarks",
-        |language, diagnostics, _build_ctx, self_property, function| {
-            if function.name != "remote_bookmarks" {
-                // TODO: Remove in jj 0.28+
-                diagnostics.add_warning(TemplateParseError::expression(
-                    "remote_branches() is deprecated; use remote_bookmarks() instead",
-                    function.name_span,
-                ));
-            }
+        |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let index = language
                 .keyword_cache
@@ -948,25 +927,20 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
                 index
                     .get(commit.id())
                     .iter()
-                    .filter(|ref_name| ref_name.is_remote())
+                    .filter(|commit_ref| commit_ref.is_remote())
                     .cloned()
                     .collect()
             });
-            Ok(L::wrap_ref_name_list(out_property))
+            Ok(L::wrap_commit_ref_list(out_property))
         },
     );
-    // TODO: Remove the following block after jj 0.28+
-    map.insert("branches", map["bookmarks"]);
-    map.insert("local_branches", map["local_bookmarks"]);
-    map.insert("remote_branches", map["remote_bookmarks"]);
-
     map.insert(
         "tags",
         |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let index = language.keyword_cache.tags_index(language.repo).clone();
             let out_property = self_property.map(move |commit| index.get(commit.id()).to_vec());
-            Ok(L::wrap_ref_name_list(out_property))
+            Ok(L::wrap_commit_ref_list(out_property))
         },
     );
     map.insert(
@@ -975,7 +949,7 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
             function.expect_no_arguments()?;
             let index = language.keyword_cache.git_refs_index(language.repo).clone();
             let out_property = self_property.map(move |commit| index.get(commit.id()).to_vec());
-            Ok(L::wrap_ref_name_list(out_property))
+            Ok(L::wrap_commit_ref_list(out_property))
         },
     );
     map.insert(
@@ -1093,9 +1067,9 @@ fn extract_working_copies(repo: &dyn Repo, commit: &Commit) -> String {
         return "".to_string();
     }
     let mut names = vec![];
-    for (workspace_id, wc_commit_id) in wc_commit_ids.iter().sorted() {
+    for (name, wc_commit_id) in wc_commit_ids {
         if wc_commit_id == commit.id() {
-            names.push(format!("{}@", workspace_id.as_str()));
+            names.push(format!("{}@", name.as_symbol()));
         }
     }
     names.join(" ")
@@ -1128,7 +1102,7 @@ fn evaluate_revset_expression<'repo>(
     let repo = language.repo;
     let symbol_resolver = revset_util::default_symbol_resolver(
         repo,
-        language.revset_parse_context.symbol_resolvers(),
+        language.revset_parse_context.extensions.symbol_resolvers(),
         language.id_prefix_context,
     );
     let revset = expression
@@ -1162,7 +1136,7 @@ fn evaluate_user_revset<'repo>(
 
 /// Bookmark or tag name with metadata.
 #[derive(Debug)]
-pub struct RefName {
+pub struct CommitRef {
     /// Local name.
     name: String,
     /// Remote name if this is a remote or Git-tracking ref.
@@ -1186,8 +1160,8 @@ struct TrackingRef {
     behind_count: OnceCell<SizeHint>,
 }
 
-impl RefName {
-    // RefName is wrapped by Rc<T> to make it cheaply cloned and share
+impl CommitRef {
+    // CommitRef is wrapped by Rc<T> to make it cheaply cloned and share
     // lazy-evaluation results across clones.
 
     /// Creates local ref representation which might track some of the
@@ -1199,8 +1173,8 @@ impl RefName {
     ) -> Rc<Self> {
         let synced = remote_refs
             .into_iter()
-            .all(|remote_ref| !remote_ref.is_tracking() || remote_ref.target == target);
-        Rc::new(RefName {
+            .all(|remote_ref| !remote_ref.is_tracked() || remote_ref.target == target);
+        Rc::new(CommitRef {
             name: name.into(),
             remote: None,
             target,
@@ -1222,8 +1196,8 @@ impl RefName {
         remote_ref: RemoteRef,
         local_target: &RefTarget,
     ) -> Rc<Self> {
-        let synced = remote_ref.is_tracking() && remote_ref.target == *local_target;
-        let tracking_ref = remote_ref.is_tracking().then(|| {
+        let synced = remote_ref.is_tracked() && remote_ref.target == *local_target;
+        let tracking_ref = remote_ref.is_tracked().then(|| {
             let count = if synced {
                 OnceCell::from((0, Some(0))) // fast path for synced remotes
             } else {
@@ -1235,7 +1209,7 @@ impl RefName {
                 behind_count: count,
             }
         });
-        Rc::new(RefName {
+        Rc::new(CommitRef {
             name: name.into(),
             remote: Some(remote_name.into()),
             target: remote_ref.target,
@@ -1250,7 +1224,7 @@ impl RefName {
         remote_name: impl Into<String>,
         target: RefTarget,
     ) -> Rc<Self> {
-        Rc::new(RefName {
+        Rc::new(CommitRef {
             name: name.into(),
             remote: Some(remote_name.into()),
             target,
@@ -1259,32 +1233,55 @@ impl RefName {
         })
     }
 
-    fn is_local(&self) -> bool {
+    /// Local name.
+    pub fn name(&self) -> &str {
+        &self.name
+    }
+
+    /// Remote name if this is a remote or Git-tracking ref.
+    pub fn remote_name(&self) -> Option<&str> {
+        self.remote.as_deref()
+    }
+
+    /// Target commit ids.
+    pub fn target(&self) -> &RefTarget {
+        &self.target
+    }
+
+    /// Returns true if this is a local ref.
+    pub fn is_local(&self) -> bool {
         self.remote.is_none()
     }
 
-    fn is_remote(&self) -> bool {
+    /// Returns true if this is a remote ref.
+    pub fn is_remote(&self) -> bool {
         self.remote.is_some()
     }
 
-    fn is_present(&self) -> bool {
+    /// Returns true if this ref points to no commit.
+    pub fn is_absent(&self) -> bool {
+        self.target.is_absent()
+    }
+
+    /// Returns true if this ref points to any commit.
+    pub fn is_present(&self) -> bool {
         self.target.is_present()
     }
 
     /// Whether the ref target has conflicts.
-    fn has_conflict(&self) -> bool {
+    pub fn has_conflict(&self) -> bool {
         self.target.has_conflict()
     }
 
     /// Returns true if this ref is tracked by a local ref. The local ref might
     /// have been deleted (but not pushed yet.)
-    fn is_tracked(&self) -> bool {
+    pub fn is_tracked(&self) -> bool {
         self.tracking_ref.is_some()
     }
 
     /// Returns true if this ref is tracked by a local ref, and if the local ref
     /// is present.
-    fn is_tracking_present(&self) -> bool {
+    pub fn is_tracking_present(&self) -> bool {
         self.tracking_ref
             .as_ref()
             .is_some_and(|tracking| tracking.target.is_present())
@@ -1322,7 +1319,7 @@ impl RefName {
 }
 
 // If wrapping with Rc<T> becomes common, add generic impl for Rc<T>.
-impl Template for Rc<RefName> {
+impl Template for Rc<CommitRef> {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         write!(formatter.labeled("name"), "{}", self.name)?;
         if let Some(remote) = &self.remote {
@@ -1340,22 +1337,22 @@ impl Template for Rc<RefName> {
     }
 }
 
-impl Template for Vec<Rc<RefName>> {
+impl Template for Vec<Rc<CommitRef>> {
     fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         templater::format_joined(formatter, self, " ")
     }
 }
 
-fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc<RefName>> {
+fn builtin_commit_ref_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc<CommitRef>> {
     type L<'repo> = CommitTemplateLanguage<'repo>;
     // Not using maplit::hashmap!{} or custom declarative macro here because
     // code completion inside macro is quite restricted.
-    let mut map = CommitTemplateBuildMethodFnMap::<Rc<RefName>>::new();
+    let mut map = CommitTemplateBuildMethodFnMap::<Rc<CommitRef>>::new();
     map.insert(
         "name",
         |_language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
-            let out_property = self_property.map(|ref_name| ref_name.name.clone());
+            let out_property = self_property.map(|commit_ref| commit_ref.name.clone());
             Ok(L::wrap_string(out_property))
         },
     );
@@ -1364,7 +1361,7 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         |_language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let out_property =
-                self_property.map(|ref_name| ref_name.remote.clone().unwrap_or_default());
+                self_property.map(|commit_ref| commit_ref.remote.clone().unwrap_or_default());
             Ok(L::wrap_string(out_property))
         },
     );
@@ -1372,7 +1369,7 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         "present",
         |_language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
-            let out_property = self_property.map(|ref_name| ref_name.is_present());
+            let out_property = self_property.map(|commit_ref| commit_ref.is_present());
             Ok(L::wrap_boolean(out_property))
         },
     );
@@ -1380,7 +1377,7 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         "conflict",
         |_language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
-            let out_property = self_property.map(|ref_name| ref_name.has_conflict());
+            let out_property = self_property.map(|commit_ref| commit_ref.has_conflict());
             Ok(L::wrap_boolean(out_property))
         },
     );
@@ -1389,8 +1386,8 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let repo = language.repo;
-            let out_property = self_property.and_then(|ref_name| {
-                let maybe_id = ref_name.target.as_normal();
+            let out_property = self_property.and_then(|commit_ref| {
+                let maybe_id = commit_ref.target.as_normal();
                 Ok(maybe_id.map(|id| repo.store().get_commit(id)).transpose()?)
             });
             Ok(L::wrap_commit_opt(out_property))
@@ -1401,8 +1398,8 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let repo = language.repo;
-            let out_property = self_property.and_then(|ref_name| {
-                let ids = ref_name.target.removed_ids();
+            let out_property = self_property.and_then(|commit_ref| {
+                let ids = commit_ref.target.removed_ids();
                 Ok(ids.map(|id| repo.store().get_commit(id)).try_collect()?)
             });
             Ok(L::wrap_commit_list(out_property))
@@ -1413,8 +1410,8 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         |language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
             let repo = language.repo;
-            let out_property = self_property.and_then(|ref_name| {
-                let ids = ref_name.target.added_ids();
+            let out_property = self_property.and_then(|commit_ref| {
+                let ids = commit_ref.target.added_ids();
                 Ok(ids.map(|id| repo.store().get_commit(id)).try_collect()?)
             });
             Ok(L::wrap_commit_list(out_property))
@@ -1424,7 +1421,7 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         "tracked",
         |_language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
-            let out_property = self_property.map(|ref_name| ref_name.is_tracked());
+            let out_property = self_property.map(|commit_ref| commit_ref.is_tracked());
             Ok(L::wrap_boolean(out_property))
         },
     );
@@ -1432,7 +1429,7 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
         "tracking_present",
         |_language, _diagnostics, _build_ctx, self_property, function| {
             function.expect_no_arguments()?;
-            let out_property = self_property.map(|ref_name| ref_name.is_tracking_present());
+            let out_property = self_property.map(|commit_ref| commit_ref.is_tracking_present());
             Ok(L::wrap_boolean(out_property))
         },
     );
@@ -1442,7 +1439,7 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
             function.expect_no_arguments()?;
             let repo = language.repo;
             let out_property =
-                self_property.and_then(|ref_name| ref_name.tracking_ahead_count(repo));
+                self_property.and_then(|commit_ref| commit_ref.tracking_ahead_count(repo));
             Ok(L::wrap_size_hint(out_property))
         },
     );
@@ -1452,7 +1449,7 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
             function.expect_no_arguments()?;
             let repo = language.repo;
             let out_property =
-                self_property.and_then(|ref_name| ref_name.tracking_behind_count(repo));
+                self_property.and_then(|commit_ref| commit_ref.tracking_behind_count(repo));
             Ok(L::wrap_size_hint(out_property))
         },
     );
@@ -1461,52 +1458,52 @@ fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Rc
 
 /// Cache for reverse lookup refs.
 #[derive(Clone, Debug, Default)]
-pub struct RefNamesIndex {
-    index: HashMap<CommitId, Vec<Rc<RefName>>>,
+pub struct CommitRefsIndex {
+    index: HashMap<CommitId, Vec<Rc<CommitRef>>>,
 }
 
-impl RefNamesIndex {
-    fn insert<'a>(&mut self, ids: impl IntoIterator<Item = &'a CommitId>, name: Rc<RefName>) {
+impl CommitRefsIndex {
+    fn insert<'a>(&mut self, ids: impl IntoIterator<Item = &'a CommitId>, name: Rc<CommitRef>) {
         for id in ids {
-            let ref_names = self.index.entry(id.clone()).or_default();
-            ref_names.push(name.clone());
+            let commit_refs = self.index.entry(id.clone()).or_default();
+            commit_refs.push(name.clone());
         }
     }
 
-    pub fn get(&self, id: &CommitId) -> &[Rc<RefName>] {
-        self.index.get(id).map_or(&[], |names: &Vec<_>| names)
+    pub fn get(&self, id: &CommitId) -> &[Rc<CommitRef>] {
+        self.index.get(id).map_or(&[], |refs: &Vec<_>| refs)
     }
 }
 
-fn build_bookmarks_index(repo: &dyn Repo) -> RefNamesIndex {
-    let mut index = RefNamesIndex::default();
+fn build_bookmarks_index(repo: &dyn Repo) -> CommitRefsIndex {
+    let mut index = CommitRefsIndex::default();
     for (bookmark_name, bookmark_target) in repo.view().bookmarks() {
         let local_target = bookmark_target.local_target;
         let remote_refs = bookmark_target.remote_refs;
         if local_target.is_present() {
-            let ref_name = RefName::local(
+            let commit_ref = CommitRef::local(
                 bookmark_name,
                 local_target.clone(),
                 remote_refs.iter().map(|&(_, remote_ref)| remote_ref),
             );
-            index.insert(local_target.added_ids(), ref_name);
+            index.insert(local_target.added_ids(), commit_ref);
         }
         for &(remote_name, remote_ref) in &remote_refs {
-            let ref_name =
-                RefName::remote(bookmark_name, remote_name, remote_ref.clone(), local_target);
-            index.insert(remote_ref.target.added_ids(), ref_name);
+            let commit_ref =
+                CommitRef::remote(bookmark_name, remote_name, remote_ref.clone(), local_target);
+            index.insert(remote_ref.target.added_ids(), commit_ref);
         }
     }
     index
 }
 
-fn build_ref_names_index<'a>(
-    ref_pairs: impl IntoIterator<Item = (&'a String, &'a RefTarget)>,
-) -> RefNamesIndex {
-    let mut index = RefNamesIndex::default();
+fn build_commit_refs_index<'a, K: Into<String>>(
+    ref_pairs: impl IntoIterator<Item = (K, &'a RefTarget)>,
+) -> CommitRefsIndex {
+    let mut index = CommitRefsIndex::default();
     for (name, target) in ref_pairs {
-        let ref_name = RefName::local_only(name, target.clone());
-        index.insert(target.added_ids(), ref_name);
+        let commit_ref = CommitRef::local_only(name, target.clone());
+        index.insert(target.added_ids(), commit_ref);
     }
     index
 }
