@@ -16,12 +16,13 @@ use std::borrow::Cow;
 use std::collections::BTreeSet;
 use std::collections::HashMap;
 use std::env;
+use std::env::split_paths;
 use std::fmt;
 use std::path::Path;
 use std::path::PathBuf;
 use std::process::Command;
 
-use itertools::Itertools;
+use itertools::Itertools as _;
 use jj_lib::config::ConfigFile;
 use jj_lib::config::ConfigGetError;
 use jj_lib::config::ConfigLayer;
@@ -34,7 +35,6 @@ use jj_lib::config::ConfigValue;
 use jj_lib::config::StackedConfig;
 use regex::Captures;
 use regex::Regex;
-use thiserror::Error;
 use tracing::instrument;
 
 use crate::command_error::config_error;
@@ -49,44 +49,23 @@ pub const CONFIG_SCHEMA: &str = include_str!("config-schema.json");
 pub fn parse_value_or_bare_string(value_str: &str) -> Result<ConfigValue, toml_edit::TomlError> {
     match value_str.parse() {
         Ok(value) => Ok(value),
-        Err(_) if value_str.as_bytes().iter().copied().all(is_bare_char) => Ok(value_str.into()),
+        Err(_) if is_bare_string(value_str) => Ok(value_str.into()),
         Err(err) => Err(err),
     }
 }
 
-const fn is_bare_char(b: u8) -> bool {
-    match b {
-        // control chars (including tabs and newlines), which are unlikely to
-        // appear in command-line arguments
-        b'\x00'..=b'\x1f' | b'\x7f' => false,
-        // space and symbols that don't construct a TOML value
-        b' ' | b'!' | b'#' | b'$' | b'%' | b'&' | b'(' | b')' | b'*' | b'/' | b';' | b'<'
-        | b'>' | b'?' | b'@' | b'\\' | b'^' | b'_' | b'`' | b'|' | b'~' => true,
-        // there may be an error in integer, float, or date-time, but that's rare
-        b'+' | b'-' | b'.' | b':' => true,
-        // comma doesn't construct a compound value by itself, and it might be
-        // used in real name #5233
-        b',' => true,
-        // equal doesn't construct a compound value by itself, but it suggest
-        // that the value is an inline table
-        b'=' => false,
-        // unpaired quotes are often typo
-        b'"' | b'\'' => false,
-        // symbols that construct an inline array or table
-        b'[' | b']' | b'{' | b'}' => false,
-        // ASCII alphanumeric
-        b'0'..=b'9' | b'A'..=b'Z' | b'a'..=b'z' => true,
-        // non-ASCII
-        b'\x80'..=b'\xff' => true,
+fn is_bare_string(value_str: &str) -> bool {
+    // leading whitespace isn't ignored when parsing TOML value expression, but
+    // "\n[]" doesn't look like a bare string.
+    let trimmed = value_str.trim_ascii().as_bytes();
+    if let (Some(&first), Some(&last)) = (trimmed.first(), trimmed.last()) {
+        // string, array, or table constructs?
+        !matches!(first, b'"' | b'\'' | b'[' | b'{') && !matches!(last, b'"' | b'\'' | b']' | b'}')
+    } else {
+        true // empty or whitespace only
     }
 }
 
-#[derive(Error, Debug)]
-pub enum ConfigEnvError {
-    #[error("Both {0} and {1} exist. Please consolidate your configs in one of them.")]
-    AmbiguousSource(PathBuf, PathBuf),
-}
-
 /// Configuration variable with its source information.
 #[derive(Clone, Debug)]
 pub struct AnnotatedValue {
@@ -96,7 +75,8 @@ pub struct AnnotatedValue {
     pub value: ConfigValue,
     /// Source of the configuration value.
     pub source: ConfigSource,
-    // TODO: add source file path
+    /// Path to the source file, if available.
+    pub path: Option<PathBuf>,
     /// True if this value is overridden in higher precedence layers.
     pub is_overridden: bool,
 }
@@ -151,6 +131,7 @@ pub fn resolved_config_values(
                     name,
                     value,
                     source: layer.source,
+                    path: layer.path.clone(),
                     is_overridden,
                 });
             }
@@ -186,23 +167,20 @@ enum ConfigPath {
     /// Could not find any config file, but a new file can be created at the
     /// specified location.
     New(PathBuf),
-    /// Could not find any config file.
-    Unavailable,
 }
 
 impl ConfigPath {
-    fn new(path: Option<PathBuf>) -> Self {
-        match path {
-            Some(path) if path.exists() => ConfigPath::Existing(path),
-            Some(path) => ConfigPath::New(path),
-            None => ConfigPath::Unavailable,
+    fn new(path: PathBuf) -> Self {
+        if path.exists() {
+            ConfigPath::Existing(path)
+        } else {
+            ConfigPath::New(path)
         }
     }
 
-    fn as_path(&self) -> Option<&Path> {
+    fn as_path(&self) -> &Path {
         match self {
-            ConfigPath::Existing(path) | ConfigPath::New(path) => Some(path),
-            ConfigPath::Unavailable => None,
+            ConfigPath::Existing(path) | ConfigPath::New(path) => path,
         }
     }
 }
@@ -214,7 +192,7 @@ fn create_dir_all(path: &Path) -> std::io::Result<()> {
     dir.recursive(true);
     #[cfg(unix)]
     {
-        use std::os::unix::fs::DirBuilderExt;
+        use std::os::unix::fs::DirBuilderExt as _;
         dir.mode(0o700);
     }
     dir.create(path)
@@ -229,31 +207,43 @@ struct UnresolvedConfigEnv {
 }
 
 impl UnresolvedConfigEnv {
-    fn resolve(self) -> Result<ConfigPath, ConfigEnvError> {
-        if let Some(path) = self.jj_config {
-            // TODO: We should probably support colon-separated (std::env::split_paths)
-            return Ok(ConfigPath::new(Some(PathBuf::from(path))));
+    fn resolve(self) -> Vec<ConfigPath> {
+        if let Some(paths) = self.jj_config {
+            return split_paths(&paths)
+                .filter(|path| !path.as_os_str().is_empty())
+                .map(ConfigPath::new)
+                .collect();
         }
-        // TODO: Should we drop the final `/config.toml` and read all files in the
-        // directory?
-        let platform_config_path = ConfigPath::new(self.config_dir.map(|mut config_dir| {
+        let mut paths = vec![];
+        let home_config_path = self.home_dir.map(|mut home_dir| {
+            home_dir.push(".jjconfig.toml");
+            ConfigPath::new(home_dir)
+        });
+        let platform_config_path = self.config_dir.clone().map(|mut config_dir| {
             config_dir.push("jj");
             config_dir.push("config.toml");
-            config_dir
-        }));
-        let home_config_path = ConfigPath::new(self.home_dir.map(|mut home_dir| {
-            home_dir.push(".jjconfig.toml");
-            home_dir
-        }));
+            ConfigPath::new(config_dir)
+        });
+        let platform_config_dir = self.config_dir.map(|mut config_dir| {
+            config_dir.push("jj");
+            config_dir.push("conf.d");
+            ConfigPath::new(config_dir)
+        });
         use ConfigPath::*;
-        match (platform_config_path, home_config_path) {
-            (Existing(platform_config_path), Existing(home_config_path)) => Err(
-                ConfigEnvError::AmbiguousSource(platform_config_path, home_config_path),
-            ),
-            (Existing(path), _) | (_, Existing(path)) => Ok(Existing(path)),
-            (New(path), _) | (_, New(path)) => Ok(New(path)),
-            (Unavailable, Unavailable) => Ok(Unavailable),
+        if let Some(path @ Existing(_)) = home_config_path {
+            paths.push(path);
+        } else if let (Some(path @ New(_)), None) = (home_config_path, &platform_config_path) {
+            paths.push(path);
         }
+        // This should be the default config created if there's
+        // no user config and `jj config edit` is executed.
+        if let Some(path) = platform_config_path {
+            paths.push(path);
+        }
+        if let Some(path @ Existing(_)) = platform_config_dir {
+            paths.push(path);
+        }
+        paths
     }
 }
 
@@ -261,14 +251,14 @@ impl UnresolvedConfigEnv {
 pub struct ConfigEnv {
     home_dir: Option<PathBuf>,
     repo_path: Option<PathBuf>,
-    user_config_path: ConfigPath,
-    repo_config_path: ConfigPath,
+    user_config_paths: Vec<ConfigPath>,
+    repo_config_path: Option<ConfigPath>,
     command: Option<String>,
 }
 
 impl ConfigEnv {
     /// Initializes configuration loader based on environment variables.
-    pub fn from_environment() -> Result<Self, ConfigEnvError> {
+    pub fn from_environment() -> Self {
         // Canonicalize home as we do canonicalize cwd in CliRunner. $HOME might
         // point to symlink.
         let home_dir = dirs::home_dir().map(|path| dunce::canonicalize(&path).unwrap_or(path));
@@ -277,30 +267,31 @@ impl ConfigEnv {
             home_dir: home_dir.clone(),
             jj_config: env::var("JJ_CONFIG").ok(),
         };
-        Ok(ConfigEnv {
+        ConfigEnv {
             home_dir,
             repo_path: None,
-            user_config_path: env.resolve()?,
-            repo_config_path: ConfigPath::Unavailable,
+            user_config_paths: env.resolve(),
+            repo_config_path: None,
             command: None,
-        })
+        }
     }
 
     pub fn set_command_name(&mut self, command: String) {
         self.command = Some(command);
     }
 
-    /// Returns a path to the user-specific config file or directory.
-    pub fn user_config_path(&self) -> Option<&Path> {
-        self.user_config_path.as_path()
+    /// Returns the paths to the user-specific config files or directories.
+    pub fn user_config_paths(&self) -> impl Iterator<Item = &Path> {
+        self.user_config_paths.iter().map(|p| p.as_path())
     }
 
-    /// Returns a path to the existing user-specific config file or directory.
-    fn existing_user_config_path(&self) -> Option<&Path> {
-        match &self.user_config_path {
-            ConfigPath::Existing(path) => Some(path),
+    /// Returns the paths to the existing user-specific config files or
+    /// directories.
+    pub fn existing_user_config_paths(&self) -> impl Iterator<Item = &Path> {
+        self.user_config_paths.iter().filter_map(|p| match p {
+            ConfigPath::Existing(path) => Some(path.as_path()),
             _ => None,
-        }
+        })
     }
 
     /// Returns user configuration files for modification. Instantiates one if
@@ -317,7 +308,8 @@ impl ConfigEnv {
     }
 
     fn new_user_config_file(&self) -> Result<Option<ConfigFile>, ConfigLoadError> {
-        self.user_config_path()
+        self.user_config_paths()
+            .next()
             .map(|path| {
                 // No need to propagate io::Error here. If the directory
                 // couldn't be created, file.save() would fail later.
@@ -336,7 +328,7 @@ impl ConfigEnv {
     #[instrument]
     pub fn reload_user_config(&self, config: &mut RawConfig) -> Result<(), ConfigLoadError> {
         config.as_mut().remove_layers(ConfigSource::User);
-        if let Some(path) = self.existing_user_config_path() {
+        for path in self.existing_user_config_paths() {
             if path.is_dir() {
                 config.as_mut().load_dir(ConfigSource::User, path)?;
             } else {
@@ -350,18 +342,18 @@ impl ConfigEnv {
     /// is usually `.jj/repo`.
     pub fn reset_repo_path(&mut self, path: &Path) {
         self.repo_path = Some(path.to_owned());
-        self.repo_config_path = ConfigPath::new(Some(path.join("config.toml")));
+        self.repo_config_path = Some(ConfigPath::new(path.join("config.toml")));
     }
 
     /// Returns a path to the repo-specific config file.
     pub fn repo_config_path(&self) -> Option<&Path> {
-        self.repo_config_path.as_path()
+        self.repo_config_path.as_ref().map(ConfigPath::as_path)
     }
 
     /// Returns a path to the existing repo-specific config file.
     fn existing_repo_config_path(&self) -> Option<&Path> {
         match &self.repo_config_path {
-            ConfigPath::Existing(path) => Some(path),
+            Some(ConfigPath::Existing(path)) => Some(path),
             _ => None,
         }
     }
@@ -433,7 +425,7 @@ fn config_files_for(
 /// Sources from the lowest precedence:
 /// 1. Default
 /// 2. Base environment variables
-/// 3. [User config](https://jj-vcs.github.io/jj/latest/config/)
+/// 3. [User configs](https://jj-vcs.github.io/jj/latest/config/)
 /// 4. Repo config `.jj/repo/config.toml`
 /// 5. TODO: Workspace config `.jj/config.toml`
 /// 6. Override environment variables
@@ -490,6 +482,7 @@ pub fn default_config_layers() -> Vec<ConfigLayer> {
     let parse = |text: &'static str| ConfigLayer::parse(ConfigSource::Default, text).unwrap();
     let mut layers = vec![
         parse(include_str!("config/colors.toml")),
+        parse(include_str!("config/hints.toml")),
         parse(include_str!("config/merge_tools.toml")),
         parse(include_str!("config/misc.toml")),
         parse(include_str!("config/revsets.toml")),
@@ -603,8 +596,6 @@ pub fn default_config_migrations() -> Vec<ConfigMigrationRule> {
     vec![
         // TODO: Delete in jj 0.32+
         ConfigMigrationRule::rename_value("git.auto-local-branch", "git.auto-local-bookmark"),
-        // TODO: Delete in jj 0.28+
-        ConfigMigrationRule::rename_value("git.push-branch-prefix", "git.push-bookmark-prefix"),
         // TODO: Delete in jj 0.33+
         ConfigMigrationRule::rename_update_value(
             "signing.sign-all",
@@ -620,6 +611,24 @@ pub fn default_config_migrations() -> Vec<ConfigMigrationRule> {
                 }
             },
         ),
+        // TODO: Delete in jj 0.34+
+        ConfigMigrationRule::rename_value(
+            "core.watchman.register_snapshot_trigger",
+            "core.watchman.register-snapshot-trigger",
+        ),
+        // TODO: Delete in jj 0.34+
+        ConfigMigrationRule::rename_value("diff.format", "ui.diff.format"),
+        // TODO: Delete with the `git.subprocess` setting.
+        #[cfg(not(feature = "git2"))]
+        ConfigMigrationRule::custom(
+            |layer| {
+                let Ok(Some(subprocess)) = layer.look_up_item("git.subprocess") else {
+                    return false;
+                };
+                subprocess.as_bool() == Some(false)
+            },
+            |_| Ok("jj was compiled without `git.subprocess = false` support".into()),
+        ),
     ]
 }
 
@@ -761,12 +770,12 @@ impl TryFrom<Vec<String>> for NonEmptyCommandArgsVec {
 
 #[cfg(test)]
 mod tests {
+    use std::env::join_paths;
     use std::fmt::Write as _;
 
-    use anyhow::anyhow;
-    use assert_matches::assert_matches;
     use indoc::indoc;
     use maplit::hashmap;
+    use test_case::test_case;
 
     use super::*;
 
@@ -793,6 +802,7 @@ mod tests {
         assert_eq!(parse("").unwrap().as_str(), Some(""));
         assert_eq!(parse("John Doe").unwrap().as_str(), Some("John Doe"));
         assert_eq!(parse("Doe, John").unwrap().as_str(), Some("Doe, John"));
+        assert_eq!(parse("It's okay").unwrap().as_str(), Some("It's okay"));
         assert_eq!(
             parse("<foo+bar@example.org>").unwrap().as_str(),
             Some("<foo+bar@example.org>")
@@ -804,9 +814,11 @@ mod tests {
 
         // Error in TOML value
         assert!(parse("'foo").is_err());
+        assert!(parse(r#" bar" "#).is_err());
         assert!(parse("[0 1]").is_err());
         assert!(parse("{ x = }").is_err());
-        assert!(parse("key = 'value'").is_err());
+        assert!(parse("\n { x").is_err());
+        assert!(parse(" x ] ").is_err());
         assert!(parse("[table]\nkey = 'value'").is_err());
     }
 
@@ -815,7 +827,6 @@ mod tests {
         assert!(parse_config_arg_item("").is_err());
         assert!(parse_config_arg_item("a").is_err());
         assert!(parse_config_arg_item("=").is_err());
-        assert!(parse_config_arg_item("a=b=c").is_err());
         // The value parser is sensitive to leading whitespaces, which seems
         // good because the parsing falls back to a bare string.
         assert!(parse_config_arg_item("a = 'b'").is_err());
@@ -832,6 +843,11 @@ mod tests {
         assert_eq!(name, ConfigNamePathBuf::from_iter(["a"]));
         assert_eq!(value.as_str(), Some(" "));
 
+        // This one is a bit cryptic, but b=c can be a bare string.
+        let (name, value) = parse_config_arg_item("a=b=c").unwrap();
+        assert_eq!(name, ConfigNamePathBuf::from_iter(["a"]));
+        assert_eq!(value.as_str(), Some("b=c"));
+
         let (name, value) = parse_config_arg_item("a.b=true").unwrap();
         assert_eq!(name, ConfigNamePathBuf::from_iter(["a", "b"]));
         assert_eq!(value.as_bool(), Some(true));
@@ -966,6 +982,7 @@ mod tests {
                     },
                 ),
                 source: EnvBase,
+                path: None,
                 is_overridden: false,
             },
             AnnotatedValue {
@@ -993,6 +1010,7 @@ mod tests {
                     },
                 ),
                 source: EnvBase,
+                path: None,
                 is_overridden: true,
             },
             AnnotatedValue {
@@ -1020,6 +1038,7 @@ mod tests {
                     },
                 ),
                 source: Repo,
+                path: None,
                 is_overridden: false,
             },
         ]
@@ -1068,6 +1087,7 @@ mod tests {
                     },
                 ),
                 source: User,
+                path: None,
                 is_overridden: false,
             },
             AnnotatedValue {
@@ -1095,6 +1115,7 @@ mod tests {
                     },
                 ),
                 source: Repo,
+                path: None,
                 is_overridden: false,
             },
         ]
@@ -1208,232 +1229,339 @@ mod tests {
         insta::assert_snapshot!(list(&layers, "a.b.c"), @r#"!a.b.c.f = "0.1""#);
     }
 
-    #[test]
-    fn test_config_path_home_dir_existing() -> anyhow::Result<()> {
+    struct TestCase {
+        files: &'static [&'static str],
+        env: UnresolvedConfigEnv,
+        wants: &'static [Want],
+    }
+
+    enum Want {
+        New(&'static str),
+        Existing(&'static str),
+    }
+
+    fn config_path_home_existing() -> TestCase {
         TestCase {
-            files: vec!["home/.jjconfig.toml"],
+            files: &["home/.jjconfig.toml"],
             env: UnresolvedConfigEnv {
                 home_dir: Some("home".into()),
                 ..Default::default()
             },
-            want: Want::ExistingAndNew("home/.jjconfig.toml"),
+            wants: &[Want::Existing("home/.jjconfig.toml")],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_home_dir_new() -> anyhow::Result<()> {
+    fn config_path_home_new() -> TestCase {
         TestCase {
-            files: vec![],
+            files: &[],
             env: UnresolvedConfigEnv {
                 home_dir: Some("home".into()),
                 ..Default::default()
             },
-            want: Want::New("home/.jjconfig.toml"),
+            wants: &[Want::New("home/.jjconfig.toml")],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_config_dir_existing() -> anyhow::Result<()> {
+    fn config_path_home_existing_platform_new() -> TestCase {
         TestCase {
-            files: vec!["config/jj/config.toml"],
+            files: &["home/.jjconfig.toml"],
             env: UnresolvedConfigEnv {
-                config_dir: Some("config".into()),
-                ..Default::default()
-            },
-            want: Want::ExistingAndNew("config/jj/config.toml"),
-        }
-        .run()
-    }
-
-    #[test]
-    fn test_config_path_config_dir_new() -> anyhow::Result<()> {
-        TestCase {
-            files: vec![],
-            env: UnresolvedConfigEnv {
-                config_dir: Some("config".into()),
-                ..Default::default()
-            },
-            want: Want::New("config/jj/config.toml"),
-        }
-        .run()
-    }
-
-    #[test]
-    fn test_config_path_new_prefer_config_dir() -> anyhow::Result<()> {
-        TestCase {
-            files: vec![],
-            env: UnresolvedConfigEnv {
-                config_dir: Some("config".into()),
                 home_dir: Some("home".into()),
+                config_dir: Some("config".into()),
                 ..Default::default()
             },
-            want: Want::New("config/jj/config.toml"),
+            wants: &[
+                Want::Existing("home/.jjconfig.toml"),
+                Want::New("config/jj/config.toml"),
+            ],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_jj_config_existing() -> anyhow::Result<()> {
+    fn config_path_platform_existing() -> TestCase {
         TestCase {
-            files: vec!["custom.toml"],
+            files: &["config/jj/config.toml"],
+            env: UnresolvedConfigEnv {
+                home_dir: Some("home".into()),
+                config_dir: Some("config".into()),
+                ..Default::default()
+            },
+            wants: &[Want::Existing("config/jj/config.toml")],
+        }
+    }
+
+    fn config_path_platform_new() -> TestCase {
+        TestCase {
+            files: &[],
+            env: UnresolvedConfigEnv {
+                config_dir: Some("config".into()),
+                ..Default::default()
+            },
+            wants: &[Want::New("config/jj/config.toml")],
+        }
+    }
+
+    fn config_path_new_prefer_platform() -> TestCase {
+        TestCase {
+            files: &[],
+            env: UnresolvedConfigEnv {
+                home_dir: Some("home".into()),
+                config_dir: Some("config".into()),
+                ..Default::default()
+            },
+            wants: &[Want::New("config/jj/config.toml")],
+        }
+    }
+
+    fn config_path_jj_config_existing() -> TestCase {
+        TestCase {
+            files: &["custom.toml"],
             env: UnresolvedConfigEnv {
                 jj_config: Some("custom.toml".into()),
                 ..Default::default()
             },
-            want: Want::ExistingAndNew("custom.toml"),
+            wants: &[Want::Existing("custom.toml")],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_jj_config_new() -> anyhow::Result<()> {
+    fn config_path_jj_config_new() -> TestCase {
         TestCase {
-            files: vec![],
+            files: &[],
             env: UnresolvedConfigEnv {
                 jj_config: Some("custom.toml".into()),
                 ..Default::default()
             },
-            want: Want::New("custom.toml"),
+            wants: &[Want::New("custom.toml")],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_config_pick_config_dir() -> anyhow::Result<()> {
+    fn config_path_jj_config_existing_multiple() -> TestCase {
         TestCase {
-            files: vec!["config/jj/config.toml"],
+            files: &["custom1.toml", "custom2.toml"],
+            env: UnresolvedConfigEnv {
+                jj_config: Some(
+                    join_paths(["custom1.toml", "custom2.toml"])
+                        .unwrap()
+                        .into_string()
+                        .unwrap(),
+                ),
+                ..Default::default()
+            },
+            wants: &[
+                Want::Existing("custom1.toml"),
+                Want::Existing("custom2.toml"),
+            ],
+        }
+    }
+
+    fn config_path_jj_config_new_multiple() -> TestCase {
+        TestCase {
+            files: &["custom1.toml"],
+            env: UnresolvedConfigEnv {
+                jj_config: Some(
+                    join_paths(["custom1.toml", "custom2.toml"])
+                        .unwrap()
+                        .into_string()
+                        .unwrap(),
+                ),
+                ..Default::default()
+            },
+            wants: &[Want::Existing("custom1.toml"), Want::New("custom2.toml")],
+        }
+    }
+
+    fn config_path_jj_config_empty_paths_filtered() -> TestCase {
+        TestCase {
+            files: &["custom1.toml"],
+            env: UnresolvedConfigEnv {
+                jj_config: Some(
+                    join_paths(["custom1.toml", "", "custom2.toml"])
+                        .unwrap()
+                        .into_string()
+                        .unwrap(),
+                ),
+                ..Default::default()
+            },
+            wants: &[Want::Existing("custom1.toml"), Want::New("custom2.toml")],
+        }
+    }
+
+    fn config_path_jj_config_empty() -> TestCase {
+        TestCase {
+            files: &[],
+            env: UnresolvedConfigEnv {
+                jj_config: Some("".to_owned()),
+                ..Default::default()
+            },
+            wants: &[],
+        }
+    }
+
+    fn config_path_config_pick_platform() -> TestCase {
+        TestCase {
+            files: &["config/jj/config.toml"],
             env: UnresolvedConfigEnv {
                 home_dir: Some("home".into()),
                 config_dir: Some("config".into()),
                 ..Default::default()
             },
-            want: Want::ExistingAndNew("config/jj/config.toml"),
+            wants: &[Want::Existing("config/jj/config.toml")],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_config_pick_home_dir() -> anyhow::Result<()> {
+    fn config_path_config_pick_home() -> TestCase {
         TestCase {
-            files: vec!["home/.jjconfig.toml"],
+            files: &["home/.jjconfig.toml"],
             env: UnresolvedConfigEnv {
                 home_dir: Some("home".into()),
                 config_dir: Some("config".into()),
                 ..Default::default()
             },
-            want: Want::ExistingAndNew("home/.jjconfig.toml"),
+            wants: &[
+                Want::Existing("home/.jjconfig.toml"),
+                Want::New("config/jj/config.toml"),
+            ],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_none() -> anyhow::Result<()> {
+    fn config_path_platform_new_conf_dir_existing() -> TestCase {
         TestCase {
-            files: vec![],
+            files: &["config/jj/conf.d/_"],
+            env: UnresolvedConfigEnv {
+                home_dir: Some("home".into()),
+                config_dir: Some("config".into()),
+                ..Default::default()
+            },
+            wants: &[
+                Want::New("config/jj/config.toml"),
+                Want::Existing("config/jj/conf.d"),
+            ],
+        }
+    }
+
+    fn config_path_platform_existing_conf_dir_existing() -> TestCase {
+        TestCase {
+            files: &["config/jj/config.toml", "config/jj/conf.d/_"],
+            env: UnresolvedConfigEnv {
+                home_dir: Some("home".into()),
+                config_dir: Some("config".into()),
+                ..Default::default()
+            },
+            wants: &[
+                Want::Existing("config/jj/config.toml"),
+                Want::Existing("config/jj/conf.d"),
+            ],
+        }
+    }
+
+    fn config_path_all_existing() -> TestCase {
+        TestCase {
+            files: &[
+                "config/jj/conf.d/_",
+                "config/jj/config.toml",
+                "home/.jjconfig.toml",
+            ],
+            env: UnresolvedConfigEnv {
+                home_dir: Some("home".into()),
+                config_dir: Some("config".into()),
+                ..Default::default()
+            },
+            // Precedence order is important
+            wants: &[
+                Want::Existing("home/.jjconfig.toml"),
+                Want::Existing("config/jj/config.toml"),
+                Want::Existing("config/jj/conf.d"),
+            ],
+        }
+    }
+
+    fn config_path_none() -> TestCase {
+        TestCase {
+            files: &[],
             env: Default::default(),
-            want: Want::None,
+            wants: &[],
         }
-        .run()
     }
 
-    #[test]
-    fn test_config_path_ambiguous() -> anyhow::Result<()> {
-        let tmp = setup_config_fs(&vec!["home/.jjconfig.toml", "config/jj/config.toml"])?;
-        let env = UnresolvedConfigEnv {
-            home_dir: Some(tmp.path().join("home")),
-            config_dir: Some(tmp.path().join("config")),
-            ..Default::default()
-        };
-        assert_matches!(env.resolve(), Err(ConfigEnvError::AmbiguousSource(_, _)));
-        Ok(())
+    #[test_case(config_path_home_existing())]
+    #[test_case(config_path_home_new())]
+    #[test_case(config_path_home_existing_platform_new())]
+    #[test_case(config_path_platform_existing())]
+    #[test_case(config_path_platform_new())]
+    #[test_case(config_path_new_prefer_platform())]
+    #[test_case(config_path_jj_config_existing())]
+    #[test_case(config_path_jj_config_new())]
+    #[test_case(config_path_jj_config_existing_multiple())]
+    #[test_case(config_path_jj_config_new_multiple())]
+    #[test_case(config_path_jj_config_empty_paths_filtered())]
+    #[test_case(config_path_jj_config_empty())]
+    #[test_case(config_path_config_pick_platform())]
+    #[test_case(config_path_config_pick_home())]
+    #[test_case(config_path_platform_new_conf_dir_existing())]
+    #[test_case(config_path_platform_existing_conf_dir_existing())]
+    #[test_case(config_path_all_existing())]
+    #[test_case(config_path_none())]
+    fn test_config_path(case: TestCase) {
+        let tmp = setup_config_fs(case.files);
+        let env = resolve_config_env(&case.env, tmp.path());
+
+        let expected_existing: Vec<PathBuf> = case
+            .wants
+            .iter()
+            .filter_map(|want| match want {
+                Want::Existing(path) => Some(tmp.path().join(path)),
+                _ => None,
+            })
+            .collect();
+        assert_eq!(
+            env.existing_user_config_paths().collect_vec(),
+            expected_existing
+        );
+
+        let expected_paths: Vec<PathBuf> = case
+            .wants
+            .iter()
+            .map(|want| match want {
+                Want::New(path) | Want::Existing(path) => tmp.path().join(path),
+            })
+            .collect();
+        assert_eq!(env.user_config_paths().collect_vec(), expected_paths);
     }
 
-    fn setup_config_fs(files: &Vec<&'static str>) -> anyhow::Result<tempfile::TempDir> {
+    fn setup_config_fs(files: &[&str]) -> tempfile::TempDir {
         let tmp = testutils::new_temp_dir();
         for file in files {
             let path = tmp.path().join(file);
             if let Some(parent) = path.parent() {
-                std::fs::create_dir_all(parent)?;
+                std::fs::create_dir_all(parent).unwrap();
             }
-            std::fs::File::create(path)?;
+            std::fs::File::create(path).unwrap();
         }
-        Ok(tmp)
+        tmp
     }
 
-    enum Want {
-        None,
-        New(&'static str),
-        ExistingAndNew(&'static str),
-    }
-
-    struct TestCase {
-        files: Vec<&'static str>,
-        env: UnresolvedConfigEnv,
-        want: Want,
-    }
-
-    impl TestCase {
-        fn resolve(&self, root: &Path) -> Result<ConfigEnv, ConfigEnvError> {
-            let home_dir = self.env.home_dir.as_ref().map(|p| root.join(p));
-            let env = UnresolvedConfigEnv {
-                config_dir: self.env.config_dir.as_ref().map(|p| root.join(p)),
-                home_dir: home_dir.clone(),
-                jj_config: self
-                    .env
-                    .jj_config
-                    .as_ref()
-                    .map(|p| root.join(p).to_str().unwrap().to_string()),
-            };
-            Ok(ConfigEnv {
-                home_dir,
-                repo_path: None,
-                user_config_path: env.resolve()?,
-                repo_config_path: ConfigPath::Unavailable,
-                command: None,
-            })
-        }
-
-        fn run(&self) -> anyhow::Result<()> {
-            let tmp = setup_config_fs(&self.files)?;
-            self.check_existing(&tmp)?;
-            self.check_new(&tmp)?;
-            Ok(())
-        }
-
-        fn check_existing(&self, tmp: &tempfile::TempDir) -> anyhow::Result<()> {
-            let want = match self.want {
-                Want::None => None,
-                Want::New(_) => None,
-                Want::ExistingAndNew(want) => Some(want),
-            }
-            .map(|p| tmp.path().join(p));
-            let env = self
-                .resolve(tmp.path())
-                .map_err(|e| anyhow!("existing_config_path: {e}"))?;
-            let got = env.existing_user_config_path();
-            if got != want.as_deref() {
-                return Err(anyhow!("existing_config_path: got {got:?}, want {want:?}"));
-            }
-            Ok(())
-        }
-
-        fn check_new(&self, tmp: &tempfile::TempDir) -> anyhow::Result<()> {
-            let want = match self.want {
-                Want::None => None,
-                Want::New(want) => Some(want),
-                Want::ExistingAndNew(want) => Some(want),
-            }
-            .map(|p| tmp.path().join(p));
-            let env = self
-                .resolve(tmp.path())
-                .map_err(|e| anyhow!("new_config_path: {e}"))?;
-            let got = env.user_config_path();
-            if got != want.as_deref() {
-                return Err(anyhow!("new_config_path: got {got:?}, want {want:?}"));
-            }
-            Ok(())
+    fn resolve_config_env(env: &UnresolvedConfigEnv, root: &Path) -> ConfigEnv {
+        let home_dir = env.home_dir.as_ref().map(|p| root.join(p));
+        let env = UnresolvedConfigEnv {
+            config_dir: env.config_dir.as_ref().map(|p| root.join(p)),
+            home_dir: home_dir.clone(),
+            jj_config: env.jj_config.as_ref().map(|p| {
+                join_paths(split_paths(p).map(|p| {
+                    if p.as_os_str().is_empty() {
+                        return p;
+                    }
+                    root.join(p)
+                }))
+                .unwrap()
+                .into_string()
+                .unwrap()
+            }),
+        };
+        ConfigEnv {
+            home_dir,
+            repo_path: None,
+            user_config_paths: env.resolve(),
+            repo_config_path: None,
+            command: None,
         }
     }
 }
