@@ -590,33 +590,28 @@ impl Ui {
     }
 
     /// Repeat the given prompt until the input is one of the specified choices.
+    /// Returns the index of the choice.
     pub fn prompt_choice(
         &self,
         prompt: &str,
         choices: &[impl AsRef<str>],
-        default: Option<&str>,
-    ) -> io::Result<String> {
-        if !Self::can_prompt() {
-            if let Some(default) = default {
-                // Choose the default automatically without waiting.
-                writeln!(self.stderr(), "{prompt}: {default}")?;
-                return Ok(default.to_owned());
-            }
-        }
-
-        loop {
-            let choice = self.prompt(prompt)?.trim().to_owned();
-            if choice.is_empty() {
-                if let Some(default) = default {
-                    return Ok(default.to_owned());
-                }
-            }
-            if choices.iter().any(|c| choice == c.as_ref()) {
-                return Ok(choice);
-            }
-
-            writeln!(self.warning_no_heading(), "unrecognized response")?;
-        }
+        default_index: Option<usize>,
+    ) -> io::Result<usize> {
+        self.prompt_choice_with(
+            prompt,
+            default_index.map(|index| {
+                choices
+                    .get(index)
+                    .expect("default_index should be within range")
+                    .as_ref()
+            }),
+            |input| {
+                choices
+                    .iter()
+                    .position(|c| input == c.as_ref())
+                    .ok_or("unrecognized response")
+            },
+        )
     }
 
     /// Prompts for a yes-or-no response, with yes = true and no = false.
@@ -626,14 +621,59 @@ impl Ui {
             Some(false) => "(yN)",
             None => "(yn)",
         };
-        let default_choice = default.map(|c| if c { "Y" } else { "N" });
-
-        let choice = self.prompt_choice(
+        self.prompt_choice_with(
             &format!("{prompt} {default_str}"),
-            &["y", "n", "yes", "no", "Yes", "No", "YES", "NO"],
-            default_choice,
-        )?;
-        Ok(choice.starts_with(['y', 'Y']))
+            default.map(|v| if v { "y" } else { "n" }),
+            |input| {
+                if input.eq_ignore_ascii_case("y") || input.eq_ignore_ascii_case("yes") {
+                    Ok(true)
+                } else if input.eq_ignore_ascii_case("n") || input.eq_ignore_ascii_case("no") {
+                    Ok(false)
+                } else {
+                    Err("unrecognized response")
+                }
+            },
+        )
+    }
+
+    /// Repeats the given prompt until `parse(input)` returns a value.
+    ///
+    /// If the default `text` is given, an empty input will be mapped to it. It
+    /// will also be used in non-interactive session. The default `text` must
+    /// be parsable. If no default is given, this function will fail in
+    /// non-interactive session.
+    pub fn prompt_choice_with<T, E: fmt::Debug + fmt::Display>(
+        &self,
+        prompt: &str,
+        default: Option<&str>,
+        mut parse: impl FnMut(&str) -> Result<T, E>,
+    ) -> io::Result<T> {
+        // Parse the default to ensure that the text is valid.
+        let default = default.map(|text| (parse(text).expect("default should be valid"), text));
+
+        if !Self::can_prompt() {
+            if let Some((value, text)) = default {
+                // Choose the default automatically without waiting.
+                writeln!(self.stderr(), "{prompt}: {text}")?;
+                return Ok(value);
+            }
+        }
+
+        loop {
+            let input = self.prompt(prompt)?;
+            let input = input.trim();
+            if input.is_empty() {
+                if let Some((value, _)) = default {
+                    return Ok(value);
+                } else {
+                    continue;
+                }
+            }
+            match parse(input) {
+                Ok(value) => return Ok(value),
+                Err(err) => writeln!(self.warning_no_heading(), "{err}")?,
+            }
+        }
     }
 
     pub fn prompt_password(&self, prompt: &str) -> io::Result<String> {
@@ -724,7 +764,7 @@ fn duplicate_child_stdin(stdin: &ChildStdin) -> io::Result<std::os::windows::io:
     stdin.as_handle().try_clone_to_owned()
 }
 
-fn format_error_with_sources(err: &dyn error::Error) -> impl fmt::Display + '_ {
+fn format_error_with_sources(err: &dyn error::Error) -> impl fmt::Display + use<'_> {
     iter::successors(Some(err), |&err| err.source()).format(": ")
 }
 
