@@ -22,9 +22,9 @@ use crate::common::TestEnvironment;
 fn test_debug_fileset() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "fileset", "all()"]);
+    let output = work_dir.run_jj(["debug", "fileset", "all()"]);
     assert_snapshot!(output, @r"
     -- Parsed:
     All
@@ -34,7 +34,7 @@ fn test_debug_fileset() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "fileset", "cwd:.."]);
+    let output = work_dir.run_jj(["debug", "fileset", "cwd:.."]);
     assert_snapshot!(output.normalize_backslash(), @r#"
     ------- stderr -------
     Error: Failed to parse fileset: Invalid file pattern
@@ -56,9 +56,9 @@ fn test_debug_fileset() {
 fn test_debug_revset() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "revset", "root()"]);
+    let output = work_dir.run_jj(["debug", "revset", "root()"]);
     insta::with_settings!({filters => vec![
         (r"(?m)(^    .*\n)+", "    ..\n"),
     ]}, {
@@ -93,8 +93,8 @@ fn test_debug_revset() {
 fn test_debug_index() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "index"]);
+    let work_dir = test_env.work_dir("repo");
+    let output = work_dir.run_jj(["debug", "index"]);
     assert_snapshot!(filter_index_stats(output), @r"
     Number of commits: 2
     Number of merges: 0
@@ -113,10 +113,10 @@ fn test_debug_index() {
 fn test_debug_reindex() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    test_env.run_jj_in(&workspace_path, ["new"]).success();
-    test_env.run_jj_in(&workspace_path, ["new"]).success();
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "index"]);
+    let work_dir = test_env.work_dir("repo");
+    work_dir.run_jj(["new"]).success();
+    work_dir.run_jj(["new"]).success();
+    let output = work_dir.run_jj(["debug", "index"]);
     assert_snapshot!(filter_index_stats(output), @r"
     Number of commits: 4
     Number of merges: 0
@@ -132,13 +132,13 @@ fn test_debug_reindex() {
         Name: [hash]
     [EOF]
     ");
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "reindex"]);
+    let output = work_dir.run_jj(["debug", "reindex"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Finished indexing 4 commits.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "index"]);
+    let output = work_dir.run_jj(["debug", "index"]);
     assert_snapshot!(filter_index_stats(output), @r"
     Number of commits: 4
     Number of merges: 0
@@ -157,15 +157,14 @@ fn test_debug_reindex() {
 fn test_debug_tree() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    let subdir = workspace_path.join("dir").join("subdir");
-    std::fs::create_dir_all(&subdir).unwrap();
-    std::fs::write(subdir.join("file1"), "contents 1").unwrap();
-    test_env.run_jj_in(&workspace_path, ["new"]).success();
-    std::fs::write(subdir.join("file2"), "contents 2").unwrap();
+    let work_dir = test_env.work_dir("repo");
+    let sub_dir = work_dir.create_dir_all("dir/subdir");
+    sub_dir.write_file("file1", "contents 1");
+    work_dir.run_jj(["new"]).success();
+    sub_dir.write_file("file2", "contents 2");
 
     // Defaults to showing the tree at the current commit
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "tree"]);
+    let output = work_dir.run_jj(["debug", "tree"]);
     assert_snapshot!(output.normalize_backslash(), @r#"
     dir/subdir/file1: Ok(Resolved(Some(File { id: FileId("498e9b01d79cb8d31cdf0df1a663cc1fcefd9de3"), executable: false })))
     dir/subdir/file2: Ok(Resolved(Some(File { id: FileId("b2496eaffe394cd50a9db4de5787f45f09fd9722"), executable: false })))
@@ -174,7 +173,7 @@ fn test_debug_tree() {
     );
 
     // Can show the tree at another commit
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "tree", "-r@-"]);
+    let output = work_dir.run_jj(["debug", "tree", "-r@-"]);
     assert_snapshot!(output.normalize_backslash(), @r#"
     dir/subdir/file1: Ok(Resolved(Some(File { id: FileId("498e9b01d79cb8d31cdf0df1a663cc1fcefd9de3"), executable: false })))
     [EOF]
@@ -182,7 +181,7 @@ fn test_debug_tree() {
     );
 
     // Can filter by paths
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "tree", "dir/subdir/file2"]);
+    let output = work_dir.run_jj(["debug", "tree", "dir/subdir/file2"]);
     assert_snapshot!(output.normalize_backslash(), @r#"
     dir/subdir/file2: Ok(Resolved(Some(File { id: FileId("b2496eaffe394cd50a9db4de5787f45f09fd9722"), executable: false })))
     [EOF]
@@ -190,14 +189,11 @@ fn test_debug_tree() {
     );
 
     // Can a show the root tree by id
-    let output = test_env.run_jj_in(
-        &workspace_path,
-        [
-            "debug",
-            "tree",
-            "--id=0958358e3f80e794f032b25ed2be96cf5825da6c",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "debug",
+        "tree",
+        "--id=0958358e3f80e794f032b25ed2be96cf5825da6c",
+    ]);
     assert_snapshot!(output.normalize_backslash(), @r#"
     dir/subdir/file1: Ok(Resolved(Some(File { id: FileId("498e9b01d79cb8d31cdf0df1a663cc1fcefd9de3"), executable: false })))
     dir/subdir/file2: Ok(Resolved(Some(File { id: FileId("b2496eaffe394cd50a9db4de5787f45f09fd9722"), executable: false })))
@@ -206,15 +202,12 @@ fn test_debug_tree() {
     );
 
     // Can a show non-root tree by id
-    let output = test_env.run_jj_in(
-        &workspace_path,
-        [
-            "debug",
-            "tree",
-            "--dir=dir",
-            "--id=6ac232efa713535ae518a1a898b77e76c0478184",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "debug",
+        "tree",
+        "--dir=dir",
+        "--id=6ac232efa713535ae518a1a898b77e76c0478184",
+    ]);
     assert_snapshot!(output.normalize_backslash(), @r#"
     dir/subdir/file1: Ok(Resolved(Some(File { id: FileId("498e9b01d79cb8d31cdf0df1a663cc1fcefd9de3"), executable: false })))
     dir/subdir/file2: Ok(Resolved(Some(File { id: FileId("b2496eaffe394cd50a9db4de5787f45f09fd9722"), executable: false })))
@@ -223,16 +216,13 @@ fn test_debug_tree() {
     );
 
     // Can filter by paths when showing non-root tree (matcher applies from root)
-    let output = test_env.run_jj_in(
-        &workspace_path,
-        [
-            "debug",
-            "tree",
-            "--dir=dir",
-            "--id=6ac232efa713535ae518a1a898b77e76c0478184",
-            "dir/subdir/file2",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "debug",
+        "tree",
+        "--dir=dir",
+        "--id=6ac232efa713535ae518a1a898b77e76c0478184",
+        "dir/subdir/file2",
+    ]);
     assert_snapshot!(output.normalize_backslash(), @r#"
     dir/subdir/file2: Ok(Resolved(Some(File { id: FileId("b2496eaffe394cd50a9db4de5787f45f09fd9722"), executable: false })))
     [EOF]
@@ -244,8 +234,8 @@ fn test_debug_tree() {
 fn test_debug_operation_id() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let workspace_path = test_env.env_root().join("repo");
-    let output = test_env.run_jj_in(&workspace_path, ["debug", "operation", "--display", "id"]);
+    let work_dir = test_env.work_dir("repo");
+    let output = work_dir.run_jj(["debug", "operation", "--display", "id"]);
     assert_snapshot!(filter_index_stats(output), @r"
     eac759b9ab75793fd3da96e60939fb48f2cd2b2a9c1f13ffe723cf620f3005b8d3e7e923634a07ea39513e4f2f360c87b9ad5d331cf90d7a844864b83b72eba1
     [EOF]
