@@ -19,18 +19,17 @@ use crate::common::TestEnvironment;
 #[test]
 fn test_diffedit() {
     let mut test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-
-    std::fs::write(repo_path.join("file1"), "a\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::write(repo_path.join("file2"), "a\n").unwrap();
-    std::fs::write(repo_path.join("file3"), "a\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::remove_file(repo_path.join("file1")).unwrap();
-    std::fs::write(repo_path.join("file2"), "b\n").unwrap();
-
     let edit_script = test_env.set_up_fake_diff_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "a\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.write_file("file2", "a\n");
+    work_dir.write_file("file3", "a\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.remove_file("file1");
+    work_dir.write_file("file2", "b\n");
 
     // Test the setup; nothing happens if we make no changes
     std::fs::write(
@@ -43,22 +42,22 @@ fn test_diffedit() {
         .join("\0"),
     )
     .unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit"]);
+    let output = work_dir.run_jj(["diffedit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("instrs")).unwrap(), @r###"
+        std::fs::read_to_string(test_env.env_root().join("instrs")).unwrap(), @r"
     You are editing changes in: kkmpptxz 3d4cce89 (no description set)
 
     The diff initially shows the commit's changes.
 
     Adjust the right side until it shows the contents you want. If you
     don't make any changes, then the operation will be aborted.
-    "###);
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    ");
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -67,16 +66,13 @@ fn test_diffedit() {
 
     // Try again with ui.diff-instructions=false
     std::fs::write(&edit_script, "files-before file1 file2\0files-after file2").unwrap();
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["diffedit", "--config=ui.diff-instructions=false"],
-    );
+    let output = work_dir.run_jj(["diffedit", "--config=ui.diff-instructions=false"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -89,20 +85,17 @@ fn test_diffedit() {
         "files-before file1 file2\0files-after JJ-INSTRUCTIONS file2",
     )
     .unwrap();
-    let output = test_env.run_jj_in(
-        &repo_path,
-        [
-            "diffedit",
-            "--config=ui.diff-editor='false'",
-            "--tool=fake-diff-editor",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "diffedit",
+        "--config=ui.diff-editor='false'",
+        "--tool=fake-diff-editor",
+    ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -111,7 +104,7 @@ fn test_diffedit() {
 
     // Nothing happens if the diff-editor exits with an error
     std::fs::write(&edit_script, "rm file2\0fail").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit"]);
+    let output = work_dir.run_jj(["diffedit"]);
     insta::assert_snapshot!(output.normalize_stderr_exit_status(), @r"
     ------- stderr -------
     Error: Failed to edit diff
@@ -119,7 +112,7 @@ fn test_diffedit() {
     [EOF]
     [exit status: 1]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -128,56 +121,54 @@ fn test_diffedit() {
 
     // Can edit changes to individual files
     std::fs::write(&edit_script, "reset file2").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit"]);
+    let output = work_dir.run_jj(["diffedit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created kkmpptxz cbc7a725 (no description set)
-    Working copy now at: kkmpptxz cbc7a725 (no description set)
-    Parent commit      : rlvkpnrz a72506cd (no description set)
+    Working copy  (@) now at: kkmpptxz cbc7a725 (no description set)
+    Parent commit (@-)      : rlvkpnrz a72506cd (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     [EOF]
     ");
 
     // Changes to a commit are propagated to descendants
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
+    work_dir.run_jj(["undo"]).success();
     std::fs::write(&edit_script, "write file3\nmodified\n").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "-r", "@-"]);
+    let output = work_dir.run_jj(["diffedit", "-r", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created rlvkpnrz d4eef3fc (no description set)
     Rebased 1 descendant commits
-    Working copy now at: kkmpptxz 59ef1b95 (no description set)
-    Parent commit      : rlvkpnrz d4eef3fc (no description set)
+    Working copy  (@) now at: kkmpptxz 59ef1b95 (no description set)
+    Parent commit (@-)      : rlvkpnrz d4eef3fc (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
-    let contents = String::from_utf8(std::fs::read(repo_path.join("file3")).unwrap()).unwrap();
-    insta::assert_snapshot!(contents, @r###"
-    modified
-    "###);
+    let contents = work_dir.read_file("file3");
+    insta::assert_snapshot!(contents, @"modified");
 
     // Test diffedit --from @--
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
+    work_dir.run_jj(["undo"]).success();
     std::fs::write(
         &edit_script,
         "files-before file1\0files-after JJ-INSTRUCTIONS file2 file3\0reset file2",
     )
     .unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "--from", "@--"]);
+    let output = work_dir.run_jj(["diffedit", "--from", "@--"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created kkmpptxz 5b585bd1 (no description set)
-    Working copy now at: kkmpptxz 5b585bd1 (no description set)
-    Parent commit      : rlvkpnrz a72506cd (no description set)
+    Working copy  (@) now at: kkmpptxz 5b585bd1 (no description set)
+    Parent commit (@-)      : rlvkpnrz a72506cd (no description set)
     Added 0 files, modified 0 files, removed 1 files
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     D file2
@@ -188,15 +179,14 @@ fn test_diffedit() {
 #[test]
 fn test_diffedit_new_file() {
     let mut test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-
-    std::fs::write(repo_path.join("file1"), "a\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::remove_file(repo_path.join("file1")).unwrap();
-    std::fs::write(repo_path.join("file2"), "b\n").unwrap();
-
     let edit_script = test_env.set_up_fake_diff_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "a\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.remove_file("file1");
+    work_dir.write_file("file2", "b\n");
 
     // Test the setup; nothing happens if we make no changes
     std::fs::write(
@@ -204,13 +194,13 @@ fn test_diffedit_new_file() {
         "files-before file1\0files-after JJ-INSTRUCTIONS file2",
     )
     .unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit"]);
+    let output = work_dir.run_jj(["diffedit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     A file2
@@ -219,16 +209,16 @@ fn test_diffedit_new_file() {
 
     // Creating `file1` on the right side is noticed by `jj diffedit`
     std::fs::write(&edit_script, "write file1\nmodified\n").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit"]);
+    let output = work_dir.run_jj(["diffedit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created rlvkpnrz b0376e2b (no description set)
-    Working copy now at: rlvkpnrz b0376e2b (no description set)
-    Parent commit      : qpvuntsm b739eb46 (no description set)
+    Working copy  (@) now at: rlvkpnrz b0376e2b (no description set)
+    Parent commit (@-)      : qpvuntsm b739eb46 (no description set)
     Added 1 files, modified 0 files, removed 0 files
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     M file1
     A file2
@@ -241,15 +231,15 @@ fn test_diffedit_new_file() {
     // On one hand, it is unexpected and potentially a minor BUG. On the other
     // hand, this prevents `jj` from loading any backup files the merge tool
     // generates.
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
+    work_dir.run_jj(["undo"]).success();
     std::fs::write(&edit_script, "write new_file\nnew file\n").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit"]);
+    let output = work_dir.run_jj(["diffedit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     A file2
@@ -260,86 +250,75 @@ fn test_diffedit_new_file() {
 #[test]
 fn test_diffedit_external_tool_conflict_marker_style() {
     let mut test_env = TestEnvironment::default();
+    let edit_script = test_env.set_up_fake_diff_editor();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    let file_path = repo_path.join("file");
+    let work_dir = test_env.work_dir("repo");
+    let file_path = "file";
 
     // Create a conflict
-    std::fs::write(
-        &file_path,
+    work_dir.write_file(
+        file_path,
         indoc! {"
-        line 1
-        line 2
-        line 3
-        line 4
-        line 5
-    "},
-    )
-    .unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["commit", "-m", "base"])
-        .success();
-    std::fs::write(
-        &file_path,
+            line 1
+            line 2
+            line 3
+            line 4
+            line 5
+        "},
+    );
+    work_dir.run_jj(["commit", "-m", "base"]).success();
+    work_dir.write_file(
+        file_path,
         indoc! {"
-        line 1
-        line 2.1
-        line 2.2
-        line 3
-        line 4.1
-        line 5
-    "},
-    )
-    .unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "side-a"])
+            line 1
+            line 2.1
+            line 2.2
+            line 3
+            line 4.1
+            line 5
+        "},
+    );
+    work_dir.run_jj(["describe", "-m", "side-a"]).success();
+    work_dir
+        .run_jj(["new", "description(base)", "-m", "side-b"])
         .success();
-    test_env
-        .run_jj_in(&repo_path, ["new", "description(base)", "-m", "side-b"])
-        .success();
-    std::fs::write(
-        &file_path,
+    work_dir.write_file(
+        file_path,
         indoc! {"
-        line 1
-        line 2.3
-        line 3
-        line 4.2
-        line 4.3
-        line 5
-    "},
-    )
-    .unwrap();
+            line 1
+            line 2.3
+            line 3
+            line 4.2
+            line 4.3
+            line 5
+        "},
+    );
 
     // Resolve one of the conflicts in the working copy
-    test_env
-        .run_jj_in(
-            &repo_path,
-            ["new", "description(side-a)", "description(side-b)"],
-        )
+    work_dir
+        .run_jj(["new", "description(side-a)", "description(side-b)"])
         .success();
-    std::fs::write(
-        &file_path,
+    work_dir.write_file(
+        file_path,
         indoc! {"
-        line 1
-        line 2.1
-        line 2.2
-        line 2.3
-        line 3
-        <<<<<<<
-        %%%%%%%
-        -line 4
-        +line 4.1
-        +++++++
-        line 4.2
-        line 4.3
-        >>>>>>>
-        line 5
-    "},
-    )
-    .unwrap();
+            line 1
+            line 2.1
+            line 2.2
+            line 2.3
+            line 3
+            <<<<<<<
+            %%%%%%%
+            -line 4
+            +line 4.1
+            +++++++
+            line 4.2
+            line 4.3
+            >>>>>>>
+            line 5
+        "},
+    );
 
     // Set up diff editor to use "snapshot" conflict markers
-    let edit_script = test_env.set_up_fake_diff_editor();
     test_env.add_config(r#"merge-tools.fake-diff-editor.conflict-marker-style = "snapshot""#);
 
     // We want to see whether the diff editor is using the correct conflict markers,
@@ -356,23 +335,22 @@ fn test_diffedit_external_tool_conflict_marker_style() {
         .join("\0"),
     )
     .unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit"]);
+    let output = work_dir.run_jj(["diffedit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created mzvwutvl fb39e804 (conflict) (empty) (no description set)
-    Working copy now at: mzvwutvl fb39e804 (conflict) (empty) (no description set)
-    Parent commit      : rlvkpnrz 3765cc27 side-a
-    Parent commit      : zsuskuln 8b3de837 side-b
+    Working copy  (@) now at: mzvwutvl fb39e804 (conflict) (empty) (no description set)
+    Parent commit (@-)      : rlvkpnrz 3765cc27 side-a
+    Parent commit (@-)      : zsuskuln 8b3de837 side-b
     Added 0 files, modified 1 files, removed 0 files
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
-    Existing conflicts were resolved or abandoned from these commits:
-      mzvwutvl hidden a813239f (conflict) (no description set)
+    Existing conflicts were resolved or abandoned from 1 commits.
     [EOF]
     ");
     // Conflicts should render using "snapshot" format in diff editor
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("before-file")).unwrap(), @r##"
+        std::fs::read_to_string(test_env.env_root().join("before-file")).unwrap(), @r"
     line 1
     <<<<<<< Conflict 1 of 2
     +++++++ Contents of side #1
@@ -394,9 +372,9 @@ fn test_diffedit_external_tool_conflict_marker_style() {
     line 4.3
     >>>>>>> Conflict 2 of 2 ends
     line 5
-    "##);
+    ");
     insta::assert_snapshot!(
-        std::fs::read_to_string(test_env.env_root().join("after-file")).unwrap(), @r##"
+        std::fs::read_to_string(test_env.env_root().join("after-file")).unwrap(), @r"
     line 1
     line 2.1
     line 2.2
@@ -412,10 +390,9 @@ fn test_diffedit_external_tool_conflict_marker_style() {
     line 4.3
     >>>>>>> Conflict 1 of 1 ends
     line 5
-    "##);
+    ");
     // Conflicts should be materialized using "diff" format in working copy
-    insta::assert_snapshot!(
-        std::fs::read_to_string(&file_path).unwrap(), @r##"
+    insta::assert_snapshot!(work_dir.read_file(file_path), @r"
     line 1
     <<<<<<< Conflict 1 of 2
     +++++++ Contents of side #1
@@ -435,15 +412,15 @@ fn test_diffedit_external_tool_conflict_marker_style() {
     line 4.3
     >>>>>>> Conflict 2 of 2 ends
     line 5
-    "##);
+    ");
 
     // File should be conflicted with no changes
-    let output = test_env.run_jj_in(&repo_path, ["st"]);
+    let output = work_dir.run_jj(["st"]);
     insta::assert_snapshot!(output, @r"
     The working copy has no changes.
-    Working copy : mzvwutvl fb39e804 (conflict) (empty) (no description set)
-    Parent commit: rlvkpnrz 3765cc27 side-a
-    Parent commit: zsuskuln 8b3de837 side-b
+    Working copy  (@) : mzvwutvl fb39e804 (conflict) (empty) (no description set)
+    Parent commit (@-): rlvkpnrz 3765cc27 side-a
+    Parent commit (@-): zsuskuln 8b3de837 side-b
     Warning: There are unresolved conflicts at these paths:
     file    2-sided conflict
     [EOF]
@@ -453,16 +430,17 @@ fn test_diffedit_external_tool_conflict_marker_style() {
 #[test]
 fn test_diffedit_3pane() {
     let mut test_env = TestEnvironment::default();
+    let edit_script = test_env.set_up_fake_diff_editor();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    std::fs::write(repo_path.join("file1"), "a\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::write(repo_path.join("file2"), "a\n").unwrap();
-    std::fs::write(repo_path.join("file3"), "a\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::remove_file(repo_path.join("file1")).unwrap();
-    std::fs::write(repo_path.join("file2"), "b\n").unwrap();
+    work_dir.write_file("file1", "a\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.write_file("file2", "a\n");
+    work_dir.write_file("file3", "a\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.remove_file("file1");
+    work_dir.write_file("file2", "b\n");
 
     // 2 configs for a 3-pane setup. In the first, "$right" is passed to what the
     // fake diff editor considers the "after" state.
@@ -470,7 +448,6 @@ fn test_diffedit_3pane() {
         "merge-tools.fake-diff-editor.edit-args=['$left', '$right', '--ignore=$output']";
     let config_with_output_as_after =
         "merge-tools.fake-diff-editor.edit-args=['$left', '$output', '--ignore=$right']";
-    let edit_script = test_env.set_up_fake_diff_editor();
     std::fs::write(&edit_script, "").unwrap();
 
     // Nothing happens if we make no changes
@@ -479,32 +456,26 @@ fn test_diffedit_3pane() {
         "files-before file1 file2\0files-after JJ-INSTRUCTIONS file2",
     )
     .unwrap();
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["diffedit", "--config", config_with_output_as_after],
-    );
+    let output = work_dir.run_jj(["diffedit", "--config", config_with_output_as_after]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
     [EOF]
     ");
     // Nothing happens if we make no changes, `config_with_right_as_after` version
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["diffedit", "--config", config_with_right_as_after],
-    );
+    let output = work_dir.run_jj(["diffedit", "--config", config_with_right_as_after]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -513,40 +484,34 @@ fn test_diffedit_3pane() {
 
     // Can edit changes to individual files
     std::fs::write(&edit_script, "reset file2").unwrap();
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["diffedit", "--config", config_with_output_as_after],
-    );
+    let output = work_dir.run_jj(["diffedit", "--config", config_with_output_as_after]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created kkmpptxz ed8aada3 (no description set)
-    Working copy now at: kkmpptxz ed8aada3 (no description set)
-    Parent commit      : rlvkpnrz a72506cd (no description set)
+    Working copy  (@) now at: kkmpptxz ed8aada3 (no description set)
+    Parent commit (@-)      : rlvkpnrz a72506cd (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     [EOF]
     ");
 
     // Can write something new to `file1`
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
+    work_dir.run_jj(["undo"]).success();
     std::fs::write(&edit_script, "write file1\nnew content").unwrap();
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["diffedit", "--config", config_with_output_as_after],
-    );
+    let output = work_dir.run_jj(["diffedit", "--config", config_with_output_as_after]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created kkmpptxz 7c19e689 (no description set)
-    Working copy now at: kkmpptxz 7c19e689 (no description set)
-    Parent commit      : rlvkpnrz a72506cd (no description set)
+    Working copy  (@) now at: kkmpptxz 7c19e689 (no description set)
+    Parent commit (@-)      : rlvkpnrz a72506cd (no description set)
     Added 1 files, modified 0 files, removed 0 files
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     M file1
     M file2
@@ -554,18 +519,15 @@ fn test_diffedit_3pane() {
     ");
 
     // But nothing happens if we modify the right side
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
+    work_dir.run_jj(["undo"]).success();
     std::fs::write(&edit_script, "write file1\nnew content").unwrap();
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["diffedit", "--config", config_with_right_as_after],
-    );
+    let output = work_dir.run_jj(["diffedit", "--config", config_with_right_as_after]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -579,64 +541,61 @@ fn test_diffedit_3pane() {
 #[test]
 fn test_diffedit_merge() {
     let mut test_env = TestEnvironment::default();
+    let edit_script = test_env.set_up_fake_diff_editor();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    std::fs::write(repo_path.join("file1"), "a\n").unwrap();
-    std::fs::write(repo_path.join("file2"), "a\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    test_env
-        .run_jj_in(&repo_path, ["bookmark", "create", "-r@", "b"])
-        .success();
-    std::fs::write(repo_path.join("file1"), "b\n").unwrap();
-    std::fs::write(repo_path.join("file2"), "b\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new", "@-"]).success();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::write(repo_path.join("file1"), "c\n").unwrap();
-    std::fs::write(repo_path.join("file2"), "c\n").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["new", "@", "b", "-m", "merge"])
+    work_dir.write_file("file1", "a\n");
+    work_dir.write_file("file2", "a\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "b"])
         .success();
+    work_dir.write_file("file1", "b\n");
+    work_dir.write_file("file2", "b\n");
+    work_dir.run_jj(["new", "@-"]).success();
+    work_dir.run_jj(["new"]).success();
+    work_dir.write_file("file1", "c\n");
+    work_dir.write_file("file2", "c\n");
+    work_dir.run_jj(["new", "@", "b", "-m", "merge"]).success();
     // Resolve the conflict in file1, but leave the conflict in file2
-    std::fs::write(repo_path.join("file1"), "d\n").unwrap();
-    std::fs::write(repo_path.join("file3"), "d\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
+    work_dir.write_file("file1", "d\n");
+    work_dir.write_file("file3", "d\n");
+    work_dir.run_jj(["new"]).success();
     // Test the setup
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-r", "@-", "-s"]);
+    let output = work_dir.run_jj(["diff", "-r", "@-", "-s"]);
     insta::assert_snapshot!(output, @r"
     M file1
     A file3
     [EOF]
     ");
 
-    let edit_script = test_env.set_up_fake_diff_editor();
-
     // Remove file1. The conflict remains in the working copy on top of the merge.
     std::fs::write(
         edit_script,
         "files-before file1\0files-after JJ-INSTRUCTIONS file1 file3\0rm file1",
     )
     .unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "-r", "@-"]);
+    let output = work_dir.run_jj(["diffedit", "-r", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created royxmykx 0105de4a (conflict) merge
     Rebased 1 descendant commits
-    Working copy now at: yqosqzyt abbb78c1 (conflict) (empty) (no description set)
-    Parent commit      : royxmykx 0105de4a (conflict) merge
+    Working copy  (@) now at: yqosqzyt abbb78c1 (conflict) (empty) (no description set)
+    Parent commit (@-)      : royxmykx 0105de4a (conflict) merge
     Added 0 files, modified 0 files, removed 1 files
     Warning: There are unresolved conflicts at these paths:
     file2    2-sided conflict
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s", "-r", "@-"]);
+    let output = work_dir.run_jj(["diff", "-s", "-r", "@-"]);
     insta::assert_snapshot!(output, @r"
     D file1
     A file3
     [EOF]
     ");
-    assert!(!repo_path.join("file1").exists());
-    let output = test_env.run_jj_in(&repo_path, ["file", "show", "file2"]);
+    assert!(!work_dir.root().join("file1").exists());
+    let output = work_dir.run_jj(["file", "show", "file2"]);
     insta::assert_snapshot!(output, @r"
     <<<<<<< Conflict 1 of 1
     %%%%%%% Changes from base to side #1
@@ -652,26 +611,25 @@ fn test_diffedit_merge() {
 #[test]
 fn test_diffedit_old_restore_interactive_tests() {
     let mut test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-
-    std::fs::write(repo_path.join("file1"), "a\n").unwrap();
-    std::fs::write(repo_path.join("file2"), "a\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::remove_file(repo_path.join("file1")).unwrap();
-    std::fs::write(repo_path.join("file2"), "b\n").unwrap();
-    std::fs::write(repo_path.join("file3"), "b\n").unwrap();
-
     let edit_script = test_env.set_up_fake_diff_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file1", "a\n");
+    work_dir.write_file("file2", "a\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.remove_file("file1");
+    work_dir.write_file("file2", "b\n");
+    work_dir.write_file("file3", "b\n");
 
     // Nothing happens if we make no changes
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "--from", "@-"]);
+    let output = work_dir.run_jj(["diffedit", "--from", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Nothing changed.
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -681,7 +639,7 @@ fn test_diffedit_old_restore_interactive_tests() {
 
     // Nothing happens if the diff-editor exits with an error
     std::fs::write(&edit_script, "rm file2\0fail").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "--from", "@-"]);
+    let output = work_dir.run_jj(["diffedit", "--from", "@-"]);
     insta::assert_snapshot!(output.normalize_stderr_exit_status(), @r"
     ------- stderr -------
     Error: Failed to edit diff
@@ -689,7 +647,7 @@ fn test_diffedit_old_restore_interactive_tests() {
     [EOF]
     [exit status: 1]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     M file2
@@ -699,34 +657,34 @@ fn test_diffedit_old_restore_interactive_tests() {
 
     // Can restore changes to individual files
     std::fs::write(&edit_script, "reset file2\0reset file3").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "--from", "@-"]);
+    let output = work_dir.run_jj(["diffedit", "--from", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created rlvkpnrz 69811eda (no description set)
-    Working copy now at: rlvkpnrz 69811eda (no description set)
-    Parent commit      : qpvuntsm fc687cb8 (no description set)
+    Working copy  (@) now at: rlvkpnrz 69811eda (no description set)
+    Parent commit (@-)      : qpvuntsm fc687cb8 (no description set)
     Added 0 files, modified 1 files, removed 1 files
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "-s"]);
+    let output = work_dir.run_jj(["diff", "-s"]);
     insta::assert_snapshot!(output, @r"
     D file1
     [EOF]
     ");
 
     // Can make unrelated edits
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
+    work_dir.run_jj(["undo"]).success();
     std::fs::write(&edit_script, "write file3\nunrelated\n").unwrap();
-    let output = test_env.run_jj_in(&repo_path, ["diffedit", "--from", "@-"]);
+    let output = work_dir.run_jj(["diffedit", "--from", "@-"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created rlvkpnrz 2b76a42e (no description set)
-    Working copy now at: rlvkpnrz 2b76a42e (no description set)
-    Parent commit      : qpvuntsm fc687cb8 (no description set)
+    Working copy  (@) now at: rlvkpnrz 2b76a42e (no description set)
+    Parent commit (@-)      : qpvuntsm fc687cb8 (no description set)
     Added 0 files, modified 1 files, removed 0 files
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "--git"]);
+    let output = work_dir.run_jj(["diff", "--git"]);
     insta::assert_snapshot!(output, @r"
     diff --git a/file1 b/file1
     deleted file mode 100644
@@ -756,32 +714,28 @@ fn test_diffedit_old_restore_interactive_tests() {
 #[test]
 fn test_diffedit_restore_descendants() {
     let mut test_env = TestEnvironment::default();
-    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-
-    std::fs::write(repo_path.join("file"), "println!(\"foo\")\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::write(repo_path.join("file"), "println!(\"bar\")\n").unwrap();
-    test_env.run_jj_in(&repo_path, ["new"]).success();
-    std::fs::write(repo_path.join("file"), "println!(\"baz\");\n").unwrap();
-
     let edit_script = test_env.set_up_fake_diff_editor();
+    test_env.run_jj_in(".", ["git", "init", "repo"]).success();
+    let work_dir = test_env.work_dir("repo");
+
+    work_dir.write_file("file", "println!(\"foo\")\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.write_file("file", "println!(\"bar\")\n");
+    work_dir.run_jj(["new"]).success();
+    work_dir.write_file("file", "println!(\"baz\");\n");
 
     // Add a ";" after the line with "bar". There should be no conflict.
     std::fs::write(edit_script, "write file\nprintln!(\"bar\");\n").unwrap();
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["diffedit", "-r", "@-", "--restore-descendants"],
-    );
+    let output = work_dir.run_jj(["diffedit", "-r", "@-", "--restore-descendants"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created rlvkpnrz 62b8c2ce (no description set)
     Rebased 1 descendant commits (while preserving their content)
-    Working copy now at: kkmpptxz 321d1cd1 (no description set)
-    Parent commit      : rlvkpnrz 62b8c2ce (no description set)
+    Working copy  (@) now at: kkmpptxz 321d1cd1 (no description set)
+    Parent commit (@-)      : rlvkpnrz 62b8c2ce (no description set)
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["diff", "--git"]);
+    let output = work_dir.run_jj(["diff", "--git"]);
     insta::assert_snapshot!(output, @r#"
     diff --git a/file b/file
     index 1a598a8fc9..7b6a85ab5a 100644
