@@ -100,10 +100,7 @@ fn clone_git_remote_into(
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_with_default_config(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "origin");
@@ -120,10 +117,7 @@ fn test_git_fetch_with_default_config(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_default_remote(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -142,10 +136,7 @@ fn test_git_fetch_default_remote(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_single_remote(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -172,10 +163,7 @@ fn test_git_fetch_single_remote(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_single_remote_all_remotes_flag(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -194,10 +182,7 @@ fn test_git_fetch_single_remote_all_remotes_flag(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_single_remote_from_arg(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -218,10 +203,7 @@ fn test_git_fetch_single_remote_from_arg(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_single_remote_from_config(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -241,10 +223,7 @@ fn test_git_fetch_single_remote_from_config(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_multiple_remotes(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -268,10 +247,7 @@ fn test_git_fetch_multiple_remotes(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_with_glob(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -291,10 +267,7 @@ fn test_git_fetch_with_glob(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_with_glob_and_exact_match(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -318,10 +291,7 @@ fn test_git_fetch_with_glob_and_exact_match(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_with_glob_from_config(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config(r#"git.fetch = "glob:rem*""#);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -343,10 +313,7 @@ fn test_git_fetch_with_glob_from_config(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_with_glob_with_no_matching_remotes(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "upstream");
@@ -369,10 +336,7 @@ fn test_git_fetch_with_glob_with_no_matching_remotes(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_all_remotes(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -402,10 +366,7 @@ fn test_git_fetch_all_remotes(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_multiple_remotes_from_config(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -428,10 +389,7 @@ fn test_git_fetch_multiple_remotes_from_config(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_nonexistent_remote(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -454,10 +412,7 @@ fn test_git_fetch_nonexistent_remote(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_nonexistent_remote_from_config(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
     add_git_remote(&test_env, &work_dir, "rem1");
@@ -481,10 +436,7 @@ fn test_git_fetch_nonexistent_remote_from_config(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_from_remote_named_git(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     let work_dir = test_env.work_dir("repo");
     init_git_remote(&test_env, "git");
@@ -563,10 +515,7 @@ fn test_git_fetch_from_remote_named_git(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_from_remote_with_slashes(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     let work_dir = test_env.work_dir("repo");
     init_git_remote(&test_env, "source");
@@ -593,10 +542,7 @@ fn test_git_fetch_from_remote_with_slashes(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_prune_before_updating_tips(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -636,10 +582,7 @@ fn test_git_fetch_prune_before_updating_tips(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_conflicting_bookmarks(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -675,10 +618,7 @@ fn test_git_fetch_conflicting_bookmarks(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_conflicting_bookmarks_colocated(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     let work_dir = test_env.work_dir("repo");
     git::init(work_dir.root());
@@ -754,10 +694,7 @@ fn create_trunk2_and_rebase_bookmarks(work_dir: &TestWorkDir) -> String {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_all(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_dir = test_env.work_dir("source");
@@ -942,10 +879,7 @@ fn test_git_fetch_all(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_some_of_many_bookmarks(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let source_dir = test_env.work_dir("source");
@@ -1209,10 +1143,7 @@ fn test_git_fetch_some_of_many_bookmarks(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_bookmarks_some_missing(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -1357,10 +1288,7 @@ fn test_git_fetch_bookmarks_missing_with_subprocess_localized_message() {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_undo(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
@@ -1456,10 +1384,7 @@ fn test_git_fetch_undo(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_fetch_undo_what(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
@@ -1583,10 +1508,7 @@ fn test_fetch_undo_what(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_remove_fetch(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -1652,10 +1574,7 @@ fn test_git_fetch_remove_fetch(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_rename_fetch(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
@@ -1709,10 +1628,7 @@ fn test_git_fetch_rename_fetch(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_removed_bookmark(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
@@ -1827,10 +1743,7 @@ fn test_git_fetch_removed_bookmark(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_removed_parent_bookmark(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     let source_dir = test_env.work_dir("source");
     git::init(source_dir.root());
@@ -1927,10 +1840,7 @@ fn test_git_fetch_removed_parent_bookmark(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_remote_only_bookmark(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
 
@@ -1999,10 +1909,7 @@ fn test_git_fetch_remote_only_bookmark(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_fetch_preserve_commits_across_repos(subprocess: bool) {
-    let test_env = TestEnvironment::default();
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     test_env.add_config("git.auto-local-bookmark = true");
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
     let work_dir = test_env.work_dir("repo");
