@@ -64,12 +64,9 @@ fn set_up(test_env: &TestEnvironment) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_nothing(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     // Show the setup. `insta` has trouble if this is done inside `set_up()`
     insta::allow_duplicates! {
     insta::assert_snapshot!(get_bookmark_output(&work_dir), @r"
@@ -94,12 +91,9 @@ fn test_git_push_nothing(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_current_bookmark(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     // Update some bookmarks. `bookmark1` is not a current bookmark, but
     // `bookmark2` and `my-bookmark` are.
@@ -195,12 +189,9 @@ fn test_git_push_current_bookmark(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_parent_bookmark(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     work_dir.run_jj(["edit", "bookmark1"]).success();
     work_dir
@@ -224,12 +215,9 @@ fn test_git_push_parent_bookmark(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_no_matching_bookmark(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir.run_jj(["new"]).success();
     let output = work_dir.run_jj(["git", "push"]);
     insta::allow_duplicates! {
@@ -245,12 +233,9 @@ fn test_git_push_no_matching_bookmark(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_matching_bookmark_unchanged(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir.run_jj(["new", "bookmark1"]).success();
     let output = work_dir.run_jj(["git", "push"]);
     insta::allow_duplicates! {
@@ -269,12 +254,9 @@ fn test_git_push_matching_bookmark_unchanged(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_other_remote_has_bookmark(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     // Create another remote (but actually the same)
     let other_remote_path = test_env
@@ -337,12 +319,9 @@ fn test_git_push_other_remote_has_bookmark(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_forward_unexpectedly_moved(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     // Move bookmark1 forward on the remote
     let origin_dir = test_env.work_dir("origin");
@@ -394,12 +373,9 @@ fn test_git_push_forward_unexpectedly_moved(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_sideways_unexpectedly_moved(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     // Move bookmark1 forward on the remote
     let origin_dir = test_env.work_dir("origin");
@@ -470,12 +446,9 @@ fn test_git_push_sideways_unexpectedly_moved(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_deletion_unexpectedly_moved(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     // Move bookmark1 forward on the remote
     let origin_dir = test_env.work_dir("origin");
@@ -542,12 +515,9 @@ fn test_git_push_deletion_unexpectedly_moved(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_unexpectedly_deleted(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     // Delete bookmark1 forward on the remote
     let origin_dir = test_env.work_dir("origin");
@@ -653,12 +623,9 @@ fn test_git_push_unexpectedly_deleted(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_creation_unexpectedly_already_exists(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     // Forget bookmark1 locally
     work_dir
@@ -713,12 +680,9 @@ fn test_git_push_creation_unexpectedly_already_exists(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_locally_created_and_rewritten(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     // Ensure that remote bookmarks aren't tracked automatically
     test_env.add_config("git.auto-local-bookmark = false");
 
@@ -786,12 +750,9 @@ fn test_git_push_locally_created_and_rewritten(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_multiple(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir
         .run_jj(["bookmark", "delete", "bookmark1"])
         .success();
@@ -901,6 +862,7 @@ fn test_git_push_multiple(subprocess: bool) {
 
     // --deleted is required to push deleted bookmarks even with --all
     let output = work_dir.run_jj(["git", "push", "--all", "--dry-run"]);
+    insta::allow_duplicates! {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Warning: Refusing to push deleted bookmark bookmark1
@@ -911,7 +873,9 @@ fn test_git_push_multiple(subprocess: bool) {
     Dry-run requested, not pushing.
     [EOF]
     ");
+    }
     let output = work_dir.run_jj(["git", "push", "--all", "--deleted", "--dry-run"]);
+    insta::allow_duplicates! {
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Changes to push to origin:
@@ -921,6 +885,7 @@ fn test_git_push_multiple(subprocess: bool) {
     Dry-run requested, not pushing.
     [EOF]
     ");
+    }
 
     let output = work_dir.run_jj(["git", "push", "--all", "--deleted"]);
     insta::allow_duplicates! {
@@ -960,12 +925,9 @@ fn test_git_push_multiple(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_changes(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir.run_jj(["describe", "-m", "foo"]).success();
     work_dir.write_file("file", "contents");
     work_dir.run_jj(["new", "-m", "bar"]).success();
@@ -1094,12 +1056,9 @@ fn test_git_push_changes(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_changes_with_name(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if subprocess {
-        test_env.add_config("git.subprocess = true");
-    }
     work_dir.run_jj(["describe", "-m", "foo"]).success();
     work_dir.write_file("file", "contents");
     work_dir.run_jj(["new", "-m", "pushed"]).success();
@@ -1230,15 +1189,12 @@ fn test_git_push_changes_with_name(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_changes_with_name_deleted_tracked(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     // Unset immutable_heads so that untracking branches does not move the working
     // copy
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
     let work_dir = test_env.work_dir("local");
-    if subprocess {
-        test_env.add_config("git.subprocess = true");
-    }
     // Create a second empty remote `another_remote`
     test_env
         .run_jj_in(".", ["git", "init", "another_remote"])
@@ -1347,18 +1303,14 @@ fn test_git_push_changes_with_name_deleted_tracked(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_changes_with_name_untracked_or_forgotten(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if subprocess {
-        test_env.add_config("git.subprocess = true");
-    }
     // Unset immutable_heads so that untracking branches does not move the working
     // copy
     test_env.add_config(r#"revset-aliases."immutable_heads()" = "none()""#);
-    work_dir
-        .run_jj(["describe", "-m", "pushed_to_remote"])
-        .success();
+    work_dir.run_jj(["describe", "-m", "parent"]).success();
+    work_dir.run_jj(["new", "-m", "pushed_to_remote"]).success();
     work_dir.write_file("file", "contents");
     work_dir
         .run_jj(["new", "-m", "child", "--no-edit"])
@@ -1383,8 +1335,9 @@ fn test_git_push_changes_with_name_untracked_or_forgotten(subprocess: bool) {
         .success();
     insta::allow_duplicates! {
     insta::assert_snapshot!(output, @r"
-    ○  c9c child
-    @  10b b1@origin pushed_to_remote
+    ○  08f child
+    @  ec9 b1@origin pushed_to_remote
+    ○  57e parent
     ◆  000
     [EOF]
     ");
@@ -1394,7 +1347,7 @@ fn test_git_push_changes_with_name_untracked_or_forgotten(subprocess: bool) {
         .success();
     insta::allow_duplicates! {
     insta::assert_snapshot!(output, @r"
-    b1@origin: yqosqzyt 10b6b209 pushed_to_remote
+    b1@origin: yostqsxw ec992a1a pushed_to_remote
     [EOF]
     ");
     }
@@ -1435,19 +1388,32 @@ fn test_git_push_changes_with_name_untracked_or_forgotten(subprocess: bool) {
 
     // Make sure push still errors if we try to push a bookmark with the same name
     // to a different location.
-    let output = work_dir.run_jj(["git", "push", "--named", "b1=@+"]);
-    insta::allow_duplicates! {
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Changes to push to origin:
-      Add bookmark b1 to c9c824c88955
-    Error: Failed to push some bookmarks
-    Hint: The following references unexpectedly moved on the remote:
-      refs/heads/b1 (reason: stale info)
-    Hint: Try fetching from the remote, then make the bookmark point to where you want it to be, and push again.
-    [EOF]
-    [exit status: 1]
-    ");
+    let output = work_dir.run_jj(["git", "push", "--named", "b1=@-"]);
+    if subprocess {
+        insta::assert_snapshot!(output, @r"
+        ------- stderr -------
+        Changes to push to origin:
+          Add bookmark b1 to 57ec90f54125
+        Error: Failed to push some bookmarks
+        Hint: The following references unexpectedly moved on the remote:
+          refs/heads/b1 (reason: stale info)
+        Hint: Try fetching from the remote, then make the bookmark point to where you want it to be, and push again.
+        [EOF]
+        [exit status: 1]
+        ");
+    } else {
+        // For libgit2, the error is the same but missing the "reason: stale info"
+        insta::assert_snapshot!(output, @r"
+        ------- stderr -------
+        Changes to push to origin:
+          Add bookmark b1 to 57ec90f54125
+        Error: Failed to push some bookmarks
+        Hint: The following references unexpectedly moved on the remote:
+          refs/heads/b1
+        Hint: Try fetching from the remote, then make the bookmark point to where you want it to be, and push again.
+        [EOF]
+        [exit status: 1]
+        ");
     }
 
     // The bookmark is still forgotten
@@ -1455,29 +1421,52 @@ fn test_git_push_changes_with_name_untracked_or_forgotten(subprocess: bool) {
     insta::allow_duplicates! {
     insta::assert_snapshot!(output, @"");
     }
-    // In this case, pushing the bookmark to the same location where it already is
-    // succeeds. TODO: This seems pretty safe, but perhaps it should still show
-    // an error or some sort of warning?
-    let output = work_dir.run_jj(["git", "push", "--named", "b1=@"]);
-    insta::allow_duplicates! {
-    insta::assert_snapshot!(output, @r"
-    ------- stderr -------
-    Changes to push to origin:
-      Add bookmark b1 to 10b6b209c4a3
-    [EOF]
-    ");
+    // This command behaves differently. In libgit2 flow, a conflict between a
+    // revision and its descendant is resolved in favor of the latter, like in `jj
+    // git fetch`. In subprocess flow, this logic is not implemented. As far as `jj
+    // git push --named` goes, the stricter behavior of the subprocess flow is
+    // probably slightly better.
+    let output = work_dir.run_jj(["git", "push", "--named", "b1=@+"]);
+    if subprocess {
+        insta::assert_snapshot!(output, @r"
+        ------- stderr -------
+        Changes to push to origin:
+          Add bookmark b1 to 08fcdf4055ae
+        Error: Failed to push some bookmarks
+        Hint: The following references unexpectedly moved on the remote:
+          refs/heads/b1 (reason: stale info)
+        Hint: Try fetching from the remote, then make the bookmark point to where you want it to be, and push again.
+        [EOF]
+        [exit status: 1]
+        ");
+        // In this case, pushing the bookmark to the same location where it already is
+        // succeeds. TODO: This seems pretty safe, but perhaps it should still show
+        // an error or some sort of warning?
+        let output = work_dir.run_jj(["git", "push", "--named", "b1=@"]);
+        insta::allow_duplicates! {
+        insta::assert_snapshot!(output, @r"
+        ------- stderr -------
+        Changes to push to origin:
+          Add bookmark b1 to ec992a1a9381
+        [EOF]
+        ");
+        }
+    } else {
+        insta::assert_snapshot!(output, @r"
+        ------- stderr -------
+        Changes to push to origin:
+          Add bookmark b1 to 08fcdf4055ae
+        [EOF]
+        ");
     }
 }
 
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_revisions(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir.run_jj(["describe", "-m", "foo"]).success();
     work_dir.write_file("file", "contents");
     work_dir.run_jj(["new", "-m", "bar"]).success();
@@ -1565,12 +1554,9 @@ fn test_git_push_revisions(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_mixed(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir.run_jj(["describe", "-m", "foo"]).success();
     work_dir.write_file("file", "contents");
     work_dir.run_jj(["new", "-m", "bar"]).success();
@@ -1631,12 +1617,9 @@ fn test_git_push_mixed(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_unsnapshotted_change(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir.run_jj(["describe", "-m", "foo"]).success();
     work_dir.write_file("file", "contents");
     work_dir.run_jj(["git", "push", "--change", "@"]).success();
@@ -1647,12 +1630,9 @@ fn test_git_push_unsnapshotted_change(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_conflict(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir.write_file("file", "first");
     work_dir.run_jj(["commit", "-m", "first"]).success();
     work_dir.write_file("file", "second");
@@ -1680,12 +1660,9 @@ fn test_git_push_conflict(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_no_description(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir
         .run_jj(["bookmark", "create", "-r@", "my-bookmark"])
         .success();
@@ -1715,12 +1692,9 @@ fn test_git_push_no_description(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_no_description_in_immutable(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir
         .run_jj(["bookmark", "create", "-r@", "imm"])
         .success();
@@ -1770,12 +1744,9 @@ fn test_git_push_no_description_in_immutable(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_missing_author(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
             .run_jj_with(|cmd| cmd.args(args).env_remove(var))
@@ -1810,12 +1781,9 @@ fn test_git_push_missing_author(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_missing_author_in_immutable(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
             .run_jj_with(|cmd| cmd.args(args).env_remove(var))
@@ -1871,12 +1839,9 @@ fn test_git_push_missing_author_in_immutable(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_missing_committer(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
             .run_jj_with(|cmd| cmd.args(args).env_remove(var))
@@ -1930,12 +1895,9 @@ fn test_git_push_missing_committer(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_missing_committer_in_immutable(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     let run_without_var = |var: &str, args: &[&str]| {
         work_dir
             .run_jj_with(|cmd| cmd.args(args).env_remove(var))
@@ -1992,12 +1954,9 @@ fn test_git_push_missing_committer_in_immutable(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_deleted(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     work_dir
         .run_jj(["bookmark", "delete", "bookmark1"])
@@ -2037,12 +1996,9 @@ fn test_git_push_deleted(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_conflicting_bookmarks(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     test_env.add_config("git.auto-local-bookmark = true");
     let git_repo = {
         let mut git_repo_path = work_dir.root().to_owned();
@@ -2139,12 +2095,9 @@ fn test_git_push_conflicting_bookmarks(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_deleted_untracked(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     // Absent local bookmark shouldn't be considered "deleted" compared to
     // non-tracking remote bookmark.
@@ -2176,12 +2129,9 @@ fn test_git_push_deleted_untracked(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_tracked_vs_all(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     work_dir
         .run_jj(["new", "bookmark1", "-mmoved bookmark1"])
         .success();
@@ -2291,12 +2241,9 @@ fn test_git_push_tracked_vs_all(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_moved_forward_untracked(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     work_dir
         .run_jj(["new", "bookmark1", "-mmoved bookmark1"])
@@ -2322,12 +2269,9 @@ fn test_git_push_moved_forward_untracked(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_moved_sideways_untracked(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
 
     work_dir
         .run_jj(["new", "root()", "-mmoved bookmark1"])
@@ -2353,12 +2297,9 @@ fn test_git_push_moved_sideways_untracked(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_to_remote_named_git(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     let git_repo_path = {
         let mut git_repo_path = work_dir.root().to_owned();
         git_repo_path.extend([".jj", "repo", "store", "git"]);
@@ -2384,12 +2325,9 @@ fn test_git_push_to_remote_named_git(subprocess: bool) {
 #[cfg_attr(feature = "git2", test_case(false; "use git2 for remote calls"))]
 #[test_case(true; "spawn a git subprocess for remote calls")]
 fn test_git_push_to_remote_with_slashes(subprocess: bool) {
-    let test_env = TestEnvironment::default();
+    let test_env = TestEnvironment::default().with_git_subprocess(subprocess);
     set_up(&test_env);
     let work_dir = test_env.work_dir("local");
-    if !subprocess {
-        test_env.add_config("git.subprocess = false");
-    }
     let git_repo_path = {
         let mut git_repo_path = work_dir.root().to_owned();
         git_repo_path.extend([".jj", "repo", "store", "git"]);
