@@ -12,25 +12,20 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-use std::path::Path;
-
 use crate::common::CommandOutput;
 use crate::common::TestEnvironment;
+use crate::common::TestWorkDir;
 
 #[test]
 fn test_new() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "add a file"])
-        .success();
-    test_env
-        .run_jj_in(&repo_path, ["new", "-m", "a new commit"])
-        .success();
+    work_dir.run_jj(["describe", "-m", "add a file"]).success();
+    work_dir.run_jj(["new", "-m", "a new commit"]).success();
 
-    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  34f3c770f1db22ac5c58df21d587aed1a030201f a new commit
     ○  bf8753cb48b860b68386c5c8cc997e8e37122485 add a file
     ◆  0000000000000000000000000000000000000000
@@ -38,10 +33,10 @@ fn test_new() {
     ");
 
     // Start a new change off of a specific commit (the root commit in this case).
-    test_env
-        .run_jj_in(&repo_path, ["new", "-m", "off of root", "root()"])
+    work_dir
+        .run_jj(["new", "-m", "off of root", "root()"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  026537ddb96b801b9cb909985d5443aab44616c1 off of root
     │ ○  34f3c770f1db22ac5c58df21d587aed1a030201f a new commit
     │ ○  bf8753cb48b860b68386c5c8cc997e8e37122485 add a file
@@ -51,10 +46,10 @@ fn test_new() {
     ");
 
     // --edit is a no-op
-    test_env
-        .run_jj_in(&repo_path, ["new", "--edit", "-m", "yet another commit"])
+    work_dir
+        .run_jj(["new", "--edit", "-m", "yet another commit"])
         .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @  101cbec5cae8049cb9850a906ef3675631ed48fa yet another commit
     ○  026537ddb96b801b9cb909985d5443aab44616c1 off of root
     │ ○  34f3c770f1db22ac5c58df21d587aed1a030201f a new commit
@@ -65,7 +60,7 @@ fn test_new() {
     ");
 
     // --edit cannot be used with --no-edit
-    let output = test_env.run_jj_in(&repo_path, ["new", "--edit", "B", "--no-edit", "D"]);
+    let output = work_dir.run_jj(["new", "--edit", "B", "--no-edit", "D"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the argument '--edit' cannot be used with '--no-edit'
@@ -82,25 +77,21 @@ fn test_new() {
 fn test_new_merge() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    test_env
-        .run_jj_in(&repo_path, ["bookmark", "create", "-r@", "main"])
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "main"])
         .success();
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "add file1"])
+    work_dir.run_jj(["describe", "-m", "add file1"]).success();
+    work_dir.write_file("file1", "a");
+    work_dir
+        .run_jj(["new", "root()", "-m", "add file2"])
         .success();
-    std::fs::write(repo_path.join("file1"), "a").unwrap();
-    test_env
-        .run_jj_in(&repo_path, ["new", "root()", "-m", "add file2"])
-        .success();
-    std::fs::write(repo_path.join("file2"), "b").unwrap();
+    work_dir.write_file("file2", "b");
 
     // Create a merge commit
-    test_env
-        .run_jj_in(&repo_path, ["new", "main", "@"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r"
+    work_dir.run_jj(["new", "main", "@"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    2f9a61ea1fef257eca52fcee2feec1cbd2e41660
     ├─╮
     │ ○  f399209d9dda06e8a25a0c8e9a0cde9f421ff35d add file2
@@ -109,20 +100,20 @@ fn test_new_merge() {
     ◆  0000000000000000000000000000000000000000
     [EOF]
     ");
-    let output = test_env.run_jj_in(&repo_path, ["file", "show", "file1"]);
+    let output = work_dir.run_jj(["file", "show", "file1"]);
     insta::assert_snapshot!(output, @"a[EOF]");
-    let output = test_env.run_jj_in(&repo_path, ["file", "show", "file2"]);
+    let output = work_dir.run_jj(["file", "show", "file2"]);
     insta::assert_snapshot!(output, @"b[EOF]");
 
     // Same test with `--no-edit`
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
-    let output = test_env.run_jj_in(&repo_path, ["new", "main", "@", "--no-edit"]);
+    work_dir.run_jj(["undo"]).success();
+    let output = work_dir.run_jj(["new", "main", "@", "--no-edit"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Created new commit znkkpsqq 496490a6 (empty) (no description set)
     [EOF]
     ");
-    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     ○    496490a66cebb31730c4103b7b22a1098d49af91
     ├─╮
     │ @  f399209d9dda06e8a25a0c8e9a0cde9f421ff35d add file2
@@ -133,11 +124,9 @@ fn test_new_merge() {
     ");
 
     // Same test with `jj new`
-    test_env.run_jj_in(&repo_path, ["undo"]).success();
-    test_env
-        .run_jj_in(&repo_path, ["new", "main", "@"])
-        .success();
-    insta::assert_snapshot!(get_log_output(&test_env, &repo_path), @r"
+    work_dir.run_jj(["undo"]).success();
+    work_dir.run_jj(["new", "main", "@"]).success();
+    insta::assert_snapshot!(get_log_output(&work_dir), @r"
     @    114023233c454e2eca22b8b209f9e42f755eb28c
     ├─╮
     │ ○  f399209d9dda06e8a25a0c8e9a0cde9f421ff35d add file2
@@ -148,7 +137,7 @@ fn test_new_merge() {
     ");
 
     // merge with non-unique revisions
-    let output = test_env.run_jj_in(&repo_path, ["new", "@", "3a44e"]);
+    let output = work_dir.run_jj(["new", "@", "3a44e"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Revision `3a44e` doesn't exist
@@ -156,16 +145,16 @@ fn test_new_merge() {
     [exit status: 1]
     ");
     // if prefixed with all:, duplicates are allowed
-    let output = test_env.run_jj_in(&repo_path, ["new", "@", "all:visible_heads()"]);
+    let output = work_dir.run_jj(["new", "@", "all:visible_heads()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
-    Working copy now at: nkmrtpmo ed2dc1d9 (empty) (no description set)
-    Parent commit      : wqnwkozp 11402323 (empty) (no description set)
+    Working copy  (@) now at: nkmrtpmo ed2dc1d9 (empty) (no description set)
+    Parent commit (@-)      : wqnwkozp 11402323 (empty) (no description set)
     [EOF]
     ");
 
     // merge with root
-    let output = test_env.run_jj_in(&repo_path, ["new", "@", "root()"]);
+    let output = work_dir.run_jj(["new", "@", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
@@ -178,9 +167,9 @@ fn test_new_merge() {
 fn test_new_insert_after() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     @    F
     ├─╮
     │ ○  E
@@ -195,19 +184,16 @@ fn test_new_insert_after() {
     ");
 
     // --insert-after can be repeated; --after is an alias
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["new", "-m", "G", "--insert-after", "B", "--after", "D"],
-    );
+    let output = work_dir.run_jj(["new", "-m", "G", "--insert-after", "B", "--after", "D"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 2 descendant commits
-    Working copy now at: kxryzmor 1fc93fd1 (empty) G
-    Parent commit      : kkmpptxz bfd4157e B | (empty) B
-    Parent commit      : vruxwmqv c9257eff D | (empty) D
+    Working copy  (@) now at: kxryzmor 1fc93fd1 (empty) G
+    Parent commit (@-)      : kkmpptxz bfd4157e B | (empty) B
+    Parent commit (@-)      : vruxwmqv c9257eff D | (empty) D
     [EOF]
     ");
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     ○  C
     │ ○  F
     ╭─┤
@@ -223,15 +209,15 @@ fn test_new_insert_after() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(&repo_path, ["new", "-m", "H", "--insert-after", "D"]);
+    let output = work_dir.run_jj(["new", "-m", "H", "--insert-after", "D"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 3 descendant commits
-    Working copy now at: uyznsvlq fcf8281b (empty) H
-    Parent commit      : vruxwmqv c9257eff D | (empty) D
+    Working copy  (@) now at: uyznsvlq fcf8281b (empty) H
+    Parent commit (@-)      : vruxwmqv c9257eff D | (empty) D
     [EOF]
     ");
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     ○  C
     │ ○  F
     ╭─┤
@@ -249,7 +235,7 @@ fn test_new_insert_after() {
     ");
 
     // --after cannot be used with revisions
-    let output = test_env.run_jj_in(&repo_path, ["new", "--after", "B", "D"]);
+    let output = work_dir.run_jj(["new", "--after", "B", "D"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the argument '--insert-after <REVSETS>' cannot be used with '[REVSETS]...'
@@ -266,9 +252,9 @@ fn test_new_insert_after() {
 fn test_new_insert_after_children() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     @    F
     ├─╮
     │ ○  E
@@ -284,33 +270,30 @@ fn test_new_insert_after_children() {
 
     // Attempting to insert G after A and C errors out due to the cycle created
     // as A is an ancestor of C.
-    let output = test_env.run_jj_in(
-        &repo_path,
-        [
-            "new",
-            "-m",
-            "G",
-            "--insert-after",
-            "A",
-            "--insert-after",
-            "C",
-        ],
-    );
-    insta::assert_snapshot!(output, @r#"
+    let output = work_dir.run_jj([
+        "new",
+        "-m",
+        "G",
+        "--insert-after",
+        "A",
+        "--insert-after",
+        "C",
+    ]);
+    insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Refusing to create a loop: commit 83376b270925 would be both an ancestor and a descendant of the new commit
     [EOF]
     [exit status: 1]
-    "#);
+    ");
 }
 
 #[test]
 fn test_new_insert_before() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     @    F
     ├─╮
     │ ○  E
@@ -324,28 +307,25 @@ fn test_new_insert_before() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &repo_path,
-        [
-            "new",
-            "-m",
-            "G",
-            "--insert-before",
-            "C",
-            "--insert-before",
-            "F",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "new",
+        "-m",
+        "G",
+        "--insert-before",
+        "C",
+        "--insert-before",
+        "F",
+    ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 2 descendant commits
-    Working copy now at: kxryzmor 7ed2d6ff (empty) G
-    Parent commit      : kkmpptxz bfd4157e B | (empty) B
-    Parent commit      : vruxwmqv c9257eff D | (empty) D
-    Parent commit      : znkkpsqq 41a89ffc E | (empty) E
+    Working copy  (@) now at: kxryzmor 7ed2d6ff (empty) G
+    Parent commit (@-)      : kkmpptxz bfd4157e B | (empty) B
+    Parent commit (@-)      : vruxwmqv c9257eff D | (empty) D
+    Parent commit (@-)      : znkkpsqq 41a89ffc E | (empty) E
     [EOF]
     ");
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     ○  F
     │ ○  C
     ├─╯
@@ -362,7 +342,7 @@ fn test_new_insert_before() {
     ");
 
     // --before cannot be used with revisions
-    let output = test_env.run_jj_in(&repo_path, ["new", "--before", "B", "D"]);
+    let output = work_dir.run_jj(["new", "--before", "B", "D"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     error: the argument '--insert-before <REVSETS>' cannot be used with '[REVSETS]...'
@@ -379,9 +359,9 @@ fn test_new_insert_before() {
 fn test_new_insert_before_root_successors() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     @    F
     ├─╮
     │ ○  E
@@ -395,26 +375,23 @@ fn test_new_insert_before_root_successors() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &repo_path,
-        [
-            "new",
-            "-m",
-            "G",
-            "--insert-before",
-            "A",
-            "--insert-before",
-            "D",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "new",
+        "-m",
+        "G",
+        "--insert-before",
+        "A",
+        "--insert-before",
+        "D",
+    ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 5 descendant commits
-    Working copy now at: kxryzmor 36541977 (empty) G
-    Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
+    Working copy  (@) now at: kxryzmor 36541977 (empty) G
+    Parent commit (@-)      : zzzzzzzz 00000000 (empty) (no description set)
     [EOF]
     ");
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     ○    F
     ├─╮
     │ ○  E
@@ -434,10 +411,10 @@ fn test_new_insert_before_root_successors() {
 fn test_new_insert_before_no_loop() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
     let template = r#"commit_id.short() ++ " " ++ if(description, description, "root")"#;
-    let output = test_env.run_jj_in(&repo_path, ["log", "-T", template]);
+    let output = work_dir.run_jj(["log", "-T", template]);
     insta::assert_snapshot!(output, @r"
     @    7705d353bf5d F
     ├─╮
@@ -452,18 +429,15 @@ fn test_new_insert_before_no_loop() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &repo_path,
-        [
-            "new",
-            "-m",
-            "G",
-            "--insert-before",
-            "A",
-            "--insert-before",
-            "C",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "new",
+        "-m",
+        "G",
+        "--insert-before",
+        "A",
+        "--insert-before",
+        "C",
+    ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Refusing to create a loop: commit bfd4157e6ea4 would be both an ancestor and a descendant of the new commit
@@ -476,9 +450,9 @@ fn test_new_insert_before_no_loop() {
 fn test_new_insert_before_no_root_merge() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     @    F
     ├─╮
     │ ○  E
@@ -492,18 +466,15 @@ fn test_new_insert_before_no_root_merge() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &repo_path,
-        [
-            "new",
-            "-m",
-            "G",
-            "--insert-before",
-            "B",
-            "--insert-before",
-            "D",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "new",
+        "-m",
+        "G",
+        "--insert-before",
+        "B",
+        "--insert-before",
+        "D",
+    ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The Git backend does not support creating merge commits with the root commit as one of the parents.
@@ -516,9 +487,9 @@ fn test_new_insert_before_no_root_merge() {
 fn test_new_insert_before_root() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     @    F
     ├─╮
     │ ○  E
@@ -532,7 +503,7 @@ fn test_new_insert_before_root() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(&repo_path, ["new", "-m", "G", "--insert-before", "root()"]);
+    let output = work_dir.run_jj(["new", "-m", "G", "--insert-before", "root()"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: The root commit 000000000000 is immutable
@@ -545,9 +516,9 @@ fn test_new_insert_before_root() {
 fn test_new_insert_after_before() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     @    F
     ├─╮
     │ ○  E
@@ -561,18 +532,15 @@ fn test_new_insert_after_before() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["new", "-m", "G", "--after", "C", "--before", "F"],
-    );
+    let output = work_dir.run_jj(["new", "-m", "G", "--after", "C", "--before", "F"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 1 descendant commits
-    Working copy now at: kxryzmor 78a97058 (empty) G
-    Parent commit      : mzvwutvl 83376b27 C | (empty) C
+    Working copy  (@) now at: kxryzmor 78a97058 (empty) G
+    Parent commit (@-)      : mzvwutvl 83376b27 C | (empty) C
     [EOF]
     ");
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     ○      F
     ├─┬─╮
     │ │ @  G
@@ -587,18 +555,15 @@ fn test_new_insert_after_before() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &repo_path,
-        ["new", "-m", "H", "--after", "D", "--before", "B"],
-    );
+    let output = work_dir.run_jj(["new", "-m", "H", "--after", "D", "--before", "B"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Rebased 4 descendant commits
-    Working copy now at: uyznsvlq fcf8281b (empty) H
-    Parent commit      : vruxwmqv c9257eff D | (empty) D
+    Working copy  (@) now at: uyznsvlq fcf8281b (empty) H
+    Parent commit (@-)      : vruxwmqv c9257eff D | (empty) D
     [EOF]
     ");
-    insta::assert_snapshot!(get_short_log_output(&test_env, &repo_path), @r"
+    insta::assert_snapshot!(get_short_log_output(&work_dir), @r"
     ○      F
     ├─┬─╮
     │ │ ○  G
@@ -621,10 +586,10 @@ fn test_new_insert_after_before() {
 fn test_new_insert_after_before_no_loop() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
-    setup_before_insertion(&test_env, &repo_path);
+    let work_dir = test_env.work_dir("repo");
+    setup_before_insertion(&work_dir);
     let template = r#"commit_id.short() ++ " " ++ if(description, description, "root")"#;
-    let output = test_env.run_jj_in(&repo_path, ["log", "-T", template]);
+    let output = work_dir.run_jj(["log", "-T", template]);
     insta::assert_snapshot!(output, @r"
     @    7705d353bf5d F
     ├─╮
@@ -639,18 +604,15 @@ fn test_new_insert_after_before_no_loop() {
     [EOF]
     ");
 
-    let output = test_env.run_jj_in(
-        &repo_path,
-        [
-            "new",
-            "-m",
-            "G",
-            "--insert-before",
-            "A",
-            "--insert-after",
-            "C",
-        ],
-    );
+    let output = work_dir.run_jj([
+        "new",
+        "-m",
+        "G",
+        "--insert-before",
+        "A",
+        "--insert-after",
+        "C",
+    ]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Refusing to create a loop: commit 83376b270925 would be both an ancestor and a descendant of the new commit
@@ -663,35 +625,28 @@ fn test_new_insert_after_before_no_loop() {
 fn test_new_conflicting_bookmarks() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "one"])
+    work_dir.run_jj(["describe", "-m", "one"]).success();
+    work_dir.run_jj(["new", "-m", "two", "@-"]).success();
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "foo"])
         .success();
-    test_env
-        .run_jj_in(&repo_path, ["new", "-m", "two", "@-"])
-        .success();
-    test_env
-        .run_jj_in(&repo_path, ["bookmark", "create", "-r@", "foo"])
-        .success();
-    test_env
-        .run_jj_in(
-            &repo_path,
-            [
-                "--at-op=@-",
-                "bookmark",
-                "create",
-                "foo",
-                "-r",
-                r#"description("one")"#,
-            ],
-        )
+    work_dir
+        .run_jj([
+            "--at-op=@-",
+            "bookmark",
+            "create",
+            "foo",
+            "-r",
+            r#"description("one")"#,
+        ])
         .success();
 
     // Trigger resolution of divergent operations
-    test_env.run_jj_in(&repo_path, ["st"]).success();
+    work_dir.run_jj(["st"]).success();
 
-    let output = test_env.run_jj_in(&repo_path, ["new", "foo"]);
+    let output = work_dir.run_jj(["new", "foo"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Revset `foo` resolved to more than one revision
@@ -709,19 +664,17 @@ fn test_new_conflicting_bookmarks() {
 fn test_new_conflicting_change_ids() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "one"])
-        .success();
-    test_env
-        .run_jj_in(&repo_path, ["--at-op=@-", "describe", "-m", "two"])
+    work_dir.run_jj(["describe", "-m", "one"]).success();
+    work_dir
+        .run_jj(["--at-op=@-", "describe", "-m", "two"])
         .success();
 
     // Trigger resolution of divergent operations
-    test_env.run_jj_in(&repo_path, ["st"]).success();
+    work_dir.run_jj(["st"]).success();
 
-    let output = test_env.run_jj_in(&repo_path, ["new", "qpvuntsm"]);
+    let output = work_dir.run_jj(["new", "qpvuntsm"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Revset `qpvuntsm` resolved to more than one revision
@@ -738,16 +691,12 @@ fn test_new_conflicting_change_ids() {
 fn test_new_error_revision_does_not_exist() {
     let test_env = TestEnvironment::default();
     test_env.run_jj_in(".", ["git", "init", "repo"]).success();
-    let repo_path = test_env.env_root().join("repo");
+    let work_dir = test_env.work_dir("repo");
 
-    test_env
-        .run_jj_in(&repo_path, ["describe", "-m", "one"])
-        .success();
-    test_env
-        .run_jj_in(&repo_path, ["new", "-m", "two"])
-        .success();
+    work_dir.run_jj(["describe", "-m", "one"]).success();
+    work_dir.run_jj(["new", "-m", "two"]).success();
 
-    let output = test_env.run_jj_in(&repo_path, ["new", "this"]);
+    let output = work_dir.run_jj(["new", "this"]);
     insta::assert_snapshot!(output, @r"
     ------- stderr -------
     Error: Revision `this` doesn't exist
@@ -756,54 +705,44 @@ fn test_new_error_revision_does_not_exist() {
     ");
 }
 
-fn setup_before_insertion(test_env: &TestEnvironment, repo_path: &Path) {
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", "A"])
+fn setup_before_insertion(work_dir: &TestWorkDir) {
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "A"])
         .success();
-    test_env
-        .run_jj_in(repo_path, ["commit", "-m", "A"])
+    work_dir.run_jj(["commit", "-m", "A"]).success();
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "B"])
         .success();
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", "B"])
+    work_dir.run_jj(["commit", "-m", "B"]).success();
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "C"])
         .success();
-    test_env
-        .run_jj_in(repo_path, ["commit", "-m", "B"])
+    work_dir.run_jj(["describe", "-m", "C"]).success();
+    work_dir.run_jj(["new", "-m", "D", "root()"]).success();
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "D"])
         .success();
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", "C"])
-        .success();
-    test_env
-        .run_jj_in(repo_path, ["describe", "-m", "C"])
-        .success();
-    test_env
-        .run_jj_in(repo_path, ["new", "-m", "D", "root()"])
-        .success();
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", "D"])
-        .success();
-    test_env
-        .run_jj_in(repo_path, ["new", "-m", "E", "root()"])
-        .success();
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", "E"])
+    work_dir.run_jj(["new", "-m", "E", "root()"]).success();
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "E"])
         .success();
     // Any number of -r's is ignored
-    test_env
-        .run_jj_in(repo_path, ["new", "-m", "F", "-r", "D", "-r", "E"])
+    work_dir
+        .run_jj(["new", "-m", "F", "-r", "D", "-r", "E"])
         .success();
-    test_env
-        .run_jj_in(repo_path, ["bookmark", "create", "-r@", "F"])
+    work_dir
+        .run_jj(["bookmark", "create", "-r@", "F"])
         .success();
 }
 
 #[must_use]
-fn get_log_output(test_env: &TestEnvironment, repo_path: &Path) -> CommandOutput {
+fn get_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     let template = r#"commit_id ++ " " ++ description"#;
-    test_env.run_jj_in(repo_path, ["log", "-T", template])
+    work_dir.run_jj(["log", "-T", template])
 }
 
 #[must_use]
-fn get_short_log_output(test_env: &TestEnvironment, repo_path: &Path) -> CommandOutput {
+fn get_short_log_output(work_dir: &TestWorkDir) -> CommandOutput {
     let template = r#"if(description, description, "root")"#;
-    test_env.run_jj_in(repo_path, ["log", "-T", template])
+    work_dir.run_jj(["log", "-T", template])
 }
