@@ -17,6 +17,7 @@
 use std::borrow::Borrow;
 use std::convert::Infallible;
 use std::fmt;
+use std::fmt::Display;
 use std::fs;
 use std::io;
 use std::ops::Range;
@@ -199,7 +200,7 @@ impl AsRef<[toml_edit::Key]> for ConfigNamePathBuf {
     }
 }
 
-impl fmt::Display for ConfigNamePathBuf {
+impl Display for ConfigNamePathBuf {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let mut components = self.0.iter().fuse();
         if let Some(key) = components.next() {
@@ -293,6 +294,20 @@ pub enum ConfigSource {
     CommandArg,
 }
 
+impl Display for ConfigSource {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        use ConfigSource::*;
+        let c = match self {
+            Default => "default",
+            User => "user",
+            Repo => "repo",
+            CommandArg => "cli",
+            EnvBase | EnvOverrides => "env",
+        };
+        write!(f, "{c}")
+    }
+}
+
 /// Set of configuration variables with source information.
 #[derive(Clone, Debug)]
 pub struct ConfigLayer {
@@ -351,8 +366,7 @@ impl ConfigLayer {
             .and_then(|dir_entries| {
                 dir_entries
                     .map(|entry| Ok(entry?.path()))
-                    // TODO: Accept only certain file extensions?
-                    .filter_ok(|path| path.is_file())
+                    .filter_ok(|path| path.is_file() && path.extension() == Some("toml".as_ref()))
                     .try_collect()
             })
             .context(path)
@@ -528,7 +542,7 @@ fn new_implicit_table() -> ConfigItem {
 
 /// Wrapper for file-based [`ConfigLayer`], providing convenient methods for
 /// modification.
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub struct ConfigFile {
     layer: Arc<ConfigLayer>,
 }
