@@ -20,14 +20,14 @@ use std::io::Write;
 use std::iter::zip;
 
 use bstr::BString;
-use bstr::ByteSlice;
+use bstr::ByteSlice as _;
 use futures::stream::BoxStream;
 use futures::try_join;
 use futures::Stream;
-use futures::StreamExt;
-use futures::TryStreamExt;
-use itertools::Itertools;
-use pollster::FutureExt;
+use futures::StreamExt as _;
+use futures::TryStreamExt as _;
+use itertools::Itertools as _;
+use pollster::FutureExt as _;
 
 use crate::backend::BackendError;
 use crate::backend::BackendResult;
@@ -134,11 +134,7 @@ pub async fn extract_as_single_hunk(
 pub enum MaterializedTreeValue {
     Absent,
     AccessDenied(Box<dyn std::error::Error + Send + Sync>),
-    File {
-        id: FileId,
-        executable: bool,
-        reader: Box<dyn Read>,
-    },
+    File(MaterializedFileValue),
     Symlink {
         id: SymlinkId,
         target: String,
@@ -167,6 +163,29 @@ impl MaterializedTreeValue {
     }
 }
 
+/// [`TreeValue::File`] with file content `reader`.
+pub struct MaterializedFileValue {
+    pub id: FileId,
+    pub executable: bool,
+    pub reader: Box<dyn Read>,
+}
+
+impl MaterializedFileValue {
+    /// Reads file content until EOF. The provided `path` is used only for error
+    /// reporting purpose.
+    pub fn read_all(&mut self, path: &RepoPath) -> BackendResult<Vec<u8>> {
+        let mut buf = Vec::new();
+        self.reader
+            .read_to_end(&mut buf)
+            .map_err(|err| BackendError::ReadFile {
+                path: path.to_owned(),
+                id: self.id.clone(),
+                source: err.into(),
+            })?;
+        Ok(buf)
+    }
+}
+
 /// Reads the data associated with a `MergedTreeValue` so it can be written to
 /// e.g. the working copy or diff.
 pub async fn materialize_tree_value(
@@ -191,11 +210,11 @@ async fn materialize_tree_value_no_access_denied(
         Ok(None) => Ok(MaterializedTreeValue::Absent),
         Ok(Some(TreeValue::File { id, executable })) => {
             let reader = store.read_file_async(path, &id).await?;
-            Ok(MaterializedTreeValue::File {
+            Ok(MaterializedTreeValue::File(MaterializedFileValue {
                 id,
                 executable,
                 reader,
-            })
+            }))
         }
         Ok(Some(TreeValue::Symlink(id))) => {
             let target = store.read_symlink_async(path, &id).await?;
@@ -665,7 +684,7 @@ pub struct MaterializedTreeDiffEntry {
 pub fn materialized_diff_stream<'a>(
     store: &'a Store,
     tree_diff: BoxStream<'a, CopiesTreeDiffEntry>,
-) -> impl Stream<Item = MaterializedTreeDiffEntry> + 'a {
+) -> impl Stream<Item = MaterializedTreeDiffEntry> + use<'a> {
     tree_diff
         .map(|CopiesTreeDiffEntry { path, values }| async {
             match values {
