@@ -47,12 +47,12 @@ pub use self::store::DefaultIndexStoreInitError;
 mod tests {
     use std::sync::Arc;
 
-    use itertools::Itertools;
+    use itertools::Itertools as _;
     use smallvec::smallvec_inline;
     use test_case::test_case;
 
     use super::composite::DynIndexSegment;
-    use super::composite::IndexSegment;
+    use super::composite::IndexSegment as _;
     use super::entry::SmallIndexPositionsVec;
     use super::mutable::MutableIndexSegment;
     use super::*;
@@ -60,9 +60,9 @@ mod tests {
     use crate::backend::CommitId;
     use crate::default_index::entry::LocalPosition;
     use crate::default_index::entry::SmallLocalPositionsVec;
-    use crate::index::Index;
+    use crate::index::Index as _;
     use crate::object_id::HexPrefix;
-    use crate::object_id::ObjectId;
+    use crate::object_id::ObjectId as _;
     use crate::object_id::PrefixResolution;
     use crate::tests::new_temp_dir;
 
@@ -175,7 +175,7 @@ mod tests {
         let id_1 = CommitId::from_hex("111111");
         let change_id1 = new_change_id();
         let id_2 = CommitId::from_hex("222222");
-        #[allow(clippy::redundant_clone)] // Work around nightly clippy false positive
+        #[expect(clippy::redundant_clone)] // Work around nightly clippy false positive
         // TODO: Remove the exception after https://github.com/rust-lang/rust-clippy/issues/10577
         // is fixed or file a new bug.
         let change_id2 = change_id1.clone();
@@ -415,7 +415,7 @@ mod tests {
     }
 
     #[test]
-    #[allow(clippy::redundant_clone)] // allow id_n.clone()
+    #[expect(clippy::redundant_clone)] // allow id_n.clone()
     fn neighbor_commit_ids() {
         let temp_dir = new_temp_dir();
         let mut new_change_id = change_id_generator();
