@@ -28,7 +28,7 @@ use std::str;
 use std::sync::Arc;
 
 use futures::StreamExt as _;
-use itertools::Itertools;
+use itertools::Itertools as _;
 use pollster::FutureExt as _;
 
 use super::rev_walk::EagerRevWalk;
@@ -36,7 +36,6 @@ use super::rev_walk::PeekableRevWalk;
 use super::rev_walk::RevWalk;
 use super::rev_walk::RevWalkBuilder;
 use super::revset_graph_iterator::RevsetGraphWalk;
-use crate::backend::BackendError;
 use crate::backend::BackendResult;
 use crate::backend::ChangeId;
 use crate::backend::CommitId;
@@ -127,14 +126,16 @@ impl<I: AsCompositeIndex + Clone> RevsetImpl<I> {
         Self { inner, index }
     }
 
-    fn positions(&self) -> impl Iterator<Item = Result<IndexPosition, RevsetEvaluationError>> + '_ {
+    fn positions(
+        &self,
+    ) -> impl Iterator<Item = Result<IndexPosition, RevsetEvaluationError>> + use<'_, I> {
         self.inner.positions().attach(self.index.as_composite())
     }
 
     pub fn iter_graph_impl(
         &self,
         skip_transitive_edges: bool,
-    ) -> impl Iterator<Item = Result<GraphNode<CommitId>, RevsetEvaluationError>> {
+    ) -> impl Iterator<Item = Result<GraphNode<CommitId>, RevsetEvaluationError>> + use<I> {
         let index = self.index.clone();
         let walk = self.inner.positions();
         let mut graph_walk = RevsetGraphWalk::new(walk, skip_transitive_edges);
@@ -1336,10 +1337,10 @@ fn matches_diff_from_parent(
     .block_on()
 }
 
-fn match_lines<'a: 'b, 'b>(
+fn match_lines<'a, 'b>(
     text: &'a [u8],
     pattern: &'b StringPattern,
-) -> impl Iterator<Item = &'a [u8]> + 'b {
+) -> impl Iterator<Item = &'a [u8]> + use<'a, 'b> {
     // The pattern is matched line by line so that it can be anchored to line
     // start/end. For example, exact:"" will match blank lines.
     text.split_inclusive(|b| *b == b'\n').filter(|line| {
@@ -1353,17 +1354,7 @@ fn to_file_content(path: &RepoPath, value: MaterializedTreeValue) -> BackendResu
     match value {
         MaterializedTreeValue::Absent => Ok(vec![]),
         MaterializedTreeValue::AccessDenied(_) => Ok(vec![]),
-        MaterializedTreeValue::File { id, mut reader, .. } => {
-            let mut content = vec![];
-            reader
-                .read_to_end(&mut content)
-                .map_err(|err| BackendError::ReadFile {
-                    path: path.to_owned(),
-                    id: id.clone(),
-                    source: err.into(),
-                })?;
-            Ok(content)
-        }
+        MaterializedTreeValue::File(mut file) => file.read_all(path),
         MaterializedTreeValue::Symlink { id: _, target } => Ok(target.into_bytes()),
         MaterializedTreeValue::GitSubmodule(_) => Ok(vec![]),
         MaterializedTreeValue::FileConflict { contents, .. } => {
