@@ -14,7 +14,6 @@
 
 //! Domain-specific language helpers.
 
-use std::array;
 use std::ascii;
 use std::collections::HashMap;
 use std::fmt;
@@ -149,7 +148,7 @@ impl<'i, T> FunctionCallNode<'i, T> {
     }
 
     /// Extracts N required arguments and remainders.
-    #[allow(clippy::type_complexity)]
+    #[expect(clippy::type_complexity)]
     pub fn expect_some_arguments<const N: usize>(
         &self,
     ) -> Result<(&[ExpressionNode<'i, T>; N], &[ExpressionNode<'i, T>]), InvalidArguments<'i>> {
@@ -163,7 +162,7 @@ impl<'i, T> FunctionCallNode<'i, T> {
     }
 
     /// Extracts N required arguments and M optional arguments.
-    #[allow(clippy::type_complexity)]
+    #[expect(clippy::type_complexity)]
     pub fn expect_arguments<const N: usize, const M: usize>(
         &self,
     ) -> Result<
@@ -194,7 +193,7 @@ impl<'i, T> FunctionCallNode<'i, T> {
     ///
     /// `names` is a list of parameter names. Unnamed positional arguments
     /// should be padded with `""`.
-    #[allow(clippy::type_complexity)]
+    #[expect(clippy::type_complexity)]
     pub fn expect_named_arguments<const N: usize, const M: usize>(
         &self,
         names: &[&str],
@@ -207,8 +206,7 @@ impl<'i, T> FunctionCallNode<'i, T> {
     > {
         if self.keyword_args.is_empty() {
             let (required, optional) = self.expect_arguments::<N, M>()?;
-            // TODO: use .each_ref() if MSRV is bumped to 1.77.0
-            Ok((array::from_fn(|i| &required[i]), optional))
+            Ok((required.each_ref(), optional))
         } else {
             let (required, optional) = self.expect_named_arguments_vec(names, N, N + M)?;
             Ok((
@@ -218,7 +216,7 @@ impl<'i, T> FunctionCallNode<'i, T> {
         }
     }
 
-    #[allow(clippy::type_complexity)]
+    #[expect(clippy::type_complexity)]
     fn expect_named_arguments_vec(
         &self,
         names: &[&str],
@@ -606,7 +604,10 @@ struct AliasFunctionOverloads<'a, V> {
 }
 
 impl<'a, V> AliasFunctionOverloads<'a, V> {
-    fn arities(&self) -> impl DoubleEndedIterator<Item = usize> + ExactSizeIterator + 'a {
+    // TODO: Perhaps, V doesn't have to be captured, but "currently, all type
+    // parameters are required to be mentioned in the precise captures list" as
+    // of rustc 1.85.0.
+    fn arities(&self) -> impl DoubleEndedIterator<Item = usize> + ExactSizeIterator + use<'a, V> {
         self.overloads.iter().map(|(params, _)| params.len())
     }
 
@@ -711,9 +712,11 @@ pub trait AliasExpandError: Sized {
 
 /// Expands aliases recursively in tree of `T`.
 #[derive(Debug)]
-struct AliasExpander<'i, T, P> {
+struct AliasExpander<'i, 'a, T, P> {
     /// Alias symbols and functions that are globally available.
     aliases_map: &'i AliasesMap<P, String>,
+    /// Local variables set in the outermost scope.
+    locals: &'a HashMap<&'i str, ExpressionNode<'i, T>>,
     /// Stack of aliases and local parameters currently expanding.
     states: Vec<AliasExpandingState<'i, T>>,
 }
@@ -724,12 +727,17 @@ struct AliasExpandingState<'i, T> {
     locals: HashMap<&'i str, ExpressionNode<'i, T>>,
 }
 
-impl<'i, T, P, E> AliasExpander<'i, T, P>
+impl<'i, T, P, E> AliasExpander<'i, '_, T, P>
 where
     T: AliasExpandableExpression<'i> + Clone,
     P: AliasDefinitionParser<Output<'i> = T, Error = E>,
     E: AliasExpandError,
 {
+    /// Local variables available to the current scope.
+    fn current_locals(&self) -> &HashMap<&'i str, ExpressionNode<'i, T>> {
+        self.states.last().map_or(self.locals, |s| &s.locals)
+    }
+
     fn expand_defn(
         &mut self,
         id: AliasId<'i>,
@@ -755,7 +763,7 @@ where
     }
 }
 
-impl<'i, T, P, E> ExpressionFolder<'i, T> for AliasExpander<'i, T, P>
+impl<'i, T, P, E> ExpressionFolder<'i, T> for AliasExpander<'i, '_, T, P>
 where
     T: AliasExpandableExpression<'i> + Clone,
     P: AliasDefinitionParser<Output<'i> = T, Error = E>,
@@ -764,7 +772,7 @@ where
     type Error = E;
 
     fn fold_identifier(&mut self, name: &'i str, span: pest::Span<'i>) -> Result<T, Self::Error> {
-        if let Some(subst) = self.states.last().and_then(|s| s.locals.get(name)) {
+        if let Some(subst) = self.current_locals().get(name) {
             let id = AliasId::Parameter(name);
             Ok(T::alias_expanded(id, Box::new(subst.clone())))
         } else if let Some((id, defn)) = self.aliases_map.get_symbol(name) {
@@ -814,6 +822,23 @@ pub fn expand_aliases<'i, T, P>(
     node: ExpressionNode<'i, T>,
     aliases_map: &'i AliasesMap<P, String>,
 ) -> Result<ExpressionNode<'i, T>, P::Error>
+where
+    T: AliasExpandableExpression<'i> + Clone,
+    P: AliasDefinitionParser<Output<'i> = T>,
+    P::Error: AliasExpandError,
+{
+    expand_aliases_with_locals(node, aliases_map, &HashMap::new())
+}
+
+/// Expands aliases recursively with the outermost local variables.
+///
+/// Local variables are similar to alias symbols, but are scoped. Alias symbols
+/// are globally accessible from alias expressions, but local variables aren't.
+pub fn expand_aliases_with_locals<'i, T, P>(
+    node: ExpressionNode<'i, T>,
+    aliases_map: &'i AliasesMap<P, String>,
+    locals: &HashMap<&'i str, ExpressionNode<'i, T>>,
+) -> Result<ExpressionNode<'i, T>, P::Error>
 where
     T: AliasExpandableExpression<'i> + Clone,
     P: AliasDefinitionParser<Output<'i> = T>,
@@ -821,6 +846,7 @@ where
 {
     let mut expander = AliasExpander {
         aliases_map,
+        locals,
         states: Vec::new(),
     };
     expander.fold_expression(node)
