@@ -47,6 +47,7 @@ pub mod file_util;
 pub mod files;
 pub mod fileset;
 mod fileset_parser;
+pub mod fix;
 pub mod fmt_util;
 pub mod fsmonitor;
 #[cfg(feature = "git")]
@@ -55,11 +56,12 @@ pub mod git;
 /// A stub module that provides a no-op implementation of some of the functions
 /// in the `git` module.
 pub mod git {
+    use crate::ref_name::RemoteName;
     /// Determine, by its name, if a remote refers to the special local-only
     /// "git" remote that is used in the Git backend.
     ///
     /// This function always returns false if the "git" feature is not enabled.
-    pub fn is_special_git_remote(_remote: &str) -> bool {
+    pub fn is_special_git_remote(_remote: &RemoteName) -> bool {
         false
     }
 }
@@ -73,7 +75,6 @@ pub mod graph;
 pub mod hex_util;
 pub mod id_prefix;
 pub mod index;
-pub mod local_backend;
 pub mod local_working_copy;
 pub mod lock;
 pub mod matchers;
@@ -84,8 +85,9 @@ pub mod op_heads_store;
 pub mod op_store;
 pub mod op_walk;
 pub mod operation;
-#[allow(missing_docs)]
+#[expect(missing_docs)]
 pub mod protos;
+pub mod ref_name;
 pub mod refs;
 pub mod repo;
 pub mod repo_path;
@@ -96,6 +98,10 @@ pub mod rewrite;
 pub mod secret_backend;
 pub mod settings;
 pub mod signing;
+// TODO: This file is mostly used for testing, whenever we no longer require it
+// in the lib it should be moved to the examples (e.g
+// "examples/simple-backend/").
+pub mod simple_backend;
 pub mod simple_op_heads_store;
 pub mod simple_op_store;
 pub mod ssh_signing;
@@ -119,7 +125,7 @@ mod tests {
     use tempfile::TempDir;
 
     /// Unlike `testutils::new_temp_dir()`, this function doesn't set up
-    /// hermetic libgit2 environment.
+    /// hermetic Git environment.
     pub fn new_temp_dir() -> TempDir {
         tempfile::Builder::new()
             .prefix("jj-test-")
