@@ -26,12 +26,12 @@ use std::fs::Metadata;
 use std::fs::OpenOptions;
 use std::io;
 use std::io::Read;
-use std::io::Write;
+use std::io::Write as _;
 use std::iter;
 use std::mem;
 use std::ops::Range;
 #[cfg(unix)]
-use std::os::unix::fs::PermissionsExt;
+use std::os::unix::fs::PermissionsExt as _;
 use std::path::Path;
 use std::path::PathBuf;
 use std::slice;
@@ -42,15 +42,15 @@ use std::sync::OnceLock;
 use std::time::UNIX_EPOCH;
 
 use either::Either;
-use futures::StreamExt;
+use futures::StreamExt as _;
 use itertools::EitherOrBoth;
-use itertools::Itertools;
+use itertools::Itertools as _;
 use once_cell::unsync::OnceCell;
-use pollster::FutureExt;
-use prost::Message;
-use rayon::iter::IntoParallelIterator;
-use rayon::prelude::IndexedParallelIterator;
-use rayon::prelude::ParallelIterator;
+use pollster::FutureExt as _;
+use prost::Message as _;
+use rayon::iter::IntoParallelIterator as _;
+use rayon::prelude::IndexedParallelIterator as _;
+use rayon::prelude::ParallelIterator as _;
 use tempfile::NamedTempFile;
 use thiserror::Error;
 use tracing::instrument;
@@ -93,9 +93,10 @@ use crate::merge::MergedTreeValue;
 use crate::merged_tree::MergedTree;
 use crate::merged_tree::MergedTreeBuilder;
 use crate::merged_tree::TreeDiffEntry;
-use crate::object_id::ObjectId;
+use crate::object_id::ObjectId as _;
 use crate::op_store::OperationId;
-use crate::op_store::WorkspaceId;
+use crate::ref_name::WorkspaceName;
+use crate::ref_name::WorkspaceNameBuf;
 use crate::repo_path::RepoPath;
 use crate::repo_path::RepoPathBuf;
 use crate::repo_path::RepoPathComponent;
@@ -385,7 +386,7 @@ impl<'a> FileStates<'a> {
     }
 
     /// Iterates sorted file paths.
-    pub fn paths(&self) -> impl ExactSizeIterator<Item = &'a RepoPath> {
+    pub fn paths(&self) -> impl ExactSizeIterator<Item = &'a RepoPath> + use<'a> {
         self.data
             .iter()
             .map(|entry| RepoPath::from_internal_string(&entry.path))
@@ -847,8 +848,7 @@ impl TreeState {
         Ok(())
     }
 
-    #[allow(unknown_lints)] // XXX FIXME (aseipp): nightly bogons; re-test this occasionally
-    #[allow(clippy::assigning_clones)]
+    #[expect(clippy::assigning_clones)]
     fn save(&mut self) -> Result<(), TreeStateError> {
         let mut proto: crate::protos::working_copy::TreeState = Default::default();
         match &self.tree_id {
@@ -1831,11 +1831,9 @@ impl TreeState {
                     deleted_files.insert(path);
                     continue;
                 }
-                MaterializedTreeValue::File {
-                    executable,
-                    mut reader,
-                    ..
-                } => self.write_file(&disk_path, &mut reader, executable)?,
+                MaterializedTreeValue::File(mut file) => {
+                    self.write_file(&disk_path, &mut file.reader, file.executable)?
+                }
                 MaterializedTreeValue::Symlink { id: _, target } => {
                     if self.symlink_support {
                         self.write_symlink(&disk_path, target)?
@@ -1962,7 +1960,7 @@ fn checkout_error_for_stat_error(err: io::Error, path: &Path) -> CheckoutError {
 #[derive(Clone, Debug)]
 struct CheckoutState {
     operation_id: OperationId,
-    workspace_id: WorkspaceId,
+    workspace_name: WorkspaceNameBuf,
 }
 
 pub struct LocalWorkingCopy {
@@ -1982,8 +1980,8 @@ impl WorkingCopy for LocalWorkingCopy {
         Self::name()
     }
 
-    fn workspace_id(&self) -> &WorkspaceId {
-        &self.checkout_state().workspace_id
+    fn workspace_name(&self) -> &WorkspaceName {
+        &self.checkout_state().workspace_name
     }
 
     fn operation_id(&self) -> &OperationId {
@@ -2023,7 +2021,7 @@ impl WorkingCopy for LocalWorkingCopy {
             old_operation_id,
             old_tree_id,
             tree_state_dirty: false,
-            new_workspace_id: None,
+            new_workspace_name: None,
         }))
     }
 }
@@ -2041,11 +2039,11 @@ impl LocalWorkingCopy {
         working_copy_path: PathBuf,
         state_path: PathBuf,
         operation_id: OperationId,
-        workspace_id: WorkspaceId,
+        workspace_name: WorkspaceNameBuf,
     ) -> Result<LocalWorkingCopy, WorkingCopyStateError> {
         let proto = crate::protos::working_copy::Checkout {
             operation_id: operation_id.to_bytes(),
-            workspace_id: workspace_id.as_str().to_string(),
+            workspace_name: workspace_name.into(),
         };
         let mut file = OpenOptions::new()
             .create_new(true)
@@ -2104,12 +2102,12 @@ impl LocalWorkingCopy {
             let proto = crate::protos::working_copy::Checkout::decode(&*buf).unwrap();
             CheckoutState {
                 operation_id: OperationId::new(proto.operation_id),
-                workspace_id: if proto.workspace_id.is_empty() {
+                workspace_name: if proto.workspace_name.is_empty() {
                     // For compatibility with old working copies.
                     // TODO: Delete in mid 2022 or so
-                    WorkspaceId::default()
+                    WorkspaceName::DEFAULT.to_owned()
                 } else {
-                    WorkspaceId::new(proto.workspace_id)
+                    proto.workspace_name.into()
                 },
             }
         })
@@ -2149,7 +2147,7 @@ impl LocalWorkingCopy {
     fn save(&mut self) {
         self.write_proto(crate::protos::working_copy::Checkout {
             operation_id: self.operation_id().to_bytes(),
-            workspace_id: self.workspace_id().as_str().to_string(),
+            workspace_name: self.workspace_name().into(),
         });
     }
 
@@ -2189,14 +2187,14 @@ impl WorkingCopyFactory for LocalWorkingCopyFactory {
         working_copy_path: PathBuf,
         state_path: PathBuf,
         operation_id: OperationId,
-        workspace_id: WorkspaceId,
+        workspace_name: WorkspaceNameBuf,
     ) -> Result<Box<dyn WorkingCopy>, WorkingCopyStateError> {
         Ok(Box::new(LocalWorkingCopy::init(
             store,
             working_copy_path,
             state_path,
             operation_id,
-            workspace_id,
+            workspace_name,
         )?))
     }
 
@@ -2218,12 +2216,12 @@ impl WorkingCopyFactory for LocalWorkingCopyFactory {
 /// `finish()` or `discard()`.
 pub struct LockedLocalWorkingCopy {
     wc: LocalWorkingCopy,
-    #[allow(dead_code)]
+    #[expect(dead_code)]
     lock: FileLock,
     old_operation_id: OperationId,
     old_tree_id: MergedTreeId,
     tree_state_dirty: bool,
-    new_workspace_id: Option<WorkspaceId>,
+    new_workspace_name: Option<WorkspaceNameBuf>,
 }
 
 impl LockedWorkingCopy for LockedLocalWorkingCopy {
@@ -2283,8 +2281,8 @@ impl LockedWorkingCopy for LockedLocalWorkingCopy {
         }
     }
 
-    fn rename_workspace(&mut self, new_workspace_id: WorkspaceId) {
-        self.new_workspace_id = Some(new_workspace_id);
+    fn rename_workspace(&mut self, new_name: WorkspaceNameBuf) {
+        self.new_workspace_name = Some(new_name);
     }
 
     fn reset(&mut self, commit: &Commit) -> Result<(), ResetError> {
@@ -2353,9 +2351,9 @@ impl LockedWorkingCopy for LockedLocalWorkingCopy {
                     err: Box::new(err),
                 })?;
         }
-        if self.old_operation_id != operation_id || self.new_workspace_id.is_some() {
-            if let Some(new_workspace_id) = self.new_workspace_id {
-                self.wc.checkout_state_mut().workspace_id = new_workspace_id;
+        if self.old_operation_id != operation_id || self.new_workspace_name.is_some() {
+            if let Some(new_name) = self.new_workspace_name {
+                self.wc.checkout_state_mut().workspace_name = new_name;
             }
             self.wc.checkout_state_mut().operation_id = operation_id;
             self.wc.save();
