@@ -18,11 +18,11 @@ use std::collections::HashMap;
 use std::collections::HashSet;
 use std::sync::Arc;
 
-use futures::StreamExt;
+use futures::StreamExt as _;
 use indexmap::IndexMap;
 use indexmap::IndexSet;
-use itertools::Itertools;
-use pollster::FutureExt;
+use itertools::Itertools as _;
+use pollster::FutureExt as _;
 use tracing::instrument;
 
 use crate::backend::BackendError;
@@ -30,7 +30,7 @@ use crate::backend::BackendResult;
 use crate::backend::CommitId;
 use crate::backend::MergedTreeId;
 use crate::commit::Commit;
-use crate::commit::CommitIteratorExt;
+use crate::commit::CommitIteratorExt as _;
 use crate::commit_builder::CommitBuilder;
 use crate::dag_walk;
 use crate::index::Index;
@@ -44,7 +44,7 @@ use crate::repo::MutableRepo;
 use crate::repo::Repo;
 use crate::repo_path::RepoPath;
 use crate::revset::RevsetExpression;
-use crate::revset::RevsetIteratorExt;
+use crate::revset::RevsetIteratorExt as _;
 use crate::store::Store;
 
 /// Merges `commits` and tries to resolve any conflicts recursively.
@@ -612,10 +612,7 @@ pub fn move_commits(
                     // `target_heads`, replace them with the target heads since we are "inserting"
                     // the target commits in between the new parents and the new children.
                     for id in old_child_parent_ids {
-                        if new_parent_ids
-                            .iter()
-                            .any(|new_parent_id| *new_parent_id == *id)
-                        {
+                        if new_parent_ids.contains(id) {
                             new_child_parent_ids.extend(target_heads.clone());
                         } else {
                             new_child_parent_ids.insert(id.clone());
@@ -1029,15 +1026,15 @@ fn compute_commits_heads(
         .collect_vec()
 }
 
-pub struct CommitToSquash {
+pub struct CommitWithSelection {
     pub commit: Commit,
     pub selected_tree: MergedTree,
     pub parent_tree: MergedTree,
 }
 
-impl CommitToSquash {
+impl CommitWithSelection {
     /// Returns true if the selection contains all changes in the commit.
-    fn is_full_selection(&self) -> bool {
+    pub fn is_full_selection(&self) -> bool {
         &self.selected_tree.id() == self.commit.tree_id()
     }
 
@@ -1046,7 +1043,7 @@ impl CommitToSquash {
     ///
     /// Both `is_full_selection()` and `is_empty_selection()`
     /// can be true if the commit is itself empty.
-    fn is_empty_selection(&self) -> bool {
+    pub fn is_empty_selection(&self) -> bool {
         self.selected_tree.id() == self.parent_tree.id()
     }
 }
@@ -1065,12 +1062,12 @@ pub struct SquashedCommit<'repo> {
 /// finishing the commit.
 pub fn squash_commits<'repo>(
     repo: &'repo mut MutableRepo,
-    sources: &[CommitToSquash],
+    sources: &[CommitWithSelection],
     destination: &Commit,
     keep_emptied: bool,
 ) -> BackendResult<Option<SquashedCommit<'repo>>> {
     struct SourceCommit<'a> {
-        commit: &'a CommitToSquash,
+        commit: &'a CommitWithSelection,
         abandon: bool,
     }
     let mut source_commits = vec![];
