@@ -18,7 +18,7 @@ use std::collections::HashMap;
 use std::fs;
 use std::fs::File;
 use std::io;
-use std::io::Write;
+use std::io::Write as _;
 use std::path::Path;
 use std::path::PathBuf;
 use std::sync::Arc;
@@ -30,12 +30,12 @@ use crate::backend::MergedTreeId;
 use crate::commit::Commit;
 use crate::file_util::IoResultExt as _;
 use crate::file_util::PathError;
-use crate::local_backend::LocalBackend;
 use crate::local_working_copy::LocalWorkingCopy;
 use crate::local_working_copy::LocalWorkingCopyFactory;
 use crate::op_heads_store::OpHeadsStoreError;
 use crate::op_store::OperationId;
-use crate::op_store::WorkspaceId;
+use crate::ref_name::WorkspaceName;
+use crate::ref_name::WorkspaceNameBuf;
 use crate::repo::read_store_type;
 use crate::repo::BackendInitializer;
 use crate::repo::CheckOutCommitError;
@@ -43,7 +43,7 @@ use crate::repo::IndexStoreInitializer;
 use crate::repo::OpHeadsStoreInitializer;
 use crate::repo::OpStoreInitializer;
 use crate::repo::ReadonlyRepo;
-use crate::repo::Repo;
+use crate::repo::Repo as _;
 use crate::repo::RepoInitError;
 use crate::repo::RepoLoader;
 use crate::repo::StoreFactories;
@@ -52,6 +52,7 @@ use crate::repo::SubmoduleStoreInitializer;
 use crate::settings::UserSettings;
 use crate::signing::SignInitError;
 use crate::signing::Signer;
+use crate::simple_backend::SimpleBackend;
 use crate::store::Store;
 use crate::working_copy::CheckoutError;
 use crate::working_copy::CheckoutOptions;
@@ -128,22 +129,22 @@ fn init_working_copy(
     workspace_root: &Path,
     jj_dir: &Path,
     working_copy_factory: &dyn WorkingCopyFactory,
-    workspace_id: WorkspaceId,
+    workspace_name: WorkspaceNameBuf,
 ) -> Result<(Box<dyn WorkingCopy>, Arc<ReadonlyRepo>), WorkspaceInitError> {
     let working_copy_state_path = jj_dir.join("working_copy");
     std::fs::create_dir(&working_copy_state_path).context(&working_copy_state_path)?;
 
     let mut tx = repo.start_transaction();
     tx.repo_mut()
-        .check_out(workspace_id.clone(), &repo.store().root_commit())?;
-    let repo = tx.commit(format!("add workspace '{}'", workspace_id.as_str()))?;
+        .check_out(workspace_name.clone(), &repo.store().root_commit())?;
+    let repo = tx.commit(format!("add workspace '{}'", workspace_name.as_symbol()))?;
 
     let working_copy = working_copy_factory.init_working_copy(
         repo.store().clone(),
         workspace_root.to_path_buf(),
         working_copy_state_path.clone(),
         repo.op_id().clone(),
-        workspace_id,
+        workspace_name,
     )?;
     let working_copy_type_path = working_copy_state_path.join("type");
     fs::write(&working_copy_type_path, working_copy.name()).context(&working_copy_type_path)?;
@@ -180,12 +181,12 @@ impl Workspace {
         }
     }
 
-    pub fn init_local(
+    pub fn init_simple(
         user_settings: &UserSettings,
         workspace_root: &Path,
     ) -> Result<(Self, Arc<ReadonlyRepo>), WorkspaceInitError> {
         let backend_initializer: &BackendInitializer =
-            &|_settings, store_path| Ok(Box::new(LocalBackend::init(store_path)));
+            &|_settings, store_path| Ok(Box::new(SimpleBackend::init(store_path)));
         let signer = Signer::from_settings(user_settings)?;
         Self::init_with_backend(user_settings, workspace_root, backend_initializer, signer)
     }
@@ -276,7 +277,7 @@ impl Workspace {
         Self::init_with_backend(user_settings, workspace_root, &backend_initializer, signer)
     }
 
-    #[allow(clippy::too_many_arguments)]
+    #[expect(clippy::too_many_arguments)]
     pub fn init_with_factories(
         user_settings: &UserSettings,
         workspace_root: &Path,
@@ -287,7 +288,7 @@ impl Workspace {
         index_store_initializer: &IndexStoreInitializer,
         submodule_store_initializer: &SubmoduleStoreInitializer,
         working_copy_factory: &dyn WorkingCopyFactory,
-        workspace_id: WorkspaceId,
+        workspace_name: WorkspaceNameBuf,
     ) -> Result<(Self, Arc<ReadonlyRepo>), WorkspaceInitError> {
         let jj_dir = create_jj_dir(workspace_root)?;
         (|| {
@@ -313,7 +314,7 @@ impl Workspace {
                 workspace_root,
                 &jj_dir,
                 working_copy_factory,
-                workspace_id,
+                workspace_name,
             )?;
             let repo_loader = repo.loader().clone();
             let workspace = Workspace::new(workspace_root, repo_dir, working_copy, repo_loader)?;
@@ -340,7 +341,7 @@ impl Workspace {
             ReadonlyRepo::default_index_store_initializer(),
             ReadonlyRepo::default_submodule_store_initializer(),
             &*default_working_copy_factory(),
-            WorkspaceId::default(),
+            WorkspaceName::DEFAULT.to_owned(),
         )
     }
 
@@ -349,7 +350,7 @@ impl Workspace {
         repo_path: &Path,
         repo: &Arc<ReadonlyRepo>,
         working_copy_factory: &dyn WorkingCopyFactory,
-        workspace_id: WorkspaceId,
+        workspace_name: WorkspaceNameBuf,
     ) -> Result<(Self, Arc<ReadonlyRepo>), WorkspaceInitError> {
         let jj_dir = create_jj_dir(workspace_root)?;
 
@@ -370,7 +371,7 @@ impl Workspace {
             workspace_root,
             &jj_dir,
             working_copy_factory,
-            workspace_id,
+            workspace_name,
         )?;
         let workspace = Workspace::new(
             workspace_root,
@@ -396,8 +397,8 @@ impl Workspace {
         &self.workspace_root
     }
 
-    pub fn workspace_id(&self) -> &WorkspaceId {
-        self.working_copy.workspace_id()
+    pub fn workspace_name(&self) -> &WorkspaceName {
+        self.working_copy.workspace_name()
     }
 
     pub fn repo_path(&self) -> &Path {
