@@ -13,7 +13,7 @@
 // limitations under the License.
 
 #[cfg(unix)]
-use std::os::unix::fs::PermissionsExt;
+use std::os::unix::fs::PermissionsExt as _;
 #[cfg(unix)]
 use std::os::unix::net::UnixListener;
 use std::path::Path;
@@ -22,7 +22,7 @@ use std::sync::Arc;
 
 use assert_matches::assert_matches;
 use indoc::indoc;
-use itertools::Itertools;
+use itertools::Itertools as _;
 use jj_lib::backend::MergedTreeId;
 use jj_lib::backend::TreeId;
 use jj_lib::backend::TreeValue;
@@ -36,9 +36,9 @@ use jj_lib::merge::MergedTreeValue;
 use jj_lib::merged_tree::MergedTree;
 use jj_lib::merged_tree::MergedTreeBuilder;
 use jj_lib::op_store::OperationId;
-use jj_lib::op_store::WorkspaceId;
+use jj_lib::ref_name::WorkspaceName;
 use jj_lib::repo::ReadonlyRepo;
-use jj_lib::repo::Repo;
+use jj_lib::repo::Repo as _;
 use jj_lib::repo_path::RepoPath;
 use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::repo_path::RepoPathComponent;
@@ -48,11 +48,11 @@ use jj_lib::working_copy::CheckoutOptions;
 use jj_lib::working_copy::CheckoutStats;
 use jj_lib::working_copy::SnapshotOptions;
 use jj_lib::working_copy::UntrackedReason;
-use jj_lib::working_copy::WorkingCopy;
+use jj_lib::working_copy::WorkingCopy as _;
 use jj_lib::workspace::default_working_copy_factories;
 use jj_lib::workspace::LockedWorkspace;
 use jj_lib::workspace::Workspace;
-use pollster::FutureExt;
+use pollster::FutureExt as _;
 use test_case::test_case;
 use testutils::commit_with_tree;
 use testutils::create_tree;
@@ -115,14 +115,14 @@ fn test_root() {
     let repo = &test_workspace.repo;
     let wc_commit_id = repo
         .view()
-        .get_wc_commit_id(&WorkspaceId::default())
+        .get_wc_commit_id(WorkspaceName::DEFAULT)
         .unwrap();
     let wc_commit = repo.store().get_commit(wc_commit_id).unwrap();
     assert_eq!(new_tree.id(), *wc_commit.tree_id());
     assert_eq!(new_tree.id(), repo.store().empty_merged_tree_id());
 }
 
-#[test_case(TestRepoBackend::Local ; "local backend")]
+#[test_case(TestRepoBackend::Simple ; "simple backend")]
 #[test_case(TestRepoBackend::Git ; "git backend")]
 fn test_checkout_file_transitions(backend: TestRepoBackend) {
     // Tests switching between commits where a certain path is of one type in one
@@ -887,7 +887,7 @@ fn test_materialize_snapshot_conflicted_files() {
     assert_eq!(file2_value.num_sides(), 3);
     insta::assert_snapshot!(
         std::fs::read_to_string(file1_path.to_fs_path_unchecked(&workspace_root)).ok().unwrap(),
-        @r###"
+        @r"
     <<<<<<< Conflict 1 of 1
     %%%%%%% Changes from base to side #1
     -b
@@ -895,10 +895,10 @@ fn test_materialize_snapshot_conflicted_files() {
     +++++++ Contents of side #2
     c
     >>>>>>> Conflict 1 of 1 ends
-    "###);
+    ");
     insta::assert_snapshot!(
         std::fs::read_to_string(file2_path.to_fs_path_unchecked(&workspace_root)).ok().unwrap(),
-        @r###"
+        @r"
     <<<<<<< Conflict 1 of 1
     %%%%%%% Changes from base to side #1
     -2
@@ -906,7 +906,7 @@ fn test_materialize_snapshot_conflicted_files() {
     +++++++ Contents of side #2
     4
     >>>>>>> Conflict 1 of 1 ends
-    "###);
+    ");
 
     // Editing a conflicted file should correctly propagate updates to each of
     // the conflicting trees.
@@ -2102,10 +2102,10 @@ fn test_fsmonitor() {
     {
         let mut locked_ws = ws.start_working_copy_mutation().unwrap();
         let tree_id = snapshot(&mut locked_ws, &[foo_path]);
-        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r###"
+        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r#"
         tree d5e38c0a1b0ee5de47c5
           file "foo" (e99c2057c15160add351): "foo\n"
-        "###);
+        "#);
     }
 
     {
@@ -2114,12 +2114,12 @@ fn test_fsmonitor() {
             &mut locked_ws,
             &[foo_path, bar_path, nested_path, ignored_path],
         );
-        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r###"
+        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r#"
         tree f408c8d080414f8e90e1
           file "bar" (94cc973e7e1aefb7eff6): "bar\n"
           file "foo" (e99c2057c15160add351): "foo\n"
           file "path/to/nested" (6209060941cd770c8d46): "nested\n"
-        "###);
+        "#);
         locked_ws.finish(repo.op_id().clone()).unwrap();
     }
 
@@ -2128,23 +2128,23 @@ fn test_fsmonitor() {
         testutils::write_working_copy_file(&workspace_root, bar_path, "updated bar\n");
         let mut locked_ws = ws.start_working_copy_mutation().unwrap();
         let tree_id = snapshot(&mut locked_ws, &[foo_path]);
-        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r###"
+        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r#"
         tree e994a93c46f41dc91704
           file "bar" (94cc973e7e1aefb7eff6): "bar\n"
           file "foo" (e0fbd106147cc04ccd05): "updated foo\n"
           file "path/to/nested" (6209060941cd770c8d46): "nested\n"
-        "###);
+        "#);
     }
 
     {
         std::fs::remove_file(foo_path.to_fs_path_unchecked(&workspace_root)).unwrap();
         let mut locked_ws = ws.start_working_copy_mutation().unwrap();
         let tree_id = snapshot(&mut locked_ws, &[foo_path]);
-        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r###"
+        insta::assert_snapshot!(testutils::dump_tree(repo.store(), &tree_id), @r#"
         tree 1df764981d4d74a4ecfa
           file "bar" (94cc973e7e1aefb7eff6): "bar\n"
           file "path/to/nested" (6209060941cd770c8d46): "nested\n"
-        "###);
+        "#);
         locked_ws.finish(repo.op_id().clone()).unwrap();
     }
 }
