@@ -13,15 +13,15 @@
 // limitations under the License.
 
 use futures::StreamExt as _;
-use itertools::Itertools;
+use itertools::Itertools as _;
 use jj_lib::local_working_copy::LocalWorkingCopy;
 use jj_lib::matchers::EverythingMatcher;
-use jj_lib::repo::Repo;
+use jj_lib::repo::Repo as _;
 use jj_lib::repo_path::RepoPath;
 use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::working_copy::CheckoutOptions;
 use jj_lib::working_copy::CheckoutStats;
-use jj_lib::working_copy::WorkingCopy;
+use jj_lib::working_copy::WorkingCopy as _;
 use pollster::FutureExt as _;
 use testutils::commit_with_tree;
 use testutils::create_tree;
