@@ -21,13 +21,17 @@ use jj_lib::merged_tree::MergedTree;
 use jj_lib::op_store::RefTarget;
 use jj_lib::op_store::RemoteRef;
 use jj_lib::op_store::RemoteRefState;
-use jj_lib::op_store::WorkspaceId;
-use jj_lib::refs::RemoteRefSymbol;
-use jj_lib::repo::Repo;
+use jj_lib::ref_name::RefName;
+use jj_lib::ref_name::RemoteName;
+use jj_lib::ref_name::RemoteRefSymbol;
+use jj_lib::ref_name::WorkspaceName;
+use jj_lib::ref_name::WorkspaceNameBuf;
+use jj_lib::repo::Repo as _;
 use jj_lib::repo_path::RepoPath;
 use jj_lib::rewrite::rebase_commit_with_options;
 use jj_lib::rewrite::restore_tree;
 use jj_lib::rewrite::CommitRewriter;
+use jj_lib::rewrite::CommitWithSelection;
 use jj_lib::rewrite::EmptyBehaviour;
 use jj_lib::rewrite::RebaseOptions;
 use jj_lib::rewrite::RewriteRefsOptions;
@@ -43,8 +47,15 @@ use testutils::write_random_commit;
 use testutils::CommitGraphBuilder;
 use testutils::TestRepo;
 
-fn remote_symbol<'a>(name: &'a str, remote: &'a str) -> RemoteRefSymbol<'a> {
-    RemoteRefSymbol { name, remote }
+fn remote_symbol<'a, N, M>(name: &'a N, remote: &'a M) -> RemoteRefSymbol<'a>
+where
+    N: AsRef<RefName> + ?Sized,
+    M: AsRef<RemoteName> + ?Sized,
+{
+    RemoteRefSymbol {
+        name: name.as_ref(),
+        remote: remote.as_ref(),
+    }
 }
 
 #[test]
@@ -930,14 +941,14 @@ fn test_rebase_descendants_basic_bookmark_update() {
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     tx.repo_mut()
-        .set_local_bookmark_target("main", RefTarget::normal(commit_b.id().clone()));
+        .set_local_bookmark_target("main".as_ref(), RefTarget::normal(commit_b.id().clone()));
     let repo = tx.commit("test").unwrap();
 
     let mut tx = repo.start_transaction();
     let commit_b2 = tx.repo_mut().rewrite_commit(&commit_b).write().unwrap();
     tx.repo_mut().rebase_descendants().unwrap();
     assert_eq!(
-        tx.repo().get_local_bookmark("main"),
+        tx.repo().get_local_bookmark("main".as_ref()),
         RefTarget::normal(commit_b2.id().clone())
     );
 
@@ -965,7 +976,7 @@ fn test_rebase_descendants_bookmark_move_two_steps() {
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     let commit_c = graph_builder.commit_with_parents(&[&commit_b]);
     tx.repo_mut()
-        .set_local_bookmark_target("main", RefTarget::normal(commit_c.id().clone()));
+        .set_local_bookmark_target("main".as_ref(), RefTarget::normal(commit_c.id().clone()));
     let repo = tx.commit("test").unwrap();
 
     let mut tx = repo.start_transaction();
@@ -989,7 +1000,7 @@ fn test_rebase_descendants_bookmark_move_two_steps() {
     assert_ne!(commit_c3.id(), commit_c2.id());
     assert_eq!(commit_c3.parent_ids(), vec![commit_b2.id().clone()]);
     assert_eq!(
-        tx.repo().get_local_bookmark("main"),
+        tx.repo().get_local_bookmark("main".as_ref()),
         RefTarget::normal(commit_c3.id().clone())
     );
 }
@@ -1013,21 +1024,21 @@ fn test_rebase_descendants_basic_bookmark_update_with_non_local_bookmark() {
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     let commit_b_remote_ref = RemoteRef {
         target: RefTarget::normal(commit_b.id().clone()),
-        state: RemoteRefState::Tracking,
+        state: RemoteRefState::Tracked,
     };
     tx.repo_mut()
-        .set_local_bookmark_target("main", RefTarget::normal(commit_b.id().clone()));
+        .set_local_bookmark_target("main".as_ref(), RefTarget::normal(commit_b.id().clone()));
     tx.repo_mut()
         .set_remote_bookmark(remote_symbol("main", "origin"), commit_b_remote_ref.clone());
     tx.repo_mut()
-        .set_tag_target("v1", RefTarget::normal(commit_b.id().clone()));
+        .set_tag_target("v1".as_ref(), RefTarget::normal(commit_b.id().clone()));
     let repo = tx.commit("test").unwrap();
 
     let mut tx = repo.start_transaction();
     let commit_b2 = tx.repo_mut().rewrite_commit(&commit_b).write().unwrap();
     tx.repo_mut().rebase_descendants().unwrap();
     assert_eq!(
-        tx.repo().get_local_bookmark("main"),
+        tx.repo().get_local_bookmark("main".as_ref()),
         RefTarget::normal(commit_b2.id().clone())
     );
     // The remote bookmark and tag should not get updated
@@ -1037,7 +1048,7 @@ fn test_rebase_descendants_basic_bookmark_update_with_non_local_bookmark() {
         commit_b_remote_ref
     );
     assert_eq!(
-        tx.repo().get_tag("v1"),
+        tx.repo().get_tag("v1".as_ref()),
         RefTarget::normal(commit_b.id().clone())
     );
 
@@ -1067,14 +1078,14 @@ fn test_rebase_descendants_update_bookmark_after_abandon(delete_abandoned_bookma
     let commit_c = graph_builder.commit_with_parents(&[&commit_b]);
     let commit_b_remote_ref = RemoteRef {
         target: RefTarget::normal(commit_b.id().clone()),
-        state: RemoteRefState::Tracking,
+        state: RemoteRefState::Tracked,
     };
     tx.repo_mut()
-        .set_local_bookmark_target("main", RefTarget::normal(commit_b.id().clone()));
+        .set_local_bookmark_target("main".as_ref(), RefTarget::normal(commit_b.id().clone()));
     tx.repo_mut()
         .set_remote_bookmark(remote_symbol("main", "origin"), commit_b_remote_ref.clone());
     tx.repo_mut()
-        .set_local_bookmark_target("other", RefTarget::normal(commit_c.id().clone()));
+        .set_local_bookmark_target("other".as_ref(), RefTarget::normal(commit_c.id().clone()));
     let repo = tx.commit("test").unwrap();
 
     let mut tx = repo.start_transaction();
@@ -1087,7 +1098,7 @@ fn test_rebase_descendants_update_bookmark_after_abandon(delete_abandoned_bookma
     };
     let rebase_map = rebase_descendants_with_options_return_map(tx.repo_mut(), &options);
     assert_eq!(
-        tx.repo().get_local_bookmark("main"),
+        tx.repo().get_local_bookmark("main".as_ref()),
         if delete_abandoned_bookmarks {
             RefTarget::absent()
         } else {
@@ -1101,7 +1112,7 @@ fn test_rebase_descendants_update_bookmark_after_abandon(delete_abandoned_bookma
         RefTarget::normal(commit_b.id().clone())
     );
     assert_eq!(
-        tx.repo().get_local_bookmark("other"),
+        tx.repo().get_local_bookmark("other".as_ref()),
         RefTarget::normal(rebase_map[commit_c.id()].clone())
     );
 
@@ -1133,9 +1144,9 @@ fn test_rebase_descendants_update_bookmarks_after_divergent_rewrite() {
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     let commit_c = graph_builder.commit_with_parents(&[&commit_b]);
     tx.repo_mut()
-        .set_local_bookmark_target("main", RefTarget::normal(commit_b.id().clone()));
+        .set_local_bookmark_target("main".as_ref(), RefTarget::normal(commit_b.id().clone()));
     tx.repo_mut()
-        .set_local_bookmark_target("other", RefTarget::normal(commit_c.id().clone()));
+        .set_local_bookmark_target("other".as_ref(), RefTarget::normal(commit_c.id().clone()));
     let repo = tx.commit("test").unwrap();
 
     let mut tx = repo.start_transaction();
@@ -1175,7 +1186,7 @@ fn test_rebase_descendants_update_bookmarks_after_divergent_rewrite() {
     );
     tx.repo_mut().rebase_descendants().unwrap();
 
-    let main_target = tx.repo().get_local_bookmark("main");
+    let main_target = tx.repo().get_local_bookmark("main".as_ref());
     assert!(main_target.has_conflict());
     // If the bookmark were moved at each rewrite point, there would be separate
     // negative terms: { commit_b => 2, commit_b4 => 1 }. Since we flatten
@@ -1194,7 +1205,7 @@ fn test_rebase_descendants_update_bookmarks_after_divergent_rewrite() {
         },
     );
 
-    let other_target = tx.repo().get_local_bookmark("other");
+    let other_target = tx.repo().get_local_bookmark("other".as_ref());
     assert_eq!(other_target.as_normal(), Some(commit_c.id()));
 
     assert_eq!(
@@ -1223,7 +1234,7 @@ fn test_rebase_descendants_rewrite_updates_bookmark_conflict() {
     let commit_b = graph_builder.initial_commit();
     let commit_c = graph_builder.initial_commit();
     tx.repo_mut().set_local_bookmark_target(
-        "main",
+        "main".as_ref(),
         RefTarget::from_legacy_form(
             [commit_a.id().clone()],
             [commit_b.id().clone(), commit_c.id().clone()],
@@ -1258,7 +1269,7 @@ fn test_rebase_descendants_rewrite_updates_bookmark_conflict() {
     );
     tx.repo_mut().rebase_descendants().unwrap();
 
-    let target = tx.repo().get_local_bookmark("main");
+    let target = tx.repo().get_local_bookmark("main".as_ref());
     assert!(target.has_conflict());
     assert_eq!(
         target.removed_ids().counts(),
@@ -1303,7 +1314,7 @@ fn test_rebase_descendants_rewrite_resolves_bookmark_conflict() {
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     let commit_c = graph_builder.commit_with_parents(&[&commit_a]);
     tx.repo_mut().set_local_bookmark_target(
-        "main",
+        "main".as_ref(),
         RefTarget::from_legacy_form(
             [commit_a.id().clone()],
             [commit_b.id().clone(), commit_c.id().clone()],
@@ -1320,7 +1331,7 @@ fn test_rebase_descendants_rewrite_resolves_bookmark_conflict() {
         .unwrap();
     tx.repo_mut().rebase_descendants().unwrap();
     assert_eq!(
-        tx.repo().get_local_bookmark("main"),
+        tx.repo().get_local_bookmark("main".as_ref()),
         RefTarget::normal(commit_b2.id().clone())
     );
 
@@ -1351,7 +1362,7 @@ fn test_rebase_descendants_bookmark_delete_modify_abandon(delete_abandoned_bookm
     let commit_a = graph_builder.initial_commit();
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     tx.repo_mut().set_local_bookmark_target(
-        "main",
+        "main".as_ref(),
         RefTarget::from_legacy_form([commit_a.id().clone()], [commit_b.id().clone()]),
     );
     let repo = tx.commit("test").unwrap();
@@ -1365,7 +1376,10 @@ fn test_rebase_descendants_bookmark_delete_modify_abandon(delete_abandoned_bookm
         ..Default::default()
     };
     let _rebase_map = rebase_descendants_with_options_return_map(tx.repo_mut(), &options);
-    assert_eq!(tx.repo().get_local_bookmark("main"), RefTarget::absent());
+    assert_eq!(
+        tx.repo().get_local_bookmark("main".as_ref()),
+        RefTarget::absent()
+    );
 }
 
 #[test_case(false; "slide down abandoned")]
@@ -1388,7 +1402,7 @@ fn test_rebase_descendants_bookmark_move_forward_abandon(delete_abandoned_bookma
     let commit_b = graph_builder.commit_with_parents(&[&commit_a]);
     let commit_c = graph_builder.commit_with_parents(&[&commit_a]);
     tx.repo_mut().set_local_bookmark_target(
-        "main",
+        "main".as_ref(),
         RefTarget::from_merge(Merge::from_vec(vec![
             Some(commit_b.id().clone()),
             Some(commit_a.id().clone()),
@@ -1407,7 +1421,7 @@ fn test_rebase_descendants_bookmark_move_forward_abandon(delete_abandoned_bookma
     };
     let _rebase_map = rebase_descendants_with_options_return_map(tx.repo_mut(), &options);
     assert_eq!(
-        tx.repo().get_local_bookmark("main"),
+        tx.repo().get_local_bookmark("main".as_ref()),
         if delete_abandoned_bookmarks {
             RefTarget::from_merge(Merge::from_vec(vec![
                 None,
@@ -1440,7 +1454,7 @@ fn test_rebase_descendants_bookmark_move_sideways_abandon(delete_abandoned_bookm
     let commit_b = graph_builder.initial_commit();
     let commit_c = graph_builder.initial_commit();
     tx.repo_mut().set_local_bookmark_target(
-        "main",
+        "main".as_ref(),
         RefTarget::from_merge(Merge::from_vec(vec![
             Some(commit_b.id().clone()),
             Some(commit_a.id().clone()),
@@ -1459,7 +1473,7 @@ fn test_rebase_descendants_bookmark_move_sideways_abandon(delete_abandoned_bookm
     };
     let _rebase_map = rebase_descendants_with_options_return_map(tx.repo_mut(), &options);
     assert_eq!(
-        tx.repo().get_local_bookmark("main"),
+        tx.repo().get_local_bookmark("main".as_ref()),
         if delete_abandoned_bookmarks {
             RefTarget::from_merge(Merge::from_vec(vec![
                 None,
@@ -1493,17 +1507,17 @@ fn test_rebase_descendants_update_checkout() {
         .set_parents(vec![commit_a.id().clone()])
         .write()
         .unwrap();
-    let ws1_id = WorkspaceId::new("ws1".to_string());
-    let ws2_id = WorkspaceId::new("ws2".to_string());
-    let ws3_id = WorkspaceId::new("ws3".to_string());
+    let ws1_name = WorkspaceNameBuf::from("ws1");
+    let ws2_name = WorkspaceNameBuf::from("ws2");
+    let ws3_name = WorkspaceNameBuf::from("ws3");
     tx.repo_mut()
-        .set_wc_commit(ws1_id.clone(), commit_b.id().clone())
+        .set_wc_commit(ws1_name.clone(), commit_b.id().clone())
         .unwrap();
     tx.repo_mut()
-        .set_wc_commit(ws2_id.clone(), commit_b.id().clone())
+        .set_wc_commit(ws2_name.clone(), commit_b.id().clone())
         .unwrap();
     tx.repo_mut()
-        .set_wc_commit(ws3_id.clone(), commit_a.id().clone())
+        .set_wc_commit(ws3_name.clone(), commit_a.id().clone())
         .unwrap();
     let repo = tx.commit("test").unwrap();
 
@@ -1519,9 +1533,9 @@ fn test_rebase_descendants_update_checkout() {
 
     // Workspaces 1 and 2 had B checked out, so they get updated to C. Workspace 3
     // had A checked out, so it doesn't get updated.
-    assert_eq!(repo.view().get_wc_commit_id(&ws1_id), Some(commit_c.id()));
-    assert_eq!(repo.view().get_wc_commit_id(&ws2_id), Some(commit_c.id()));
-    assert_eq!(repo.view().get_wc_commit_id(&ws3_id), Some(commit_a.id()));
+    assert_eq!(repo.view().get_wc_commit_id(&ws1_name), Some(commit_c.id()));
+    assert_eq!(repo.view().get_wc_commit_id(&ws2_name), Some(commit_c.id()));
+    assert_eq!(repo.view().get_wc_commit_id(&ws3_name), Some(commit_a.id()));
 }
 
 #[test]
@@ -1541,17 +1555,17 @@ fn test_rebase_descendants_update_checkout_abandoned() {
         .set_parents(vec![commit_a.id().clone()])
         .write()
         .unwrap();
-    let ws1_id = WorkspaceId::new("ws1".to_string());
-    let ws2_id = WorkspaceId::new("ws2".to_string());
-    let ws3_id = WorkspaceId::new("ws3".to_string());
+    let ws1_name = WorkspaceNameBuf::from("ws1");
+    let ws2_name = WorkspaceNameBuf::from("ws2");
+    let ws3_name = WorkspaceNameBuf::from("ws3");
     tx.repo_mut()
-        .set_wc_commit(ws1_id.clone(), commit_b.id().clone())
+        .set_wc_commit(ws1_name.clone(), commit_b.id().clone())
         .unwrap();
     tx.repo_mut()
-        .set_wc_commit(ws2_id.clone(), commit_b.id().clone())
+        .set_wc_commit(ws2_name.clone(), commit_b.id().clone())
         .unwrap();
     tx.repo_mut()
-        .set_wc_commit(ws3_id.clone(), commit_a.id().clone())
+        .set_wc_commit(ws3_name.clone(), commit_a.id().clone())
         .unwrap();
     let repo = tx.commit("test").unwrap();
 
@@ -1563,15 +1577,15 @@ fn test_rebase_descendants_update_checkout_abandoned() {
     // Workspaces 1 and 2 had B checked out, so they get updated to the same new
     // commit on top of C. Workspace 3 had A checked out, so it doesn't get updated.
     assert_eq!(
-        repo.view().get_wc_commit_id(&ws1_id),
-        repo.view().get_wc_commit_id(&ws2_id)
+        repo.view().get_wc_commit_id(&ws1_name),
+        repo.view().get_wc_commit_id(&ws2_name)
     );
     let checkout = repo
         .store()
-        .get_commit(repo.view().get_wc_commit_id(&ws1_id).unwrap())
+        .get_commit(repo.view().get_wc_commit_id(&ws1_name).unwrap())
         .unwrap();
     assert_eq!(checkout.parent_ids(), vec![commit_a.id().clone()]);
-    assert_eq!(repo.view().get_wc_commit_id(&ws3_id), Some(commit_a.id()));
+    assert_eq!(repo.view().get_wc_commit_id(&ws3_name), Some(commit_a.id()));
 }
 
 #[test]
@@ -1601,9 +1615,9 @@ fn test_rebase_descendants_update_checkout_abandoned_merge() {
         .set_parents(vec![commit_b.id().clone(), commit_c.id().clone()])
         .write()
         .unwrap();
-    let workspace_id = WorkspaceId::default();
+    let ws_name = WorkspaceName::DEFAULT.to_owned();
     tx.repo_mut()
-        .set_wc_commit(workspace_id.clone(), commit_d.id().clone())
+        .set_wc_commit(ws_name.clone(), commit_d.id().clone())
         .unwrap();
     let repo = tx.commit("test").unwrap();
 
@@ -1612,7 +1626,7 @@ fn test_rebase_descendants_update_checkout_abandoned_merge() {
     tx.repo_mut().rebase_descendants().unwrap();
     let repo = tx.commit("test").unwrap();
 
-    let new_checkout_id = repo.view().get_wc_commit_id(&workspace_id).unwrap();
+    let new_checkout_id = repo.view().get_wc_commit_id(&ws_name).unwrap();
     let checkout = repo.store().get_commit(new_checkout_id).unwrap();
     assert_eq!(
         checkout.parent_ids(),
@@ -1816,7 +1830,7 @@ fn test_rebase_abandoning_empty() {
         .write()
         .unwrap();
 
-    let workspace = WorkspaceId::new("ws".to_string());
+    let workspace = WorkspaceNameBuf::from("ws");
     tx.repo_mut()
         .set_wc_commit(workspace.clone(), commit_e.id().clone())
         .unwrap();
@@ -1852,3 +1866,29 @@ fn test_rebase_abandoning_empty() {
         hashset! {new_commit_f.id().clone(), new_wc_commit_id.clone()}
     );
 }
+
+#[test]
+fn test_commit_with_selection() {
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction();
+    let root_tree = repo.store().root_commit().tree().unwrap();
+    let commit = create_random_commit(tx.repo_mut()).write().unwrap();
+    let commit_tree = commit.tree().unwrap();
+    let empty_selection = CommitWithSelection {
+        commit: commit.clone(),
+        selected_tree: root_tree.clone(),
+        parent_tree: root_tree.clone(),
+    };
+    assert!(empty_selection.is_empty_selection());
+    assert!(!empty_selection.is_full_selection());
+
+    let full_selection = CommitWithSelection {
+        commit,
+        selected_tree: commit_tree,
+        parent_tree: root_tree,
+    };
+    assert!(!full_selection.is_empty_selection());
+    assert!(full_selection.is_full_selection());
+}
